/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013, 2014. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
	dojo.provide("com.ibm.rational.datacollection.web.ui.internal.common.DataCollectionClient"); //$NON-NLS-1$
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	dojo.require("dojo.cookie"); //$NON-NLS-1$
	dojo.require("dojox.xml.parser"); //$NON-NLS-1$
	dojo.require("jazz.client.xhr"); //$NON-NLS-1$
	dojo.require("net.jazz.ajax.BootstrapProperties"); //$NON-NLS-1$
	dojo.require("com.ibm.rational.datacollection.web.ui.internal.common.DataCollectionConstants"); //$NON-NLS-1$
	var DataCollectionConstants = com.ibm.rational.datacollection.web.ui.internal.common.DataCollectionConstants;
	var DataCollectionClient = com.ibm.rational.datacollection.web.ui.internal.common.DataCollectionClient;
	var JTS_URI_ROOT = getJtsRoot();
	var lastModified = null;
	
	function getJtsRoot() {
		var discoverySvcUrl = net.jazz.ajax.BootstrapProperties.DISCOVERYSERVICE_URL;
		var index = discoverySvcUrl.lastIndexOf('/'); //$NON-NLS-1$
		if(index == discoverySvcUrl.length - 1) {
			index = discoverySvcUrl.lastIndexOf('/', index); //$NON-NLS-1$
		}
		return discoverySvcUrl.substring(0, index);
	}
	
	DataCollectionClient.getLastModified = function() {
		return lastModified;
	};

	DataCollectionClient.getBaseResourceGroupCollection = function(args) {
		var url = net.jazz.ajax.contextRoot() + "/datacollection/resource/baseResourceGroup"; //$NON-NLS-1$
		var headers = {
			"Accept-Charset": "UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
			"Accept": "application/json" //$NON-NLS-1$ //$NON-NLS-2$
		};
		if (lastModified) {
			headers[DataCollectionConstants.HEADER_IF_MODIFIED_SINCE] = lastModified;
		}
		
		jazz.client.xhrGet({
			preventCache: true,
			url: url,
			headers: headers,
			handleAs: "json", //$NON-NLS-1$
			load: args.load,
			error: args.error
		});
	};
	
	DataCollectionClient.getResourceGroupCollection = function(args) {
		var url = net.jazz.ajax.contextRoot() + "/datacollection/resource/resourceGroup"; //$NON-NLS-1$
		var headers = {
			"Accept-Charset": "UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
			"Accept": "application/json" //$NON-NLS-1$ //$NON-NLS-2$
		};
		if (lastModified) {
			headers[DataCollectionConstants.HEADER_IF_MODIFIED_SINCE] = lastModified;
		}
		
		jazz.client.xhrGet({
			preventCache: true,
			url: url,
			headers: headers,
			handleAs: "json", //$NON-NLS-1$
			load: args.load,
			error: args.error
		});
	};
	
	DataCollectionClient.getResourceGroup = function(args) {
		var headers = {
			"Accept-Charset": "UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
			"Accept": "application/json" //$NON-NLS-1$ //$NON-NLS-2$
		};
		if (lastModified) {
			headers[DataCollectionConstants.HEADER_IF_MODIFIED_SINCE] = lastModified;
		}
		
		jazz.client.xhrGet({
			preventCache: true,
			url: args.url,
			headers: headers,
			handleAs: "json", //$NON-NLS-1$
			error: args.error,
			load: args.load
		});
	};
	
	DataCollectionClient._demarshallDataQuery = function(srcRows) {
		var targetRows= [];
		for (var i=0; i<srcRows.length; i++) {
			var srcCols= srcRows[i].dataArgs;
			var targetCols= [];
			for (var j=0; j<srcCols.length; j++) {
				targetCols.push(srcCols[j].value);
			}
			targetRows.push(targetCols);
		}
		return targetRows;
	} 
	
	DataCollectionClient._demarshallResponse = function(evaldObj) {
		var returnValue= evaldObj["soapenv:Body"].response.returnValue; //$NON-NLS-1$
		var obj= null;
		if (returnValue.values) { // array of EMF objects
			obj = returnValue.values;
		} else if (returnValue.value) { // single EMF object
			obj = returnValue.value;
		} else if (returnValue.dataArgs) { // data query
			obj = DataCollectionClient._demarshallDataQuery(returnValue.dataArgs);
		}
		return obj;
	};
	
	
	
	DataCollectionClient.testDWConnection = function(args){
		
		var resourceId = args.resourceId;
		var url = net.jazz.ajax.contextRoot() + "/service/com.ibm.team.repository.service.internal.IServerConfigurationRestService/configurationValidation"; //$NON-NLS-1$
		var headers = {
			"Accept-Charset": "UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
			"Content-Type": "application/x-www-form-urlencoded; charset=utf-8", //$NON-NLS-1$ //$NON-NLS-2$
			"Accept": "text/json" //$NON-NLS-1$ //$NON-NLS-2$
		};
		
		var validationString = [{ 
		      "name":"db.vendor", //$NON-NLS-1$ //$NON-NLS-2$
		      "value": args.vendor,  //$NON-NLS-1$
		      "serviceImplementationName":"com.ibm.team.datawarehouse.service.internal.RemoteDatabaseService" //$NON-NLS-1$ //$NON-NLS-2$
		    },{
		      "name":"db.jdbc.password", //$NON-NLS-1$ //$NON-NLS-2$
		      "value": args.password, //$NON-NLS-1$
		      "serviceImplementationName":"com.ibm.team.datawarehouse.service.internal.RemoteDatabaseService" //$NON-NLS-1$ //$NON-NLS-2$
		    },{
		      "name":"db.jdbc.location","value": args.location, //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		      "serviceImplementationName":"com.ibm.team.datawarehouse.service.internal.RemoteDatabaseService" //$NON-NLS-1$ //$NON-NLS-2$
		    }
		];
		
		var changeArgs = {
			updates: dojo.toJson(validationString),
			fast: false
		}; 
		
		var postData = dojo.objectToQuery(changeArgs);
		
		jazz.client.xhrPost({
			url: url,
			headers: headers,
			handleAs: "json", //$NON-NLS-1$
			postData: postData,
			load: dojo.hitch(this, function(data, ioArgs) {
				lastModified = ioArgs.xhr.getResponseHeader(DataCollectionConstants.HEADER_LASTMODIFIED);
				var responseObj = DataCollectionClient._demarshallResponse(data);
				args.load(responseObj, ioArgs);
			}),
			error: args.error
		});
		
	};
	
	DataCollectionClient.testResourceGroupConnection = function(args) {
		var resourceId = args.resourceId;
		var url = net.jazz.ajax.contextRoot() + "/datacollection/resource/resourceGroup/"; //$NON-NLS-1$
		var headers = {
			"Accept-Charset": "UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
			"Content-Type": "application/json;charset=UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
			"Accept": "application/json", //$NON-NLS-1$ //$NON-NLS-2$
			"Name": resourceId, //$NON-NLS-1$
			"TestRSRC": true //$NON-NLS-1$
		};
		var postData = dojo.toJson(args.data);
		jazz.client.xhrPost({
			url: url,
			headers: headers,
			handleAs: "json", //$NON-NLS-1$
			postData: postData,
			load: dojo.hitch(this, function(data, ioArgs) {
				lastModified = ioArgs.xhr.getResponseHeader(DataCollectionConstants.HEADER_LASTMODIFIED);
				args.load(data, ioArgs);
			}),
			error: args.error
		});
	};
	
	
	DataCollectionClient.createResourceGroup = function(args) {
		var resourceId = args.resourceId;
		var url = net.jazz.ajax.contextRoot() + "/datacollection/resource/resourceGroup/"; //$NON-NLS-1$
		var headers = {
			"Accept-Charset": "UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
			"Content-Type": "application/json;charset=UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
			"Accept": "application/json", //$NON-NLS-1$ //$NON-NLS-2$
			"Name": resourceId //$NON-NLS-1$
		};
		var postData = dojo.toJson(args.data);
		jazz.client.xhrPost({
			url: url,
			headers: headers,
			handleAs: "json", //$NON-NLS-1$
			postData: postData,
			load: dojo.hitch(this, function(data, ioArgs) {
				lastModified = ioArgs.xhr.getResponseHeader(DataCollectionConstants.HEADER_LASTMODIFIED);
				args.load(data, ioArgs);
			}),
			error: args.error
		});
	};
	
	DataCollectionClient.deleteResourceGroup = function(args) {
		var resourceId = args.resourceId;
		var url = net.jazz.ajax.contextRoot() + "/datacollection/resource/resourceGroup/" + resourceId; //$NON-NLS-1$
		jazz.client.xhrDelete({
			url: url,
			handleAs: "json", //$NON-NLS-1$
			load: dojo.hitch(this, function(data, ioArgs) {
				lastModified = ioArgs.xhr.getResponseHeader(DataCollectionConstants.HEADER_LASTMODIFIED);
				args.load(data, ioArgs);
			}),
			error: args.error
		});
	};
	
	DataCollectionClient.updateResourceGroup = function(args) {
		var resourceId = args.resourceId;
		var url = net.jazz.ajax.contextRoot() + "/datacollection/resource/resourceGroup/" + resourceId; //$NON-NLS-1$
		var headers = {
			"Accept-Charset": "UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
			"Content-Type": "application/json;charset=UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
			"Accept": "application/json" //$NON-NLS-1$ //$NON-NLS-2$
		};
		if (args.lastModified) {
			headers[DataCollectionConstants.HEADER_IF_MODIFIED_SINCE] = args.lastModified;
		}
		if (args.etag) {
			headers[DataCollectionConstants.HEADER_IF_MATCH] = args.etag;
		}
		var putData = dojo.toJson(args.data);
	
		jazz.client.xhrPut({
			url: url,
			headers: headers,
			handleAs: "json", //$NON-NLS-1$
			putData: putData,
			load: dojo.hitch(this, function(data, ioArgs) {
				lastModified = ioArgs.xhr.getResponseHeader(DataCollectionConstants.HEADER_LASTMODIFIED);
				args.load(data, ioArgs);
			}),
			error: args.error
		});
	};
	
	DataCollectionClient.executeTask = function(args) {
		var url = net.jazz.ajax.contextRoot() + "/datacollection/resource/task"; //$NON-NLS-1$
		var headers = {
			"Accept-Charset": "UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
			"Content-Type": "text/plain;charset=UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
			"Accept": "application/json" //$NON-NLS-1$ //$NON-NLS-2$
		};
		var postData = "type=executeTask&mapgroup="; //$NON-NLS-1$
		if (args.mapgroupId) {
			postData += args.mapgroupId;
		} else {
			for (var i = 0; i < args.mapgroups.length; i++) {
				if (i === args.mapgroups.length-1) {
					postData += args.mapgroups[i];
				} else {
					postData += args.mapgroups[i] + "|"; //$NON-NLS-1$
				}
			}
		}
		jazz.client.xhrPost({
			url: url,
			headers: headers,
			handleAs: "json", //$NON-NLS-1$
			postData: postData,
			load: args.load,
			error: args.error
		});
	};
	
	DataCollectionClient.stopTask = function(args) {
		var url = net.jazz.ajax.contextRoot() + "/datacollection/resource/task"; //$NON-NLS-1$
		var headers = {
			"Accept-Charset": "UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
			"Content-Type": "text/plain;charset=UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
			"Accept": "application/json" //$NON-NLS-1$ //$NON-NLS-2$
		};
		var postData = "type=stopTask&mapgroup="; //$NON-NLS-1$
		if (args.mapgroupId) {
			postData += args.mapgroupId;
		} else {
			for (var i = 0; i < args.mapgroups.length; i++) {
				if (i === args.mapgroups.length-1) {
					postData += args.mapgroups[i];
				} else {
					postData += args.mapgroups[i] + "|"; //$NON-NLS-1$
				}
			}
		}
		jazz.client.xhrPost({
			url: url,
			headers: headers,
			handleAs: "json", //$NON-NLS-1$
			postData: postData,
			load: args.load,
			error: args.error
		});
	};
	
	DataCollectionClient.setSchedule = function(args) {
		var url = net.jazz.ajax.contextRoot() + "/datacollection/resource/task"; //$NON-NLS-1$
		var headers = {
			"Accept-Charset": "UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
			"Content-Type": "text/plain;charset=UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
			"Accept": "text/plain" //$NON-NLS-1$ //$NON-NLS-2$
		};
		var postData = "type=schedule&hour=" + args.hour; //$NON-NLS-1$
		jazz.client.xhrPost({
			url: url,
			headers: headers,
			handleAs: "text", //$NON-NLS-1$
			postData: postData,
			load: args.load,
			error: args.error
		});
	};
	
	DataCollectionClient.getSchedule = function(args) {
		var url = net.jazz.ajax.contextRoot() + "/datacollection/resource/task?type=schedule"; //$NON-NLS-1$
		var headers = {
			"Accept-Charset": "UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
			"Accept": "text/plain" //$NON-NLS-1$ //$NON-NLS-2$
		};

		jazz.client.xhrGet({
			preventCache: true,
			url: url,
			headers: headers,
			handleAs: "json", //$NON-NLS-1$
			load: args.load,
			error: args.error
		});
	};
	
	DataCollectionClient.getConfiguration = function(args) {
		var url = net.jazz.ajax.contextRoot() + "/datacollection/resource/configuration"; //$NON-NLS-1$
		var headers = {
			"Accept-Charset": "UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
			"Content-Type": "application/json;charset=UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
			"Accept": "application/json" //$NON-NLS-1$ //$NON-NLS-2$
		};
		
		jazz.client.xhrGet({
			preventCache: true,
			url: url,
			headers: headers,
			handleAs: "json", //$NON-NLS-1$
			load: args.load,
			error: args.error
		});
	};
	
	DataCollectionClient.updateConfiguration = function(args) {
		var url = net.jazz.ajax.contextRoot() + "/datacollection/resource/configuration"; //$NON-NLS-1$
		var headers = {
			"Accept-Charset": "UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
			"Content-Type": "application/json;charset=UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
			"Accept": "application/json" //$NON-NLS-1$ //$NON-NLS-2$
		};
		var postData = dojo.toJson(args.data);
		jazz.client.xhrPost({
			url: url,
			headers: headers,
			handleAs: "json", //$NON-NLS-1$
			postData: postData,
			load: args.load,
			error: args.error
		});
	};
	
	DataCollectionClient.getJobs = function(args) {
		var url = net.jazz.ajax.contextRoot() + "/datacollection/resource/task?showAll=" + (args.showAll? "true" : "false"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		var headers = {
			"Accept-Charset": "UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
			"Content-Type": "application/json;charset=UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
			"Accept": "application/json" //$NON-NLS-1$ //$NON-NLS-2$
		};
		if (lastModified) {
			headers[DataCollectionConstants.HEADER_IF_MODIFIED_SINCE] = lastModified;
		}
		
		return jazz.client.xhrGet({
			preventCache: true,
			url: url,
			headers: headers,
			handleAs: "json", //$NON-NLS-1$
			load: args.load,
			error: args.error
		});
	};
	
	DataCollectionClient.getRunningJobs = function(args) {
		var url = net.jazz.ajax.contextRoot() + "/datacollection/resource/task?type=running"; //$NON-NLS-1$
		var headers = {
			"Accept-Charset": "UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
			"Content-Type": "application/json;charset=UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
			"Accept": "application/json" //$NON-NLS-1$ //$NON-NLS-2$
		};
		if (lastModified) {
			headers[DataCollectionConstants.HEADER_IF_MODIFIED_SINCE] = lastModified;
		}
		
		jazz.client.xhrGet({
			preventCache: true,
			url: url,
			headers: headers,
			handleAs: "json", //$NON-NLS-1$
			load: args.load,
			error: args.error
		});
	};
	
	DataCollectionClient.getHistoryJobs = function(args) {
		var url = net.jazz.ajax.contextRoot() + "/datacollection/resource/task?type=history&pageSize=" +  args.pageSize; //$NON-NLS-1$
		var headers = {
			"Accept-Charset": "UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
			"Content-Type": "application/json;charset=UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
			"Accept": "application/json" //$NON-NLS-1$ //$NON-NLS-2$
		};
		if (lastModified) {
			headers[DataCollectionConstants.HEADER_IF_MODIFIED_SINCE] = lastModified;
		}
		
		jazz.client.xhrGet({
			preventCache: true,
			url: url,
			headers: headers,
			handleAs: "json", //$NON-NLS-1$
			load: args.load,
			error: args.error
		});
	};
	
	DataCollectionClient.deleteHistoryJobs = function(args) {
		var url = net.jazz.ajax.contextRoot() + "/datacollection/resource/task/statuses"; //$NON-NLS-1$
		if ( args.before && args.after ) {
			url = url + "?" + "before="+args.before + "&after="+args.after;
			
		} else if ( args.before ) {
			url = url + "?" + "before="+args.before;
		} else if ( args.after) {
			url = url + "?" + "after="+args.after;
		}
		jazz.client.xhrDelete({
			preventCache: true,
			url: url,
			load: args.load,
			error: args.error
		});
	};
	
	DataCollectionClient.updateDeltaLoadConfiguration = function(args) {
		var url = net.jazz.ajax.contextRoot() + "/datacollection/resource/resourceGroup"; //$NON-NLS-1$
		var headers = {
			"Accept-Charset": "UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
			"Content-Type": "application/json;charset=UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
			"Accept": "application/json" //$NON-NLS-1$ //$NON-NLS-2$
		};

		var putData = dojo.toJson(args.data);
	
		jazz.client.xhrPut({
			url: url,
			headers: headers,
			handleAs: "json", //$NON-NLS-1$
			putData: putData,
			load: dojo.hitch(this, function(data, ioArgs) {
				lastModified = ioArgs.xhr.getResponseHeader(DataCollectionConstants.HEADER_LASTMODIFIED);
				args.load(data, ioArgs);
			}),
			error: args.error
		});
	};
	
	DataCollectionClient.updateMapGroupCollection = function(args) {
		var url = net.jazz.ajax.contextRoot() + "/datacollection/resource/mapGroup"; //$NON-NLS-1$
		var headers = {
			"Accept-Charset": "UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
			"Content-Type": "application/json;charset=UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
			"Accept": "application/json" //$NON-NLS-1$ //$NON-NLS-2$
		};

		var putData = dojo.toJson(args.data);
	
		return jazz.client.xhrPut({
			url: url,
			headers: headers,
			handleAs: "json", //$NON-NLS-1$
			putData: putData,
			load: dojo.hitch(this, function(data, ioArgs) {
				lastModified = ioArgs.xhr.getResponseHeader(DataCollectionConstants.HEADER_LASTMODIFIED);
				args.load(data, ioArgs);
			}),
			error: args.error
		});
	};
	
	DataCollectionClient.importTTL = function(args) {
		var url = net.jazz.ajax.contextRoot() + "/datacollection/importer"; //$NON-NLS-1$
		
		dojo.io.iframe.send({
			form: args.form,
			handleAs: "json", //$NON-NLS-1$
			url: url,
			load: args.load,
			error: args.error
		});
	};
	
	DataCollectionClient.getRegisteredApplications = function(args) {
		var url = net.jazz.ajax.contextRoot() + "/datacollection/appGroupDescription"; //$NON-NLS-1$
		var headers = {
			"Accept-Charset": "UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
			"Content-Type": "application/json;charset=UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
			"Accept": "application/json" //$NON-NLS-1$ //$NON-NLS-2$
		};
		
		var postData = dojo.objectToQuery(args.data);
		
		jazz.client.xhrPost({
			preventCache: true,
			url: url,
			headers: headers,
			handleAs: "json", //$NON-NLS-1$
			postData: postData,
			load: args.load,
			error: args.error
		});
	};
	
	DataCollectionClient.getAllScheduledTasks = function(args) {
		var url = net.jazz.ajax.contextRoot() + "/datacollection/resource/schedule";
		if (args.type) {
			url += "?type=" + encodeURIComponent(args.type);
		}
		var headers = {
				"Accept-Charset": "UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
				"Accept": "application/json" //$NON-NLS-1$ //$NON-NLS-2$
			};
		return jazz.client.xhrGet({
			preventCache: true,
			url: url,
			headers: headers,
			handleAs: "json", //$NON-NLS-1$
			load: args.load,
			error: args.error
		});
	};
	
	DataCollectionClient.removeScheduledTask = function(args) {
		var url = net.jazz.ajax.contextRoot() + "/datacollection/resource/schedule/" + args.id; //$NON-NLS-1$
		return jazz.client.xhrDelete({
			preventCache: true,
			url: url,
			load: args.load,
			error: args.error
		});
	};

	DataCollectionClient.getScheduledTask = function(args) {
		var url = net.jazz.ajax.contextRoot() + "/datacollection/resource/schedule/" + args.id; //$NON-NLS-1$
		var headers = {
				"Accept-Charset": "UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
				"Accept": "application/json" //$NON-NLS-1$ //$NON-NLS-2$
			};
		return jazz.client.xhrGet({
			preventCache: true,
			url: url,
			headers: headers,
			handleAs: "json", //$NON-NLS-1$
			load: args.load,
			error: args.error
		});
	};
	
	DataCollectionClient.createScheduledTask = function(args) {
		var url = net.jazz.ajax.contextRoot() + "/datacollection/resource/schedule"; //$NON-NLS-1$
		var headers = {
				"Accept-Charset": "UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
				"Content-Type": "application/json;charset=UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
				"Accept": "application/json" //$NON-NLS-1$ //$NON-NLS-2$
			};
		return jazz.client.xhrPost({
			preventCache: true,
			url: url,
			headers: headers,
			handleAs: "json", //$NON-NLS-1$
			postData: dojo.toJson(args.data),
			load: args.load,
			error: args.error
		});
	};
	
	DataCollectionClient.modifyScheduledTask = function(args) {
		var url = net.jazz.ajax.contextRoot() + "/datacollection/resource/schedule/" + args.id; //$NON-NLS-1$
		var headers = {
				"Accept-Charset": "UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
				"Content-Type": "application/json;charset=UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
				"Accept": "application/json" //$NON-NLS-1$ //$NON-NLS-2$
			};
		return jazz.client.xhrPut({
			preventCache: true,
			url: url,
			headers: headers,
			handleAs: "json", //$NON-NLS-1$
			postData: dojo.toJson(args.data),
			load: args.load,
			error: args.error
		});
	};
	
	DataCollectionClient.getDccEnabled = function(args) {
		var url = net.jazz.ajax.contextRoot() + "/datacollection/resource/properties?name=enabled"; //$NON-NLS-1$
		var headers = {
				"Accept-Charset": "UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
				"Content-Type": "application/json;charset=UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
				"Accept": "application/json" //$NON-NLS-1$ //$NON-NLS-2$
			};
		return jazz.client.xhrGet({
			preventCache: true,
			url: url,
			headers: headers,
			handleAs: "json", //$NON-NLS-1$
			load: args.load,
			error: args.error
		});
	};
	
	DataCollectionClient.setDccEnabled = function(args) {
		var url = net.jazz.ajax.contextRoot() + "/datacollection/resource/properties"; //$NON-NLS-1$
		var headers = {
				"Accept-Charset": "UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
				"Content-Type": "application/json;charset=UTF-8", //$NON-NLS-1$ //$NON-NLS-2$
				"Accept": "application/json" //$NON-NLS-1$ //$NON-NLS-2$
			};
		var data = dojo.toJson({"name":"enabled", "value":args.value});
		return jazz.client.xhrPost({
			preventCache: true,
			url: url,
			headers: headers,
			handleAs: "json", //$NON-NLS-1$
			postData: data,
			load: args.load,
			error: args.error
		});
	};


})();
