/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
	dojo.provide("com.ibm.rational.datacollection.web.ui.internal.common.ResourceHelper");//$NON-NLS-1$
	dojo.require("com.ibm.rational.datacollection.web.ui.internal.common.DataCollectionConstants"); //$NON-NLS-1$
	
	var DataCollectionConstants = com.ibm.rational.datacollection.web.ui.internal.common.DataCollectionConstants;
	com.ibm.rational.datacollection.web.ui.internal.common.ResourceHelper = {
		getAuthenticationConfiguration: function(resourceGroup){
			var authenticationConfiguration = null;
			for (var key in resourceGroup) {
				if (resourceGroup[key][DataCollectionConstants.NS_AUTH_CONFIG]) {
					var nodeId = resourceGroup[key][DataCollectionConstants.NS_AUTH_CONFIG][0].value;
					authenticationConfiguration = resourceGroup[nodeId];
					break;
				}
			}
			
			return authenticationConfiguration;
		},
		
		getResourceGroup: function(item) {
			var resourceGroup = null;
			for (var key in item) {
				if (item[key][DataCollectionConstants.NS_AUTH_CONFIG]) {
					resourceGroup = item[key];
					break;
				}
			}
			
			return resourceGroup;
		},
		
		getNodeId: function(item, node) {
			var nodeId = null;
			for (var key in item) {
				if (item[key] === node) {
					nodeId = key;
					break;
				}
			}
			
			return nodeId;
		},
		
		getParameterArray: function(item) {
			var parameters = [];
			var resourceGroupMainPart = this.getResourceGroup(item);
			if (resourceGroupMainPart[DataCollectionConstants.NS_PARAMETER]) {
				for (var i = 0; i < resourceGroupMainPart[DataCollectionConstants.NS_PARAMETER].length; i++) {
					var nodeId = resourceGroupMainPart[DataCollectionConstants.NS_PARAMETER][i]["value"]; //$NON-NLS-1$
					parameters.push(item[nodeId]);
				}
			}
			
			return parameters;
		},
		
		getBaseResourceGroupUri: function(resourceGroup) {
			var uri = null;
			for (var key in resourceGroup) {
				if (resourceGroup[key][DataCollectionConstants.NS_BASE_RESOURCE_GROUP]) {
					var uri = resourceGroup[key][DataCollectionConstants.NS_BASE_RESOURCE_GROUP][0].value;
					break;
				}
			}
			
			return uri;
		},
		
		isRM: function(resourceGroup) {
			var uri = this.getBaseResourceGroupUri(resourceGroup);
			return this.endsWith(uri, DataCollectionConstants.RM_BASE_RESOURCE_GROUP_NAME);
		},
		
		getResourceUri:  function(resource) {
			var uri = null;
			for (var key in resource) {
				if (resource[key][DataCollectionConstants.NS_RDF_TYPE]) {
					uri = key;
					break;
				}
			}
			
			return uri;
		},
		
		getBaseURL:  function(resource) {
			var baseURL = null;
			for (var key in resource) {
				if (resource[key][DataCollectionConstants.NS_BASE_URL]) {
					baseURL = resource[key][DataCollectionConstants.NS_BASE_URL][0].value;
					break;
				}
			}
			
			return baseURL;
		},
		
		getResourceId:  function(resource) {
			var uri = this.getResourceUri(resource);
			if (uri) {
				var segments = uri.split(DataCollectionConstants.FORWARD_SLASH);
				return segments[segments.length-1];
			} else {
				return null;
			}
		},
		
		getResourceIdByUri:  function(uri) {
			if (uri) {
				var segments = uri.split(DataCollectionConstants.FORWARD_SLASH);
				return segments[segments.length-1];
			} else {
				return null;
			}
		},
		
		calculateStatus: function(oldStatus, newStatus) {
			if (newStatus === DataCollectionConstants.STATUS_DELETE) {
				if (oldStatus) {
					if (oldStatus === DataCollectionConstants.STATUS_NEW) {
						return DataCollectionConstants.STATUS_IGNORE;
					} else if (oldStatus === DataCollectionConstants.STATUS_UPDATE) {
						return DataCollectionConstants.STATUS_DELETE;
					} else {
						return oldStatus;
					}
				} else {
					return DataCollectionConstants.STATUS_DELETE;
				}
			} else if (newStatus === DataCollectionConstants.STATUS_UPDATE) {
				if (oldStatus) {
					if (oldStatus === DataCollectionConstants.STATUS_NEW) {
						return DataCollectionConstants.STATUS_NEW;
					}
				} else {
					return DataCollectionConstants.STATUS_UPDATE;
				}
			}
			return oldStatus;
		},
		
		endsWith: function (str, end) {
			if((str.length - end.length) < 0){
				return false;
			}
			return str.lastIndexOf(end) == str.length - end.length;
		}
	}
	
})();
