/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013, 2014. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
	dojo.provide("com.ibm.rational.datacollection.web.ui.internal.common.WorkingCopy"); //$NON-NLS-1$
	dojo.require("dojox.xml.parser"); //$NON-NLS-1$
	dojo.require("dojox.uuid.Uuid"); //$NON-NLS-1$
	dojo.require("com.ibm.rational.datacollection.web.ui.internal.common.DataCollectionConstants"); //$NON-NLS-1$
	dojo.require("com.ibm.rational.datacollection.web.ui.internal.common.DataCollectionClient"); //$NON-NLS-1$
	dojo.require("com.ibm.rational.datacollection.web.ui.internal.common.ResourceHelper");//$NON-NLS-1$
	dojo.requireLocalization("com.ibm.rational.datacollection.web.ui.internal.common", "WorkingCopyMessages");//$NON-NLS-1$//$NON-NLS-2$

	var DataCollectionConstants = com.ibm.rational.datacollection.web.ui.internal.common.DataCollectionConstants;
	var DataCollectionClient = com.ibm.rational.datacollection.web.ui.internal.common.DataCollectionClient;
	var ResourceHelper = com.ibm.rational.datacollection.web.ui.internal.common.ResourceHelper;
	dojo.declare("com.ibm.rational.datacollection.web.ui.internal.common.WorkingCopy", null, { //$NON-NLS-1$
		baseResourceGroupMap: null,
		resourceGroupList: null,
		currentIndex: 0,
		constructor: function(args) {
			this.messages = dojo.i18n.getLocalization("com.ibm.rational.datacollection.web.ui.internal.common", "WorkingCopyMessages");//$NON-NLS-1$ //$NON-NLS-2$
			this.baseResourceGroupMap = {};
			this.resourceGroupList = [];
			
			this.resourceGroupNameMap = {};
			this.resourceGroupNameMap[DataCollectionConstants.APPID_JTS] = {
				name: DataCollectionConstants.JTS_RESOURCE_GROUP_NAME,
				title: this.messages.WorkingCopy_6
			};
			this.resourceGroupNameMap[DataCollectionConstants.APPID_CCM] = {
				name: DataCollectionConstants.CCM_RESOURCE_GROUP_NAME,
				title: this.messages.WorkingCopy_1
			};
			this.resourceGroupNameMap[DataCollectionConstants.APPID_QM] = {
				name: DataCollectionConstants.QM_RESOURCE_GROUP_NAME,
				title: this.messages.WorkingCopy_2
			};
			this.resourceGroupNameMap[DataCollectionConstants.APPID_RM] = {
				name: DataCollectionConstants.RM_RESOURCE_GROUP_NAME,
				title: this.messages.WorkingCopy_7
			};
			this.resourceGroupNameMap[DataCollectionConstants.APPID_CUSTOM] = {
				name: DataCollectionConstants.CUSTOM_RESOURCE_GROUP_NAME,
				title: this.messages.WorkingCopy_4
			}
			
			this.resourceGroupNameMap[DataCollectionConstants.APPID_RDB] = {
				name: DataCollectionConstants.RDB_RESOURCE_GROUP_NAME,
				title: this.messages.WorkingCopy_5
			}
		},
		
		initializeBaseResourceGroup: function (baseResourceGroupCollection) {
			for (var i = 0; i < baseResourceGroupCollection.length; i++) {
				var baseResourceGroup = baseResourceGroupCollection[i];
				this.baseResourceGroupMap[baseResourceGroup.uri] = baseResourceGroup;
			}
		},
		
		getBaseResourceGroup: function(uri) {
			return this.baseResourceGroupMap[uri];
		},
		
		addResourceGroup: function(data, etag) {
			var resourceId = ResourceHelper.getResourceId(data);
			var item = {
				data: data,
				etag: etag,
				resourceId: resourceId
			}
			
			this.resourceGroupList.push(item);
			return item;
		},
		
		createParameter: function(item) {
			var resourceGroupMainPart = ResourceHelper.getResourceGroup(item.data);
			var parameter = {};
			var uuid = dojox.uuid.generateTimeBasedUuid();
			var nodeId = "_:" + uuid; //$NON-NLS-1$
			if (!resourceGroupMainPart[DataCollectionConstants.NS_PARAMETER]) {
				resourceGroupMainPart[DataCollectionConstants.NS_PARAMETER] = [{}];
				resourceGroupMainPart[DataCollectionConstants.NS_PARAMETER][0]["type"] = "bnode"; //$NON-NLS-1$ //$NON-NLS-2$
				resourceGroupMainPart[DataCollectionConstants.NS_PARAMETER][0]["value"] = nodeId; //$NON-NLS-1$
			} else {
				resourceGroupMainPart[DataCollectionConstants.NS_PARAMETER].push({
					type: "bnode", //$NON-NLS-1$
					value: nodeId
				});
			}
			
			parameter[DataCollectionConstants.NS_NAME] = [{}];
			parameter[DataCollectionConstants.NS_NAME][0]["type"] = "literal"; //$NON-NLS-1$ //$NON-NLS-2$
			parameter[DataCollectionConstants.NS_NAME][0]["value"] = ""; //$NON-NLS-1$ //$NON-NLS-2$
			
			parameter[DataCollectionConstants.NS_LOCAL_TRANSFORM] = [{}];
			parameter[DataCollectionConstants.NS_LOCAL_TRANSFORM][0]["type"] = "literal"; //$NON-NLS-1$ //$NON-NLS-2$
			parameter[DataCollectionConstants.NS_LOCAL_TRANSFORM][0]["value"] = ""; //$NON-NLS-1$ //$NON-NLS-2$
			item.data[nodeId] = parameter;
			
			return parameter;
		},
		
		removeParameter: function(item, parameter) {
			var resourceGroupMainPart = ResourceHelper.getResourceGroup(item.data);
			var nodeId = ResourceHelper.getNodeId(item.data, parameter);
			delete item.data[nodeId];
			if (resourceGroupMainPart[DataCollectionConstants.NS_PARAMETER].length == 1) {
				delete resourceGroupMainPart[DataCollectionConstants.NS_PARAMETER];
			} else {
				for (var i = 0; i < resourceGroupMainPart[DataCollectionConstants.NS_PARAMETER].length; i++) {
					var p = resourceGroupMainPart[DataCollectionConstants.NS_PARAMETER][i];
					if (p.value === nodeId) {
						resourceGroupMainPart[DataCollectionConstants.NS_PARAMETER].splice(dojo.indexOf(resourceGroupMainPart[DataCollectionConstants.NS_PARAMETER], p), 1);
						break;
					}
				}
			}
		},
		
		createResourceGroup: function(baseResourceGroupUri, baseURL) {
			var info = this.generateResourceGroupInfo(baseResourceGroupUri);
			var emptyResourceGroup = this.generateEmptyResourceGroup(baseResourceGroupUri, info.title, baseURL);
			
			var item = {
				data: emptyResourceGroup,
				resourceId: info.name,
				status: DataCollectionConstants.STATUS_NEW
			}
			this.resourceGroupList.push(item);
			return item;
		},
		
		generateResourceGroupInfo: function(baseResourceGroupUri) {
			var baseResourceGroup = this.baseResourceGroupMap[baseResourceGroupUri];
			var identifier = baseResourceGroup["dcterms:identifier"]; //$NON-NLS-1$
			var name = this.resourceGroupNameMap[identifier].name;
			var title = this.resourceGroupNameMap[identifier].title;
			var i = 0;
			var conflict;
			do {
				conflict = false;
				for (var j = 0; j < this.resourceGroupList.length; j++) {
					if (this.resourceGroupList[j].resourceId === (name+i)) {
						conflict = true;
						break;
					}
				}
				if (conflict) {
					i++;
				}
			} while (conflict);
			
			return {
				name: name+i,
				title: dojo.string.substitute(title, [i])
			};
		},
		
		generateEmptyResourceGroup: function(baseResourceGroupUri, title, baseURL) {
			var resourceGroup = {};
			var resourceGroupMainPart = {};
			var authConfig = {};
			var uuid = dojox.uuid.generateTimeBasedUuid();
			var nodeId = "_:" + uuid; //$NON-NLS-1$
			
			resourceGroup[""] = resourceGroupMainPart; //$NON-NLS-1$
			resourceGroupMainPart["http://www.w3.org/1999/02/22-rdf-syntax-ns#type"] = [{}]; //$NON-NLS-1$
			resourceGroupMainPart["http://www.w3.org/1999/02/22-rdf-syntax-ns#type"][0]["type"] = "uri"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			resourceGroupMainPart["http://www.w3.org/1999/02/22-rdf-syntax-ns#type"][0]["value"] = "http://jazz.net/ns/dcs#ResourceGroup"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			
			resourceGroupMainPart["http://purl.org/dc/terms/title"] = [{}]; //$NON-NLS-1$
			resourceGroupMainPart["http://purl.org/dc/terms/title"][0]["type"] = "literal"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			resourceGroupMainPart["http://purl.org/dc/terms/title"][0]["value"] = title; //$NON-NLS-1$ //$NON-NLS-2$
			
			resourceGroupMainPart["http://jazz.net/ns/dcs#resourceGroupType"] = [{}]; //$NON-NLS-1$
			resourceGroupMainPart["http://jazz.net/ns/dcs#resourceGroupType"][0]["type"] = "literal"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			resourceGroupMainPart["http://jazz.net/ns/dcs#resourceGroupType"][0]["value"] = "http://jazz.net/ns/dcs#DataService"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			
			resourceGroupMainPart["http://jazz.net/ns/dcs#baseURL"] = [{}]; //$NON-NLS-1$
			resourceGroupMainPart["http://jazz.net/ns/dcs#baseURL"][0]["type"] = "literal"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			resourceGroupMainPart["http://jazz.net/ns/dcs#baseURL"][0]["value"] = baseURL ? baseURL : ""; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			
			resourceGroupMainPart["http://jazz.net/ns/dcs#baseResourceGroup"] = [{}]; //$NON-NLS-1$
			resourceGroupMainPart["http://jazz.net/ns/dcs#baseResourceGroup"][0]["type"] = "uri"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			resourceGroupMainPart["http://jazz.net/ns/dcs#baseResourceGroup"][0]["value"] = baseResourceGroupUri; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			
			resourceGroupMainPart["http://jazz.net/ns/dcs#enabled"] = [{}]; //$NON-NLS-1$
			resourceGroupMainPart["http://jazz.net/ns/dcs#enabled"][0]["type"] = "literal"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			resourceGroupMainPart["http://jazz.net/ns/dcs#enabled"][0]["value"] = "true"; //$NON-NLS-2$ //$NON-NLS-1$ //$NON-NLS-3$
			
			resourceGroupMainPart["http://jazz.net/ns/dcs#loadType"] = [{}]; //$NON-NLS-1$
			resourceGroupMainPart["http://jazz.net/ns/dcs#loadType"][0]["type"] = "literal"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			resourceGroupMainPart["http://jazz.net/ns/dcs#loadType"][0]["value"] = DataCollectionConstants.DELTA_LOAD; //$NON-NLS-1$; //$NON-NLS-2$ 
			
			resourceGroupMainPart["http://jazz.net/ns/dcs#version"] = [{}]; //$NON-NLS-1$
			resourceGroupMainPart["http://jazz.net/ns/dcs#version"][0]["type"] = "literal"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			resourceGroupMainPart["http://jazz.net/ns/dcs#version"][0]["value"] = DataCollectionConstants.VERSION_50_VALUE; //$NON-NLS-1$ //$NON-NLS-2$
			
			resourceGroupMainPart["http://jazz.net/ns/dcs#supportQuery"] = [{}]; //$NON-NLS-1$
			resourceGroupMainPart["http://jazz.net/ns/dcs#supportQuery"][0]["type"] = "literal"; //$NON-NLS-1$ //$NON-NLS-3$ //$NON-NLS-2$
			resourceGroupMainPart["http://jazz.net/ns/dcs#supportQuery"][0]["value"] = "true"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$

			resourceGroupMainPart["http://jazz.net/ns/dcs#authenticationConfiguration"] = [{}]; //$NON-NLS-1$
			resourceGroupMainPart["http://jazz.net/ns/dcs#authenticationConfiguration"][0]["type"] = "bnode"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			resourceGroupMainPart["http://jazz.net/ns/dcs#authenticationConfiguration"][0]["value"] = nodeId; //$NON-NLS-1$ //$NON-NLS-2$
			
			resourceGroup[nodeId] = authConfig;
			authConfig["http://jazz.net/ns/dcs#password"] = [{}]; //$NON-NLS-1$
			authConfig["http://jazz.net/ns/dcs#password"][0]["type"] = "literal"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			authConfig["http://jazz.net/ns/dcs#password"][0]["value"] = ""; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			
			authConfig["http://jazz.net/ns/dcs#username"] = [{}]; //$NON-NLS-1$
			authConfig["http://jazz.net/ns/dcs#username"][0]["type"] = "literal"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			authConfig["http://jazz.net/ns/dcs#username"][0]["value"] = ""; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			
			authConfig["http://jazz.net/ns/dcs#authenticationType"] = [{}]; //$NON-NLS-1$
			authConfig["http://jazz.net/ns/dcs#authenticationType"][0]["type"] = "literal"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			
			if(baseResourceGroupUri.indexOf(DataCollectionConstants.RDB_BASE_RESOURCE_GROUP_NAME) < 0){
				authConfig["http://jazz.net/ns/dcs#authenticationType"][0]["value"] = DataCollectionConstants.AUTH_TYPE_BASIC_VALUE; //$NON-NLS-1$ //$NON-NLS-2$
			}
			
			if (baseResourceGroupUri.indexOf(DataCollectionConstants.RDB_BASE_RESOURCE_GROUP_NAME) !== -1) {
				resourceGroupMainPart[DataCollectionConstants.NS_RESOURCE_GROUP_TYPE][0].value = DataCollectionConstants.RCS_GROUP_TYPE_DB_VALUE;
				resourceGroupMainPart[DataCollectionConstants.NS_DB_TYPE] = [{type: "literal", value: DataCollectionConstants.DB_TYPE_DB2_VALUE}]; //$NON-NLS-1$
				authConfig[DataCollectionConstants.NS_AUTH_TYPE][0].value = DataCollectionConstants.AUTH_TYPE_JDBC_VALUE;
			} else {
				authConfig[DataCollectionConstants.NS_AUTH_TYPE][0].value = DataCollectionConstants.AUTH_TYPE_BASIC_VALUE; 
			}
			
			return resourceGroup;
		},
		
		prepareResourceGroupList: function() {
			var otherResourceGroupList = [];
			var rmResourceGroupList = [];
			
			dojo.forEach(this.resourceGroupList, function(item) {
				if (ResourceHelper.isRM(item.data)) {
					rmResourceGroupList.push(item);
				} else {
					otherResourceGroupList.push(item);
				}
			});
			
			this.resourceGroupList = otherResourceGroupList.concat(rmResourceGroupList);
		},
		
		doSave: function(callback) {
			this.prepareResourceGroupList();
			this._doSave(callback);
		},
		
		_doSave: function(callback) {
			if (this.currentIndex === this.resourceGroupList.length) {
				callback();
			} else {
				var item = this.resourceGroupList[this.currentIndex];
				if (item.status === DataCollectionConstants.STATUS_NEW) {
					DataCollectionClient.createResourceGroup({
						data: item.data,
						resourceId: item.resourceId,
						load: dojo.hitch(this, function(data, ioArgs) {
							this.currentIndex++;
							this._doSave(callback);
						}),
						error: dojo.hitch(this, function(error, ioArgs) {
							callback(error);
						})
					});
				} else if (item.status === DataCollectionConstants.STATUS_DELETE) {
					DataCollectionClient.deleteResourceGroup({
						resourceId: item.resourceId,
						load: dojo.hitch(this, function(data, ioArgs) {
							this.currentIndex++;
							this._doSave(callback);
						}),
						error: dojo.hitch(this, function(error, ioArgs) {
							callback(error);
						})
					});
				} else if (item.status === DataCollectionConstants.STATUS_UPDATE) {
					DataCollectionClient.updateResourceGroup({
						data: item.data,
						etag: item.etag,
						resourceId: item.resourceId,
						load: dojo.hitch(this, function(data, ioArgs) {
							this.currentIndex++;
							this._doSave(callback);
						}),
						error: dojo.hitch(this, function(error, ioArgs) {
							callback(error);
						})
					});
				} else {
					this.currentIndex++;
					this._doSave(callback);
				}
			}
		}
	});
	
})();
