/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.rational.datacollection.web.ui.internal.dialogs.ImportDialog"); //$NON-NLS-1$
dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("dojox.xml.parser"); //$NON-NLS-1$
dojo.require("dojo.io.iframe"); //$NON-NLS-1$
dojo.require("dijit.ProgressBar"); //$NON-NLS-1$
dojo.require("dojox.uuid"); //$NON-NLS-1$
dojo.require("dojox.uuid.generateTimeBasedUuid"); //$NON-NLS-1$
dojo.require("jazz.ui.Dialog");//$NON-NLS-1$
dojo.require("com.ibm.rational.datacollection.web.ui.internal.common.DataCollectionClient"); //$NON-NLS-1$
dojo.require("com.ibm.rational.datacollection.web.ui.internal.common.DataCollectionConstants"); //$NON-NLS-1$
dojo.require("com.ibm.rational.datacollection.web.ui.internal.common.ResourceHelper");//$NON-NLS-1$
dojo.requireLocalization("com.ibm.rational.datacollection.web.ui.internal.dialogs", "ImportDialogMessages");//$NON-NLS-1$//$NON-NLS-2$

// Sample url: https://app:9443/jazz/_ajax-modules/com.ibm.rational.datacollection.web.ImportMapping?debug=true
(function() {
	
var _uploadFileInput_onchange_handle;
var _uploadFileInput;
var _okButton_onclick_handle;
var _jsonResponse;
var _currentRequest;
var _WIDTH_IN_EM = 50;
var DataCollectionClient = com.ibm.rational.datacollection.web.ui.internal.common.DataCollectionClient;
var DataCollectionConstants = com.ibm.rational.datacollection.web.ui.internal.common.DataCollectionConstants;
var ResourceHelper = com.ibm.rational.datacollection.web.ui.internal.common.ResourceHelper;	

dojo.declare("com.ibm.rational.datacollection.web.ui.internal.dialogs.ImportDialog", [dijit._Widget, dijit._Templated], { //$NON-NLS-1$
	templatePath: dojo.moduleUrl("com.ibm.rational.datacollection.web", "ui/internal/dialogs/templates/ImportDialog.html"),//$NON-NLS-1$ //$NON-NLS-2$ 
	widgetsInTemplate: true,
	constructor: function(args) {
		if (!args) {
			args = {};
		}
		
		this.width = _WIDTH_IN_EM + "em"; //$NON-NLS-1$
		this.uploadId = null;
		this.intervalId = null;
		var properties = {
			"width": this.width //$NON-NLS-1$
		};
		this.messages = dojo.i18n.getLocalization("com.ibm.rational.datacollection.web.ui.internal.dialogs", "ImportDialogMessages");//$NON-NLS-1$ //$NON-NLS-2$
	},
	
	postCreate: function() {
		
		this.dialog = new jazz.ui.Dialog({
			contentNode: this.contentNode,
			primaryTitle: this.messages.ImportDialog_2,
			width: this.width,
			destroyContents: true
		});
		this._topText.innerHTML = this.messages.ImportDialog_1;

		var div = this._uploadInputDiv;
		dojo.empty(div);
		var input1 = document.createElement("input"); //$NON-NLS-1$
		input1.type = "file"; //$NON-NLS-1$
		input1.name = "uploadFileInput";//$NON-NLS-1$
		input1.size = _WIDTH_IN_EM - 5; //size is the width of the input in characters
		this._uploadFileInput = input1;
		this._uploadFileInput_onchange_handle = dojo.connect(this._uploadFileInput, "onchange", this, "_handleDirectoryChange"); //$NON-NLS-1$ //$NON-NLS-2$
		if (!dojo.isIE) {
			// setting focus here causes IE to not show the field, so don't 
			//do it 
			this._uploadFileInput.focus();
		}
		div.appendChild(input1);
		
		this.connect(this.cancelButton, "onclick", "onCancel");  //$NON-NLS-1$ //$NON-NLS-2$ 
		this.connect(this.closeButton, "onclick", "onClose");  //$NON-NLS-1$ //$NON-NLS-2$ 
		this._okButton_onclick_handle = this.connect(this.okButton, "onclick", "_onOk"); //$NON-NLS-1$ //$NON-NLS-2$
	},

	_handleDirectoryChange: function(e) {
		if (e) {
			dojo.stopEvent(e);
		}
		
		this._clearStatusMessage();
		var fullPath = this._uploadFileInput.value;
		if (fullPath.length !== 0) {
			var parts = fullPath.split("\\"); //$NON-NLS-1$
			var fileName = parts[parts.length - 1];
			var extensionIndex = fileName.toLocaleLowerCase().lastIndexOf(".ttl"); //$NON-NLS-1$
			if (!(extensionIndex == fileName.length - 4)) {
				this._setStatusMessage(this.messages.ImportDialog_3, true);
				return;
			} else {
				this.okButton.disabled = false;
			}
		}
	},
	_runSelect: function(e) {
		dojo.stopEvent(e);
		
	},
	_runDeselect: function(e) {
		dojo.stopEvent(e);
		
	},
	_onOk: function(e) {
		if (e) {
			dojo.stopEvent(e);
		}
		this._setStatusMessage(this.messages.ImportDialog_4);
		this._showProgressBar(true);
		
		// create unique id for this upload
		this.uploadId = dojox.uuid.generateTimeBasedUuid();
				
		DataCollectionClient.importTTL({
			form: this._uploadForm,
			load: dojo.hitch(this, this._handleLoad),
			error: dojo.hitch(this, this._handleErrorOnUpload)
		});
		
		this.okButton.disabled = true;
		this._uploadFileInput.disabled = true;
	},
	
	onCancel: function(e) {
		dojo.stopEvent(e);
		
		this.dialog.close();
	},
	
	onClose: function(e) {
		dojo.stopEvent(e);
		this.dialog.close();
	},
	
	_handleLoad: function(response, ioArgs) {
		if (response instanceof Object) {
			this._jsonResponse = response;
		} else {
			this._jsonResponse = dojo.fromJson(response);
		}
		switch (this._jsonResponse.status) {
			case 200:
				//success case
				clearInterval(this.intervalId);
				this.onSuccess(this._jsonResponse);
				break;
			case 202:
				//upload is done, import starting
				this._setStatusMessage(this._jsonResponse.message, false);
				break;
			default:
				//any other error
				clearInterval(this.intervalId);
				this._setStatusMessage(this._jsonResponse.message, true);
				this._progress.domNode.style.display = "none"; //$NON-NLS-1$
				this._uploadFileInput.disabled = false;
				//enable the OK button so they can fix the problem and try again
				this.okButton.disabled = false;
				break;
		}
	},
	
	_formatSummary: function(msg) {
		var lines = msg.split("\n"); //$NON-NLS-1$
		var summary = ""; //$NON-NLS-1$
		for (var i = 0; i < lines.length; i++) {
			summary = summary + lines[i] + "<br>"; //$NON-NLS-1$
		}
		return summary;
	},
	
	onSuccess: function(json) {
		this._hideFileUploadSection();
		_currentRequest = null;
		this.okButton.style.display = "none"; //$NON-NLS-1$
		this.cancelButton.style.display = "none"; //$NON-NLS-1$
		this.closeButton.style.display = ""; //$NON-NLS-1$
		this.closeButton.innerHTML = this.messages.ImportDialog_17;
		if (json.message) {
			// show import summary
			this._topText.innerHTML = this._formatSummary(json.message);			
		} else {
			this._dialogHeading.style.display = "none"; //$NON-NLS-1$
		}
		if (json.errorMessage) {				
			// show error message
			this._setStatusMessage(json.errorMessage, true);
		} else {
			this.status.style.display = "none"; //$NON-NLS-1$
		}
	},
	
	_handleErrorOnUpload: function(response, ioArgs) {
		if (!response.dojoType || !response.dojoType === "cancel") { //$NON-NLS-1$
			clearInterval(this.intervalId);
			this._setStatusMessage(response, true);
		}
	},
	
	_hideFileUploadSection: function() {
		this._clearStatusMessage();
		this._uploadForm.style.display = "none"; //$NON-NLS-1$
		this._progress.domNode.style.display = "none"; //$NON-NLS-1$
	},
	
	_showFileUploadSection: function() {
		this._uploadForm.style.display = "block";//$NON-NLS-1$
	},
	
	_clearStatusMessage: function() {
		this.status.style.visibility = "hidden";//$NON-NLS-1$
	},
	
	_setStatusMessage: function(msg, isError) {
		this.status.style.visibility = "visible";//$NON-NLS-1$
		dojox.xml.parser.replaceChildren(this.status, document.createTextNode(msg));
		this.status.className = isError ? "ErrorStatus" : "NormalStatus";//$NON-NLS-1$ //$NON-NLS-2$
	},
	
	_showProgressBar: function(indeterminate) {
		this._progress.update({
			indeterminate: indeterminate
		});
		this._progress.domNode.style.display = "block";//$NON-NLS-1$
	},
	
	uninitialize: function() {
		if (this._uploadFileInput_onchange_handle) {
			dojo.disconnect(this._uploadFileInput_onchange_handle);
		}
		this.inherited(arguments);
	}
});
})();
