/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.rational.datacollection.web.ui.internal.editors.ConfigureDWConnection"); //$NON-NLS-1$

dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang"); //$NON-NLS-1$
dojo.require("com.ibm.rational.datacollection.web.ui.internal.common.DataCollectionClient"); //$NON-NLS-1$
dojo.require("com.ibm.rational.datacollection.web.ui.internal.common.ResourceHelper");//$NON-NLS-1$
dojo.require("com.ibm.rational.datacollection.web.ui.internal.editors.StringProperty"); //$NON-NLS-1$
dojo.require("com.ibm.rational.datacollection.web.ui.internal.editors.AbstractEditor"); //$NON-NLS-1$
dojo.require("jazz.client.xhr"); //$NON-NLS-1$
dojo.require("jazz.ui.underlay"); //$NON-NLS-1$
dojo.require("jazz.app.ua.PageHoverHelp"); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.rational.datacollection.web.ui.internal.editors", "ConfigureDWConnectionMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function() {
	
var DataCollectionClient = com.ibm.rational.datacollection.web.ui.internal.common.DataCollectionClient;
var StringProperty = com.ibm.rational.datacollection.web.ui.internal.editors.StringProperty;
var ResourceHelper = com.ibm.rational.datacollection.web.ui.internal.common.ResourceHelper;
var AbstractEditor = com.ibm.rational.datacollection.web.ui.internal.editors.AbstractEditor;
var PageHoverHelp = jazz.app.ua.PageHoverHelp;

dojo.declare("com.ibm.rational.datacollection.web.ui.internal.editors.ConfigureDWConnection", [AbstractEditor], { //$NON-NLS-1$

	templatePath: dojo.moduleUrl("com.ibm.rational.datacollection.web", "ui/internal/editors/templates/ConfigureDWConnection.html"), //$NON-NLS-1$ //$NON-NLS-2$
	propertyWidgets: null,
	
	// widget initialization
	constructor: function() {
		this.messages = dojo.i18n.getLocalization("com.ibm.rational.datacollection.web.ui.internal.editors", "ConfigureDWConnectionMessages"); //$NON-NLS-1$ //$NON-NLS-2$
	},

	postCreate: function () {
		this.inherited(arguments);
		
		this.propertyWidgets = [];
		this.refresh();
		this.connect(this._saveButton, "onclick", function(e) { //$NON-NLS-1$
			jazz.ui.underlay.show();
			var configData = {};
			for (var i = 0; i < this.propertyWidgets.length; i++) {
				var propsData = this.propertyWidgets[i].getData();
				configData[propsData.key] = propsData.value;
			}
			
			DataCollectionClient.testDWConnection({
				vendor: this.propertyWidgets[0].value,
				password: this.propertyWidgets[2].value,
				location: this.propertyWidgets[1].value,
				load: dojo.hitch(this, function(response, ioArgs) {
					var _self = this;
					var error = false;
					if (response != null) {
						for (var i = 0; i < response.length; i++) {
							if (response[i].code === "ERROR") { //$NON-NLS-1$
								this.displayDetailedErrorMessage(this.messages.testConnectionFailed, response[i]);
								jazz.ui.underlay.hide();
								//Check to see if we need to create Tables (i.e. error code 3)
//								if (response[i].id == 3) {
//									
//									break;
//								}
								error = true;
							}
						}
					} 
					
					if(!error){
						// Update the new changes to the server
						this.updateDBConfiguration();
					}
					
				}),
				error: dojo.hitch(this, function(error, ioArgs) {
					this.displayDetailedErrorMessage(this.messages.testConnectionFailed, error);
					jazz.ui.underlay.hide();
				})
			});
			
		});
		
		this.connect(this._testConnButton, "onclick", function(e) { //$NON-NLS-1$
			this.testConnection();
		});
		
		this.connect(this._editLink, "onclick", function(e) { //$NON-NLS-1$
			this.toggleEditMode(true);
		});
		
		this.connect(this._viewLink, "onclick", function(e) { //$NON-NLS-1$
			this.toggleEditMode(false);
		});
		
		this.connect(this._etlEditLink, "onclick", function(e) { //$NON-NLS-1$
			this.toggleETLEditMode(true);
		});
		
		this.connect(this._etlViewLink, "onclick", function(e) { //$NON-NLS-1$
			this.toggleETLEditMode(false);
		});
		
		// Add the page hover description
		this._pageHelpHover = new PageHoverHelp({ contentString: this.messages.pageHoverHelpDescription });
		this.helpHover.appendChild(this._pageHelpHover.domNode);
	},
	
	updateDBConfiguration : function (){
		var configData = {};
		for (var i = 0; i < this.propertyWidgets.length; i++) {
			var propsData = this.propertyWidgets[i].getData();
			configData[propsData.key] = propsData.value;
		}
		DataCollectionClient.updateConfiguration({
			data: configData,
			load: dojo.hitch(this, function(data, ioArgs) {
				this.refresh();
			}),
			error: dojo.hitch(this, function(error, ioArgs) {
				this.displayDetailedErrorMessage(this.messages.updateConfigurationFailed, error);
				jazz.ui.underlay.hide();
			})
		})
	},
	
	testConnection: function () {
		this.displayInfoMessage(this.messages.testConnectionInProgress);
		DataCollectionClient.testDWConnection({
			vendor: this.propertyWidgets[0].value,
			password: this.propertyWidgets[2].value,
			location: this.propertyWidgets[1].value,      // "//localhost:50000/ridw:user=db2admin;password={password};"
			load: dojo.hitch(this, function(response, ioArgs) {
				var _self = this;
				var error = false;
				var i = 0;
				if (response != null) {
					for (i; i < response.length; i++) {
						if (response[i].code === "ERROR") { //$NON-NLS-1$
							this.displayDetailedErrorMessage(this.messages.testConnectionFailed, response[i]);
							jazz.ui.underlay.hide();
							//Check to see if we need to create Tables (i.e. error code 3)
//							if (response[i].id == 3) {
//								
//								break;
//							}
							error = true;
						}
					}
					if(!error){
						this.displaySuccessMessage(this.messages.testConnectionSuccess);
						jazz.ui.underlay.hide();
					}
				}else{
					this.displaySuccessMessage(this.messages.testConnectionSuccess);
				}
			}),
			error: dojo.hitch(this, function(error, ioArgs) {
				this.displayDetailedErrorMessage(this.messages.testConnectionFailed, error);
				jazz.ui.underlay.hide();
			})
		});
	},
	
	toggleEditMode: function (edit) {
		dojo.toggleClass(this._editLink, "hide", edit); //$NON-NLS-1$
		dojo.toggleClass(this._viewLink, "hide", !edit); //$NON-NLS-1$
		for (var i = 0; i < this.propertyWidgets.length; i++) {
			var property = this.propertyWidgets[i];
			if("db.jdbc.password" == property.params.key){ //$NON-NLS-1$
				property.toggleEditMode(edit);
			}
		}
	},
	
	toggleETLEditMode: function (edit) {
		dojo.toggleClass(this._etlEditLink, "hide", edit); //$NON-NLS-1$
		dojo.toggleClass(this._etlViewLink, "hide", !edit); //$NON-NLS-1$
		for (var i = 0; i < this.propertyWidgets.length; i++) {
			var property = this.propertyWidgets[i];
			if("etl.thread.size" == property.params.key || "etl.storage.size" == property.params.key //$NON-NLS-1$ //$NON-NLS-2$
					|| "etl.jdbc.connection.size" == property.params.key || "etl.connection.retrytimes" == property.params.key //$NON-NLS-1$ //$NON-NLS-2$
					|| "etl.connection.timeout" == property.params.key || "etl.queue.wait.timeout" == property.params.key //$NON-NLS-1$ //$NON-NLS-2$
					|| "etl.db2db.sql.batch.size" == property.params.key || "etl.jdbc.connection.timeout" == property.params.key //$NON-NLS-1$ //$NON-NLS-2$
					|| "etl.db2db.consumer.threads.size" == property.params.key) { //$NON-NLS-1$
				property.toggleEditMode(edit);
			}
		}
	},

	_initializeContent: function () {
		DataCollectionClient.getResourceGroupCollection({
			load: dojo.hitch(this, function(data, ioArgs) { //$NON-NLS-1$
				this.resourceGroupNumber = data.length;
				DataCollectionClient.getConfiguration({
					load: dojo.hitch(this, function(data, ioArgs) {
						var _self = this;
						var dbVendor = data["db.vendor"];  //$NON-NLS-1$
						var location = data["db.jdbc.location"];  //$NON-NLS-1$
						var password = data["db.jdbc.password"];  //$NON-NLS-1$
						var threadSize = data["etl.thread.size"];  //$NON-NLS-1$
						var connectionSize = data["etl.jdbc.connection.size"];  //$NON-NLS-1$
						var storageSize = data["etl.storage.size"];  //$NON-NLS-1$
						var retryTimes = data["etl.connection.retrytimes"];  //$NON-NLS-1$
						var timeout = data["etl.connection.timeout"];  //$NON-NLS-1$
						var jdbcTimeout = data["etl.jdbc.connection.timeout"];  //$NON-NLS-1$
						var queueWaitTimeout = data["etl.queue.wait.timeout"];  //$NON-NLS-1$
						var sqlBatchSize = data["etl.db2db.sql.batch.size"];  //$NON-NLS-1$
						var consumerThreadsSize = data["etl.db2db.consumer.threads.size"];  //$NON-NLS-1$
						
						var dbPropertyWidget = new StringProperty({
							"name": this.messages.jdbcVendor,  //$NON-NLS-1$
							"key": "db.vendor",  //$NON-NLS-1$ //$NON-NLS-2$
							"value": dbVendor,  //$NON-NLS-1$
							"textHoverHelp": this.messages.jdbcVendorDescpt,  //$NON-NLS-1$]
							"defaultValue": this.messages.jdbcVendorDefaultValue,  //$NON-NLS-1$
							"readonly": false  //$NON-NLS-1$
						});
						this.propertyWidgets.push(dbPropertyWidget);
						this._propertiesTable.appendChild(dbPropertyWidget.domNode);
						
						var locationPropertyWidget = new StringProperty({
							"name": this.messages.jdbcLocation,  //$NON-NLS-1$
							"key": "db.jdbc.location",  //$NON-NLS-1$ //$NON-NLS-2$
							"value": location,  //$NON-NLS-1$
							"textHoverHelp": this.messages.jdbcLocationDescpt,  //$NON-NLS-1$
							"defaultValue": this.messages.jdbcLocationDefaultValue,  //$NON-NLS-1$
							"readonly": false  //$NON-NLS-1$
						});
						this.propertyWidgets.push(locationPropertyWidget);
						this._propertiesTable.appendChild(locationPropertyWidget.domNode);
						
						var passwordPropertyWidget = new StringProperty({
							"name": this.messages.jdbcPassword,  //$NON-NLS-1$
							"key": "db.jdbc.password",  //$NON-NLS-1$ //$NON-NLS-2$
							"value": password,  //$NON-NLS-1$
							"textHoverHelp": this.messages.jdbcPasswordDescpt,  //$NON-NLS-1$
							"defaultValue": this.messages.jdbcPasswordDefaultValue,  //$NON-NLS-1$
							"readonly": false,  //$NON-NLS-1$
							"onChange": function(value) { //$NON-NLS-1$
								_self.setChange();
							},
							"isPassword": true  //$NON-NLS-1$
						});
						this.propertyWidgets.push(passwordPropertyWidget);
						this._propertiesTable.appendChild(passwordPropertyWidget.domNode);
						
						var threadSizePropertyWidget = new StringProperty({
							"name": this.messages.threadSize,  //$NON-NLS-1$
							"key": "etl.thread.size",  //$NON-NLS-1$ //$NON-NLS-2$
							"value": threadSize,  //$NON-NLS-1$
							"textHoverHelp": this.messages.threadSizeDescpt,  //$NON-NLS-1$
							"defaultValue": this.messages.threadSizeDefaultValue,  //$NON-NLS-1$
							"errorMessage": this.messages.threadSizeErrorMessage,  //$NON-NLS-1$
							"readonly": false,  //$NON-NLS-1$
							"onChange": function(value) { //$NON-NLS-1$
								_self.setChange();
							},
							"validate": function(value) { //$NON-NLS-1$
								var valid = value - 0 == value && value.length > 0 && value >= 1 && value % 1 == 0;
								return valid;
							}
						});
						this.propertyWidgets.push(threadSizePropertyWidget);
						this._etlPropertiesTable.appendChild(threadSizePropertyWidget.domNode);
						
						var connectionSizeWidget = new StringProperty({
							"name": this.messages.connectionSize,  //$NON-NLS-1$
							"key": "etl.jdbc.connection.size",  //$NON-NLS-1$ //$NON-NLS-2$
							"value": connectionSize,  //$NON-NLS-1$
							"textHoverHelp": this.messages.connectionSizeDescpt,  //$NON-NLS-1$
							"defaultValue": this.messages.connectionSizeDefaultValue,  //$NON-NLS-1$
							"errorMessage": dojo.string.substitute(this.messages.connectionSizeErrorMessage, [this.resourceGroupNumber]),  //$NON-NLS-1$
							"readonly": false,  //$NON-NLS-1$
							"onChange": function(value) { //$NON-NLS-1$
								_self.setChange();
							},
							"validate": function(value) { //$NON-NLS-1$
								var valid = value - 0 == value && value.length > 0 && value >= _self.resourceGroupNumber && value % 1 == 0;
								return valid;
							}
						});
						this.propertyWidgets.push(connectionSizeWidget);
						this._etlPropertiesTable.appendChild(connectionSizeWidget.domNode);
						
						var storageSizePropertyWidget = new StringProperty({
							"name": this.messages.storageSize,  //$NON-NLS-1$
							"key": "etl.storage.size",  //$NON-NLS-1$ //$NON-NLS-2$
							"value": storageSize,  //$NON-NLS-1$
							"textHoverHelp": this.messages.storageSizeDescpt,  //$NON-NLS-1$
							"defaultValue": this.messages.storageSizeDefaultValue,  //$NON-NLS-1$
							"errorMessage": this.messages.storageSizeErrorMessage,  //$NON-NLS-1$
							"readonly": false,  //$NON-NLS-1$
							"onChange": function(value) { //$NON-NLS-1$
								_self.setChange();
							},
							"validate": function(value) { //$NON-NLS-1$
								var valid = value - 0 == value && value.length > 0 && value >= 1 && value % 1 == 0;
								return valid;
							}
						});
						this.propertyWidgets.push(storageSizePropertyWidget);
						this._etlPropertiesTable.appendChild(storageSizePropertyWidget.domNode);
						
						var retryTimesPropertyWidget = new StringProperty({
							"name": this.messages.retryTimes,  //$NON-NLS-1$
							"key": "etl.connection.retrytimes",  //$NON-NLS-1$ //$NON-NLS-2$
							"value": retryTimes,  //$NON-NLS-1$
							"textHoverHelp": this.messages.retryTimesDescpt,  //$NON-NLS-1$
							"defaultValue": this.messages.retryTimesDefaultValue,  //$NON-NLS-1$
							"errorMessage": this.messages.retryTimesErrorMessage,  //$NON-NLS-1$
							"readonly": false,  //$NON-NLS-1$
							"onChange": function(value) { //$NON-NLS-1$
								_self.setChange();
							},
							"validate": function(value) { //$NON-NLS-1$
								var valid = value - 0 == value && value.length > 0 && value >= 1 && value % 1 == 0;
								return valid;
							}
						});
						this.propertyWidgets.push(retryTimesPropertyWidget);
						this._etlPropertiesTable.appendChild(retryTimesPropertyWidget.domNode);
						
						var timeoutPropertyWidget = new StringProperty({
							"name": this.messages.timeout,  //$NON-NLS-1$
							"key": "etl.connection.timeout",  //$NON-NLS-1$ //$NON-NLS-2$
							"value": timeout,  //$NON-NLS-1$
							"textHoverHelp": this.messages.timeoutDescpt,  //$NON-NLS-1$
							"defaultValue": this.messages.timeoutDefaultValue,  //$NON-NLS-1$
							"errorMessage": this.messages.timeoutErrorMessage,  //$NON-NLS-1$
							"readonly": false,  //$NON-NLS-1$
							"onChange": function(value) { //$NON-NLS-1$
								_self.setChange();
							},
							"validate": function(value) { //$NON-NLS-1$
								var valid = value - 0 == value && value.length > 0 && value >= 1 && value % 1 == 0;
								return valid;
							}
						});
						this.propertyWidgets.push(timeoutPropertyWidget);
						this._etlPropertiesTable.appendChild(timeoutPropertyWidget.domNode);
						
						var jdbcTimeoutPropertyWidget = new StringProperty({
							"name": this.messages.jdbcTimeout,  //$NON-NLS-1$
							"key": "etl.jdbc.connection.timeout",  //$NON-NLS-1$ //$NON-NLS-2$
							"value": jdbcTimeout,  //$NON-NLS-1$
							"textHoverHelp": this.messages.jdbcTimeoutDescpt,  //$NON-NLS-1$
							"defaultValue": this.messages.jdbcTimeoutDefaultValue,  //$NON-NLS-1$
							"errorMessage": this.messages.jdbcTimeoutErrorMessage,  //$NON-NLS-1$
							"readonly": false,  //$NON-NLS-1$
							"onChange": function(value) { //$NON-NLS-1$
								_self.setChange();
							},
							"validate": function(value) { //$NON-NLS-1$
								var valid = value - 0 == value && value.length > 0 && value >= 1 && value % 1 == 0;
								return valid;
							}
						});
						this.propertyWidgets.push(jdbcTimeoutPropertyWidget);
						this._etlPropertiesTable.appendChild(jdbcTimeoutPropertyWidget.domNode);
						
						var queueWaitTimeoutPropertyWidget = new StringProperty({
							"name": this.messages.queueWaitTimeout,  //$NON-NLS-1$
							"key": "etl.queue.wait.timeout",  //$NON-NLS-1$ //$NON-NLS-2$
							"value": queueWaitTimeout,  //$NON-NLS-1$
							"textHoverHelp": this.messages.queueWaitTimeoutDescpt,  //$NON-NLS-1$
							"defaultValue": this.messages.queueWaitTimeoutDefaultValue,  //$NON-NLS-1$
							"errorMessage": this.messages.queueWaitTimeoutErrorMessage,  //$NON-NLS-1$
							"readonly": false,  //$NON-NLS-1$
							"onChange": function(value) { //$NON-NLS-1$
								_self.setChange();
							},
							"validate": function(value) { //$NON-NLS-1$
								var valid = value - 0 == value && value.length > 0 && value >= 10 && value % 1 == 0;
								return valid;
							}
						});
						this.propertyWidgets.push(queueWaitTimeoutPropertyWidget);
						this._etlPropertiesTable.appendChild(queueWaitTimeoutPropertyWidget.domNode);
						
						var sqlBatchSizePropertyWidget = new StringProperty({
							"name": this.messages.sqlBatchSize,  //$NON-NLS-1$
							"key": "etl.db2db.sql.batch.size",  //$NON-NLS-1$ //$NON-NLS-2$
							"value": sqlBatchSize,  //$NON-NLS-1$
							"textHoverHelp": this.messages.sqlBatchSizeDescpt,  //$NON-NLS-1$
							"defaultValue": this.messages.sqlBatchSizeDefaultValue,  //$NON-NLS-1$
							"errorMessage": this.messages.sqlBatchSizeErrorMessage,  //$NON-NLS-1$
							"readonly": false,  //$NON-NLS-1$
							"onChange": function(value) { //$NON-NLS-1$
								_self.setChange();
							},
							"validate": function(value) { //$NON-NLS-1$
								var valid = value - 0 == value && value.length > 0 && value >= 1 && value % 1 == 0;
								return valid;
							}
						});
						this.propertyWidgets.push(sqlBatchSizePropertyWidget);
						this._etlPropertiesTable.appendChild(sqlBatchSizePropertyWidget.domNode);
						
						this.onLoadingComplete();
					}),
					error: dojo.hitch(this, function(error, ioArgs) {
						this.displayDetailedErrorMessage(this.messages.getConfigurationFailed, error);
						jazz.ui.underlay.hide();
					})
				});
			}),
			error: dojo.hitch(this, function(error, ioArgs) {
				this.displayDetailedErrorMessage(this.messages.getConfigurationFailed, error);
				jazz.ui.underlay.hide();
			})
		});
	},
	
	onLoadingComplete: function () {
		this.hideMessage();
		jazz.ui.underlay.hide();
	},
	
	setChange: function() {
		var isValid = true;
		for (var i = 0; i < this.propertyWidgets.length; i++) {
			if (!this.propertyWidgets[i].isValid()) {
				isValid = false;
				break;
			}
		}
		this._saveButton.disabled = !isValid;
	},
	
	refresh: function() {
		this.hideMessage();
		this.displayInfoMessage(this.messages.loading);
		
		for (var i = 0; i < this.propertyWidgets.length; i++) {
			this.propertyWidgets[i].destroyRecursive();
		}
		this._saveButton.disabled = true;
		this.propertyWidgets = [];
		jazz.ui.underlay.show();
		this._initializeContent();
		dojo.toggleClass(this._editLink, "hide", false); //$NON-NLS-1$
		dojo.toggleClass(this._viewLink, "hide", true); //$NON-NLS-1$
		dojo.toggleClass(this._etlEditLink, "hide", false); //$NON-NLS-1$
		dojo.toggleClass(this._etlViewLink, "hide", true); //$NON-NLS-1$
	},
	
	uninitialize: function(){
		this.inherited(arguments);
	}
});

})();