/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
(function() {
	dojo.provide("com.ibm.rational.datacollection.web.ui.internal.editors.DayTimesScheduleWidget"); //$NON-NLS-1$
	
	dojo.require("com.ibm.rational.datacollection.web.ui.internal.common.DataCollectionConstants"); //$NON-NLS-1$
	dojo.require("com.ibm.rational.datacollection.web.ui.internal.common.Toolbar"); //$NON-NLS-1$

	dojo.require("dojo.i18n"); //$NON-NLS-1$
	dojo.requireLocalization("com.ibm.rational.datacollection.web.ui.internal.editors", "DayTimesScheduleWidgetMessages"); //$NON-NLS-1$ //$NON-NLS-2$
	
	var DataCollectionConstants = com.ibm.rational.datacollection.web.ui.internal.common.DataCollectionConstants;
	var Toolbar = com.ibm.rational.datacollection.web.ui.internal.common.Toolbar;
	
	dojo.declare("com.ibm.rational.datacollection.web.ui.internal.editors.DayTimesScheduleWidget", [dijit._Widget, dijit._Templated], { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.rational.datacollection.web", "ui/internal/editors/templates/DayTimesScheduleWidget.html"), //$NON-NLS-1$ //$NON-NLS-2$
		
		constructor: function(args) {
			this.messages = dojo.i18n.getLocalization("com.ibm.rational.datacollection.web.ui.internal.editors", "DayTimesScheduleWidgetMessages");//$NON-NLS-1$ //$NON-NLS-2$
		},
		
		postCreate: function() {
			this.inherited(arguments);
		},
		
		init: function(schedule) {
			if (schedule.weekDays) {
				for ( var i = 0; i < schedule.weekDays.length; i++) {
					for (var j = 0; j < this._daysSelect.options.length; j++) {
						if (schedule.weekDays[i] === this._daysSelect.options[j].value) {
							this._daysSelect.options[j].selected = true;
						}
					}
				}
			}
			
			if (schedule.times) {
				for ( var i = 0; i < schedule.times.length; i++) {
					this._createRow(schedule.times[i].hour, schedule.times[i].minute);
				}
			}
			
			this._enablement.checked = schedule.enabled;
		},
		
		_onSelectAllDays: function() {
			for ( var i = 0; i < this._daysSelect.options.length; i++ ) {
				this._daysSelect.options[i].selected = true;
			}
		},
		
		_onDeselectAllDays: function() {
			for ( var i = 0; i < this._daysSelect.options.length; i++ ) {
				this._daysSelect.options[i].selected = false;
			}
		},
		
		_createRow: function(hour, minute) {
			var row, time, actions, hourSelect, minuteSelect, toolbar;
			row = dojo.create("tr"); //$NON-NLS-1$

			time = dojo.create("td"); //$NON-NLS-1$
			actions = dojo.create("td");
			
			hourSelect = dojo.create("select"); //$NON-NLS-1$
			for (var i = 0; i < 24; i++) {
				var option = dojo.create("option"); //$NON-NLS-1$
				option.value = i;
				if (i < 10) {
					option.appendChild(dojo.doc.createTextNode("0" + i)); //$NON-NLS-1$
				} else {
					option.appendChild(dojo.doc.createTextNode(i));
				}
				hourSelect.appendChild(option);
				if (hour && hour === i) {
					hourSelect.selectedIndex = i;
				}
			}
			time.appendChild(hourSelect);
			
			minuteSelect = dojo.create("select"); //$NON-NLS-1$
			for (var i = 0; i < 60; i++) {
				var option = dojo.create("option"); //$NON-NLS-1$
				option.value = i;
				if (i < 10) {
					option.appendChild(dojo.doc.createTextNode("0" + i)); //$NON-NLS-1$
				} else {
					option.appendChild(dojo.doc.createTextNode(i));
				}
				minuteSelect.appendChild(option);
				if (minute && minute === i) {
					minuteSelect.selectedIndex = i;
				}
			}
			time.appendChild(minuteSelect);
			
			toolbar = new Toolbar();
			toolbar.addButton(this, 
				dojo.hitch(this, function() {
					for (var i = 0; i < this._timesTable.rows.length; i++) {
						if (this._timesTable.rows[i] === row) {
							this._timesTable.deleteRow(i);
							return;
						}
					}
				}),
				{
					label: this.messages.removeTime,
					imgSrc: dojo.moduleUrl("com.ibm.rational.datacollection.web","graphics/icons/remove.gif") //$NON-NLS-1$
				}
			);						
			dojo.addClass(toolbar.domNode, "table-actions"); //$NON-NLS-1$
//			toolbar.domNode.style.visibility = "hidden";
//			row.onmouseover = dojo.hitch(this, function(e) {
//				toolbar.domNode.style.visibility = "visible";
//			});
//			row.onmouseout = dojo.hitch(this, function(e) {
//				toolbar.domNode.style.visibility = "hidden";
//			});
			actions.appendChild(toolbar.domNode);
			
			row.appendChild(time);
			row.appendChild(actions);
			
			row.hour = hourSelect;
			row.minute = minuteSelect;
			
			this._timesTable.appendChild(row);
		},
		
		_onCreateTime: function() {
			this._createRow();
		},
		
		getSchedule: function(originalSchedule) {
			var weekDays = [];
			for ( var i = 0; i < this._daysSelect.options.length; i++ ) {
				if (this._daysSelect.options[i].selected) {
					weekDays.push(this._daysSelect.options[i].value);
				}
			}
			var times = [];
			for (var i = 0; i < this._timesTable.rows.length; i++) {
				var row = this._timesTable.rows[i];
				if (row.hour.value && row.minute.value) {
					times.push({
						hour: parseInt(row.hour.value),
						minute: parseInt(row.minute.value)
					});
				}
			}
			var schedule = {
				identifier: DataCollectionConstants.NS_DAILY_SCHEDULE,
				enabled: this._enablement.checked,
				weekDays: weekDays,
				times: times
			};
			return this._schedulesEqual(schedule, originalSchedule) ? originalSchedule : schedule;
		},
		
		_schedulesEqual: function(s1, s2) {
			if (!s1 && !s2) {
				return true;
			}
			
			if ((s1 && !s2) || (!s1 && s2)) {
				return false;
			}
			
			if (s1 && s2 && s1.identifier === s2.identifier 
					&& s1.enabled === s2.enabled) {
				if (!s1.weekDays && !s2.weekDays) {
					return true;
				}
				
				if (s1.weekDays && s2.weekDays && s1.weekDays.length === s2.weekDays.length) {
					for (var i = 0; i < s1.weekDays.length; i++) {
						var found = false;
						for (var j = 0; j < s2.weekDays.length && !found; j++) {
							found = s1.weekDays[i] === s2.weekDays[j];
						}
						if (!found) {
							return false;
						}
					}
					if (s1.times && s2.times && s1.times.length === s2.times.length) {
						for (var i = 0; i < s1.times.length; i++) {
							var found = false;
							for (var j = 0; j < s2.times.length && !found; j++) {
								found = s1.times[i].hour === s2.times[j].hour 
									&& s1.times[i].minute === s2.times[j].minute;
							}
							if (!found) {
								return false;
							}
						}
						return true;
					}
				}
			}
			return false;
		},
		
		validate: function() {
		}
				
	});
	
})();
