/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
(function() {
	dojo.provide("com.ibm.rational.datacollection.web.ui.internal.editors.IntervalScheduleWidget"); //$NON-NLS-1$
	
	dojo.require("com.ibm.rational.datacollection.web.ui.internal.common.DataCollectionConstants"); //$NON-NLS-1$

	dojo.require("dojo.i18n"); //$NON-NLS-1$
	dojo.requireLocalization("com.ibm.rational.datacollection.web.ui.internal.editors", "IntervalScheduleWidgetMessages"); //$NON-NLS-1$ //$NON-NLS-2$
	
	var DataCollectionConstants = com.ibm.rational.datacollection.web.ui.internal.common.DataCollectionConstants;
	
	dojo.declare("com.ibm.rational.datacollection.web.ui.internal.editors.IntervalScheduleWidget", [dijit._Widget, dijit._Templated], { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.rational.datacollection.web", "ui/internal/editors/templates/IntervalScheduleWidget.html"), //$NON-NLS-1$ //$NON-NLS-2$
		
		constructor: function(args) {
			this.messages = dojo.i18n.getLocalization("com.ibm.rational.datacollection.web.ui.internal.editors", "IntervalScheduleWidgetMessages");//$NON-NLS-1$ //$NON-NLS-2$
		},
		
		postCreate: function() {
			this.inherited(arguments);
		},
		
		init: function(schedule) {
			if (schedule.interval) {
				this._intervalInput.value = schedule.interval;
			}
			
			if (schedule.weekDays) {
				for ( var i = 0; i < schedule.weekDays.length; i++) {
					for (var j = 0; j < this._daysSelect.options.length; j++) {
						if (schedule.weekDays[i] === this._daysSelect.options[j].value) {
							this._daysSelect.options[j].selected = true;
						}
					}
				}
			}
			
			this._enablement.checked = schedule.enabled;
		},
		
		_onSelectAllDays: function() {
			for ( var i = 0; i < this._daysSelect.options.length; i++ ) {
				this._daysSelect.options[i].selected = true;
			}
		},
		
		_onDeselectAllDays: function() {
			for ( var i = 0; i < this._daysSelect.options.length; i++ ) {
				this._daysSelect.options[i].selected = false;
			}
		},
		
		getSchedule: function(originalSchedule) {
			var interval = parseInt(this._intervalInput.value);
			var weekDays = [];
			for ( var i = 0; i < this._daysSelect.options.length; i++ ) {
				if (this._daysSelect.options[i].selected) {
					weekDays.push(this._daysSelect.options[i].value);
				}
			}
			var schedule = {
				identifier: DataCollectionConstants.NS_INTERVAL_SCHEDULE,
				enabled: this._enablement.checked,
				interval: interval,
				weekDays: weekDays
			};
			return this._schedulesEqual(schedule, originalSchedule) ? originalSchedule : schedule; 
		},
		
		_schedulesEqual: function(s1, s2) {
			if (!s1 && !s2) {
				return true;
			}
			
			if ((s1 && !s2) || (!s1 && s2)) {
				return false;
			}
			
			if (s1 && s2 && s1.identifier === s2.identifier 
					&& s1.interval === s2.interval
					&& s1.enabled === s2.enabled) {
				if (!s1.weekDays && !s2.weekDays) {
					return true;
				}
				
				if (s1.weekDays && s2.weekDays && s1.weekDays.length === s2.weekDays.length) {
					for (var i = 0; i < s1.weekDays.length; i++) {
						var found = false;
						for (var j = 0; j < s2.weekDays.length && !found; j++) {
							found = s1.weekDays[i] === s2.weekDays[j];
						}
						if (!found) {
							return false;
						}
					}
					return true;
				}
			}
			return false;
		},
		
		validate: function() {
			var interval = parseInt(this._intervalInput.value);
			if (!interval || interval <= 0) {
				return this.messages.invalidInterval;
			}
		}
				
	});
	
})();
