/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.rational.datacollection.web.ui.internal.editors.StringProperty"); //$NON-NLS-1$

dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("com.ibm.rational.datacollection.web.ui.internal.editors.AbstractProperty"); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require("jazz.app.ua.TextHoverHelp"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.rational.datacollection.web.ui.internal.editors", "StringPropertyMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function() {

var AbstractProperty = com.ibm.rational.datacollection.web.ui.internal.editors.AbstractProperty;
var TextHoverHelp = jazz.app.ua.TextHoverHelp;

dojo.declare("com.ibm.rational.datacollection.web.ui.internal.editors.StringProperty", [AbstractProperty], { //$NON-NLS-1$

	templatePath: dojo.moduleUrl("com.ibm.rational.datacollection.web", "ui/internal/editors/templates/StringProperty.html"), //$NON-NLS-2$ //$NON-NLS-1$
	key: "", //$NON-NLS-1$
	value: "", //$NON-NLS-1$
	name: "", //$NON-NLS-1$
	defaultValue: "", //$NON-NLS-1$
	textHoverHelp: "", //$NON-NLS-1$
	isPassword: false,
	readonly: false,
	// widget initialization
	
	constructor: function() {
		this._errIconScr = dojo.moduleUrl("com.ibm.rational.datacollection.web", "graphics/icons/inline-error.gif"); //$NON-NLS-2$ //$NON-NLS-1$
		this.messages = dojo.i18n.getLocalization("com.ibm.rational.datacollection.web.ui.internal.editors", "StringPropertyMessages"); //$NON-NLS-2$ //$NON-NLS-1$
	},

	postCreate: function () {
		this.connect(this.editValue, "onchange", function(e) { //$NON-NLS-1$
			this.value = this.editValue.value;
			//If it's password then should replace it with "*"
			if(this.isPassword){
				this.viewValue.innerHTML = this.disguisePassword(this.value); 
			}else{
				this.viewValue.innerHTML = this.editValue.value;
			}
		});
		
		this.connect(this.passwordValue, "onchange", function(e) { //$NON-NLS-1$
			this.value = this.passwordValue.value;
			//If it's password then should replace it with "*"
			if(this.isPassword){
				this.viewValue.innerHTML = this.disguisePassword(this.value); 
			}else{
				this.viewValue.innerHTML = this.editValue.value;
			}
		});
		
		this._initializeContent();
	},

	_initializeContent: function () {
		this.viewValue.innerHTML = this.value;
		this.prepareTextHoverHelp();
		if(this.isPassword){
			this.passwordValue.value = this.value;
			this.viewValue.innerHTML = this.disguisePassword(this.value); 
		}else{
			this.editValue.value = this.value;
		}
	},
	
	prepareTextHoverHelp: function (){
		if(this.textHoverHelp){
			new TextHoverHelp({
				helpContentString: this.textHoverHelp,
				contentNode: this._propertyName
			}); 
		}
	},
	
	disguisePassword: function(password) {
		var hiddenPassword = ""; //$NON-NLS-1$
		for(var i=0;i< password.length; i++){
			hiddenPassword = hiddenPassword + "*"; //$NON-NLS-1$
		}
		return hiddenPassword;
	},
	
	toggleEditMode: function (edit) {
		if (edit && !this.readonly) {
			if(this.isPassword){
				dojo.style(this.passwordValue, "display", ""); //$NON-NLS-1$  //$NON-NLS-2$
			}else{
				dojo.style(this.editValue, "display", ""); //$NON-NLS-1$  //$NON-NLS-2$
			}
			dojo.style(this.viewValue, "display", "none"); //$NON-NLS-1$  //$NON-NLS-2$
		} else {
			if(this.isPassword){
				dojo.style(this.passwordValue, "display", "none"); //$NON-NLS-1$  //$NON-NLS-2$
			}else{
				dojo.style(this.editValue, "display", "none"); //$NON-NLS-1$  //$NON-NLS-2$
			}
			dojo.style(this.viewValue, "display", ""); //$NON-NLS-1$  //$NON-NLS-2$
		}
		
	},
	
	onChange: function (value) {
	},
	
	_handleKeyUp : function(e) {
		if (e.target && e.target.tagName === "INPUT") { //$NON-NLS-1$
			if (this.onKeyUpTimout) {
				clearTimeout(this.onKeyUpTimout);
			}
			var f = dojo.hitch(this, function() {
				this._checkValidity(e.target.value);
			});
			var timeout = 500;
	
			this.onKeyUpTimout = setTimeout(f, timeout);
		}
	},
	
	_checkValidity : function(value) {
		if (this.validate(value) === true) {
			this._markValid();
		} else {
			this._markInvalid();
		}
		
		this.onChange(value);
	},
	
	validate : function(value) {
		return true;
	},
	
	isValid : function(){
		return !this._invalid;
	},
	
	_markValid : function() {
		while (this._invalidValueMsg.childNodes.length > 0) {
			this._invalidValueMsg.removeChild(this._invalidValueMsg.firstChild);
		}
		this._errIcon.style.display = ""; //$NON-NLS-1$
		this._errIcon.style.visibility = "hidden"; //$NON-NLS-1$
		this._invalid = false;
	},
	
	_markInvalid : function() {
		this._invalid = true;
		while (this._invalidValueMsg.childNodes.length > 0) {
			this._invalidValueMsg.removeChild(this._invalidValueMsg.firstChild);
		}
	
		if (this.errorMessage) {
			this._invalidValueMsg.appendChild(document.createTextNode(this.errorMessage));
		}
		this._errIcon.style.visibility = "visible"; //$NON-NLS-1$
	},
	
	uninitialize: function(){
		this.inherited(arguments);
	}
});

})();