/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.rational.datacollection.web.ui.internal.pages.DataCollectionManagementPage"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("jazz.ui.SideNavigationLayout"); //$NON-NLS-1$
dojo.require("net.jazz.ajax.ui.PlatformUI"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.ui.internal.admin.NotAuthorizedView"); //$NON-NLS-1$
dojo.require("com.ibm.rational.datacollection.web.ui.internal.editors.ConfigureResourceGroup"); //$NON-NLS-1$
dojo.require("com.ibm.rational.datacollection.web.ui.internal.editors.ConfigureETLJob"); //$NON-NLS-1$
dojo.require("com.ibm.rational.datacollection.web.ui.internal.editors.ConfigureDWConnection"); //$NON-NLS-1$
dojo.require("com.ibm.rational.datacollection.web.ui.internal.editors.ConfigureDeltaLoad"); //$NON-NLS-1$
dojo.require("com.ibm.rational.datacollection.web.ui.internal.editors.DataCollectionJobStatus"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.rational.datacollection.web.ui.internal.pages", "DataCollectionManagementPageMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function(){ // module container
    var PlatformUI = net.jazz.ajax.ui.PlatformUI;
    var ActionRegistry = null;
    var PREFIX = "com.ibm.rational.datacollection.manageResources."; //$NON-NLS-1$
    var SERVER_PACKAGE = "com.ibm.rational.datacollection.web.ui.internal.editors."; //$NON-NLS-1$
    var ACTION_MAP = {};
    
    var ACTION_SETS = [{
        labelKey: 'dataCollection', //$NON-NLS-1$
        actions: [{
            labelKey: 'dataCollectionJobsHeading', //$NON-NLS-1$
            shortId: "configureETLJob", //$NON-NLS-1$
            widget: "ConfigureETLJob" //$NON-NLS-1$
        },{
            labelKey: 'dataCollectionJobStatusHeading', //$NON-NLS-1$
            shortId: "dataCollectionJobStatus", //$NON-NLS-1$
            widget: "DataCollectionJobStatus" //$NON-NLS-1$
        }]
    },{
        labelKey: 'configuration', //$NON-NLS-1$
        actions: [{
            labelKey: 'dataWarehouseHeading', //$NON-NLS-1$
            shortId: "configureDWConnection", //$NON-NLS-1$
            widget: "ConfigureDWConnection" //$NON-NLS-1$
        }, {
            labelKey: 'resourceGroupHeading', //$NON-NLS-1$
            shortId: "configureResourceGroup", //$NON-NLS-1$
            widget: "ConfigureResourceGroup" //$NON-NLS-1$
        }, {
            labelKey: 'deltaLoadHeading', //$NON-NLS-1$
            shortId: "configureDeltaLoad", //$NON-NLS-1$
            widget: "ConfigureDeltaLoad" //$NON-NLS-1$
        }]
    }];

    dojo.declare("com.ibm.rational.datacollection.web.ui.internal.pages.DataCollectionManagementPage", dijit._Widget, { //$NON-NLS-1$
    
        constructor: function(){
            this.messages = dojo.i18n.getLocalization("com.ibm.rational.datacollection.web.ui.internal.pages", "DataCollectionManagementPageMessages"); //$NON-NLS-1$ //$NON-NLS-2$
        },
        
        reloadCurrent: function(){
            this._previousActionEntry = this._currentActionEntry || null;
            this._showCurrentWidget(/*isReload*/true);
        },
        
        postCreate: function(){
            // call superclass's postCreate method
            this.inherited(arguments);
            
            dojo.addClass(this.domNode, "com-ibm-team-data-collection-service-content-area"); //$NON-NLS-1$
            
            // initialize widget cache
            this._widgetCache = {};
            
            // instantiate the side-nav layout UI and append it to the page
            this._appView = new jazz.ui.SideNavigationLayout();
            this.domNode.appendChild(this._appView.domNode);
            
            // register actions and create side-nav hyperlinks
            ActionRegistry = getWorkbench().getActionRegistry();
            ActionRegistry.registerAction(PREFIX + "_defaultAction", this, "_defaultAction"); //$NON-NLS-1$ //$NON-NLS-2$
            
            var self = this;
            
            this.allLinks = [];
            dojo.forEach(ACTION_SETS, function(actionSet){
                self._createNavSection(actionSet);
            });
         },
        
        _createNavSection: function(actionSet){
            var self = this;
            var navSection = this._appView.addNavigationSection(document.createElement("div"), this.messages[actionSet.labelKey], true); //$NON-NLS-1$
            dojo.forEach(actionSet.actions, function(action){
                var id = PREFIX + action.shortId;
                var link = navSection.addSectionRow(self.messages[action.labelKey], "#action=" + id); //$NON-NLS-1$
                link.__jazzActionId = action.shortId;
                self.allLinks.push(link);
                navSection.addArtifactCommand(link, self.messages.refresh, dojo.hitch(self, self._onRefreshClick), "admin-refresh-command"); //$NON-NLS-1$
                self._registerAction(action);
            });
            
        },
        
        _registerAction: function(action){
            var id = PREFIX + action.shortId;
            ACTION_MAP[action.shortId] = action;
            ActionRegistry.registerAction(id, this, "_handleAction"); //$NON-NLS-1$
        },
        
        _handleAction: function(args){
            if (!isAdmin()) {
            	this._showUnauthorized();
                return;
            }
            
            if (!args.action) {
                console.debug("ServerPage.js#_handleAction: missing required param 'args.action'"); //$NON-NLS-1$
                return;
            }
            
            // cache the previous entry to see if the widget should be destroyed or
            // not
            this._previousActionEntry = this._currentActionEntry || null;
            
            // parse out the short id from the fully-qualified id
            var shortId = args.action.substring(args.action.lastIndexOf(".") + 1); //$NON-NLS-1$
            this._currentActionEntry = ACTION_MAP[shortId];
            this._currentArgs = args;
            this._currentArgs.parent = this;
            
            // set the title for the action
            getWorkbench().setTitle(this.messages[this._currentActionEntry.labelKey]);
            
            // highlight the selected link and unhighlight the others
            dojo.forEach(this.allLinks, function(link){
                if (link.__jazzActionId === shortId) 
                    dojo.addClass(link, "selected"); //$NON-NLS-1$
                else 
                    dojo.removeClass(link, "selected"); //$NON-NLS-1$
            });
            
            // call the actual method implementation
            this._showCurrentWidget();
        },
        
        _onRefreshClick: function(e){
            dojo.stopEvent(e);
            this.reloadCurrent();
        },
        
        _showCurrentWidget: function(isReload){
            // save a reference to the prior content widget
            this._previousContentWidget = this._currentContentWidget || null;
            
            if (this._currentActionEntry.cached) {
                this._showCachedWidget(isReload);
            }
            else {
                this._showUncachedWidget();
            }
            
            // help the garbage collector do its job
            if (this._previousContentWidget && !this._previousActionEntry.cached) {
                this._previousContentWidget.destroyRecursive();
            }
        },
        
        _showUncachedWidget: function(){
            var constructor = dojo.getObject(SERVER_PACKAGE + this._currentActionEntry.widget);
            this._currentContentWidget = new constructor(this._currentArgs);
            this._appView.setMainContent(this._currentContentWidget.domNode);
        },
        
        _defaultAction: function(){
            // by default, show the first section from the left-nav
            this._handleAction({
                action: PREFIX + ACTION_SETS[0].actions[0].shortId
            });
        },
    	
    	_showUnauthorized: function(){
    		dojox.xml.parser.removeChildren(this.domNode);
    		if (!this._notAuth) {
    			this._notAuth = new com.ibm.team.repository.web.ui.internal.admin.NotAuthorizedView();
    		}	
    		this.domNode.appendChild(this._notAuth.domNode);
    	}
        
    });
    
    function getWorkbench(){
        return net.jazz.ajax.ui.PlatformUI.getWorkbench();
    }
    
    function isAdmin(){
        return com.ibm.team.repository.web.noAdminAuth ? false : true;
    }
})();
