/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2014.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.attributes.OverviewSection");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit._base");
dojo.require("dojo.i18n");
dojo.require("dijit.form.Form");
dojo.require("dijit.layout.ContentPane");

dojo.require("com.ibm.rdm.web.attributes.AttributeGroupSection");
dojo.require("com.ibm.rdm.web.common.UserHover");
dojo.require("com.ibm.rdm.web.project.EditManager");
dojo.require("com.ibm.rdm.web.common.ProjectCache");
dojo.require("com.ibm.rdm.web.server.client.utils.UserCache");
dojo.require("com.ibm.rdm.web.notifications.ChangeNotifier");
dojo.require("com.ibm.rdm.web.server.client.utils.TeamAreaUtil");
dojo.require("com.ibm.rdm.web.server.cache.ProcessOperationsCache");
dojo.require("com.ibm.rdm.web.picker.ProcessAreaPickerForm");
dojo.require("com.ibm.rdm.web.picker.MoveFolderPicker"); 
dojo.require("com.ibm.rdm.web.common.ArtifactIcons");
dojo.require("com.ibm.rdm.web.common.ResourceLinksHandler");
dojo.require("com.ibm.rdm.web.common.MimeTypes");
dojo.require("com.ibm.rdm.web.model.types.RRMTypes");
dojo.require("com.ibm.rdm.web.util.bidiutils"); //bidi
//ACGC_Bidi_start date support for non-gregorian calendars
dojo.require("dojox.date.islamic.Date");
dojo.require("dojox.date.islamic.locale");
//ACGC_Bidi_End
dojo.require("com.ibm.rdm.web.model.types.TypesManager");
dojo.require("com.ibm.rdm.web.suspicion.SuspectLinksUtil");
dojo.require("com.ibm.rdm.web.suspicion.ClearSuspicionButton");
dojo.require("com.ibm.rdm.web.suspicion.SuspicionStatus");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.require("com.ibm.rdm.web.artifactgrid.ArtifactPermissionsCache");


dojo.requireLocalization("com.ibm.rdm.web.attributes", "OverviewSection");
dojo.requireLocalization("com.ibm.rdm.web.common", "common");


var domutils = com.ibm.rdm.web.util.domutils;
var ChangeNotifier = com.ibm.rdm.web.notifications.ChangeNotifier;
var ProcessAreaPickerForm = com.ibm.rdm.web.picker.ProcessAreaPickerForm;
var ProcessOperationsCache = com.ibm.rdm.web.server.cache.ProcessOperationsCache;
var ProjectCache = com.ibm.rdm.web.common.ProjectCache;
var TeamAreaUtil = com.ibm.rdm.web.server.client.utils.TeamAreaUtil;
var UserCache = com.ibm.rdm.web.server.client.utils.UserCache;
var RRMTypes = com.ibm.rdm.web.model.types.RRMTypes;
var BidiUtils = com.ibm.rdm.web.util.bidiutils; //bidi
var TypesManager = com.ibm.rdm.web.model.types.TypesManager;
var SuspectLinksUtil = com.ibm.rdm.web.suspicion.SuspectLinksUtil;
var EditManager = com.ibm.rdm.web.project.EditManager;


dojo.declare(
	"com.ibm.rdm.web.attributes.OverviewSection",
	[dijit._Widget, dijit._Templated],
{
	// summary:
	//		This class is responsible for the Overview section that appears
	//		in the sidebar.
	// description:
	//		This section displays high level artifact information such as the
	//		name, creation date, modified by, etc. as well as all attribute
	//		group settings for this artifact.

	templatePath: dojo.moduleUrl("com.ibm.rdm.web", "attributes/templates/OverviewSection.html"),
	widgetsInTemplate: true,

	///////////////////////////////////////////////////////////////////////////////////
	// Public fields

	isModule: null,
	resourceURI: null,
	contentType: null,
	disableAttributeWidgets: false,
	resource: null,

	///////////////////////////////////////////////////////////////////////////////////
	// NLS fields requiring translation

	project: null,
	createdOn: null,
	createdBy: null,
	modifiedOn: null,
	modifiedBy: null,

	///////////////////////////////////////////////////////////////////////////////////
	// Private fields

	_messages: null,
	_commonMessages: null,
	_createdUserHover: null,
	_modifiedUserHover: null,
	_resource: null,
	_attributeGroupSection: null,
	_teamOwnershipFieldValue: null,
	_moduleLocationFieldValue: null,
	_teamOwnershipTextBox: null,
	_processAreaPickerForm: null,
	_projectName: null,

	///////////////////////////////////////////////////////////////////////////////////
	// Attach points

	_contentPane: null,
	_documentNameSpan: null,
	_projectView: null,
	_teamOwnershipGroup: null,
	_teamOwnershipView: null,
	_setTeamOwnershipButton: null,
	_setModuleLocationButton: null,
	_createdOnView: null,
	_createdByView: null,
	_modifiedOnView: null,
	_modifiedByView: null,
	_suspicionView: null,
	_attributeGroupsDiv: null,
	_form: null,
	_descriptionView: null,
	_descriptionInput: null,
	
	// Interface to container
	// callbacks.  An object containing methods setTitle, requestActivate, and isActive
	//		 	setTitle: a method that takes a string and sets the title.
	_setTitle: null,
	//			requestActivate: called to request that this widget is activated.
	_requestActivate: null,

	_lastSelectedResourceURI: null,
	
	//This widget should be activated by default.
	activateOnLoad: true,
	
	//suspicion
	_noSuspicion: null,
	_selectProfile: null,
	_selectProfileHover: null,
	_suspicionStatusWidget: null,
	_clearSuspicionButton: null,
	_suspicionEntriesListener: null,
	_suspicionProfileListener: null,

	///////////////////////////////////////////////////////////////////////////
	// Public API

	initialize: function() {
		this._contentPane.startup();
	},
	
	startup: function(){
		if (this.resource){
			this.load(this.resource);
		}
		this._setTitle(this._messages.overview);
		this._addButtons();
	},
	
	postCreate: function(){
		var resourceUriScope = this.resourceURI;
		this._active = false;
		if (this.isForSelection) {
			//Don't limit the listening scope when we could be for any artifact in a module
			resourceUriScope = null;
			//And don't make this active initially
			this.activateOnLoad = false;
		}
		this.editModeListener = ChangeNotifier.addListener({
			eventTopics: [com.ibm.rdm.web.notifications.EventTopics.ARTIFACT_EDIT_MODE],
			callbackFunction: dojo.hitch(this, function(params){
				if (!this._active || (params.notifierParams.failedSave && this._selectionChangedSinceEdit)) {
					//Ignore if we are not active, or the selection has already changed since save issued
					return;
				}
				if (// Ignore if this OverviewSection is for the "Selected Artifact" tab in a module
						// and the user has just returned after having clicked through a row and performed
						// an edit for the row's associated modulefact.
						// Note: the 'this.isForSelection' condition is only true for the OverviewSection
						// used by the "Selected Artifact" tab (and without checking for this we'd be
						// inappropriately disabling editing for the "Module" tab as well [see Defect 93181]).
						this.isModule && this.isForSelection && !params.notifierParams.isForSelection
						// Also ignore if we've just navigated back to a previously opened modulefact
						// page after the user has just used the "Selected Artifact" tab in a module to
						// perform an edit for this modulefact (see Defect 93181 Comment 3 for details).
						|| !this.isModule && params.notifierParams.isForSelection)
				{
					return;
				}
				var isViewOnlyMode = false;
				this._loadDescriptionField(isViewOnlyMode);
				if (!this.isForSelection) {
					//Don't offer these fields for edit on selections in modules
					this._loadTeamOwnershipField(isViewOnlyMode);
					this._loadAssetLocationField(isViewOnlyMode);
				}
				dojo.addClass(this.domNode, 'underEdit');
				if (this.isForSelection) {
					this._startDescriptionEdit();
				}
			}),
			callbackScope: this,
			resourceUriScope: resourceUriScope
		}); 
		
		this.connect(
			this._teamOwnershipTextBox,
			"onclick",
			dojo.hitch(this, function(event){
				if (!this._setTeamOwnershipButton.get("disabled")) {
					this._onChangeTeamOwnershipClicked(event);
				}
			})
		);
		this.connect(this._moduleLocationTextBox, "onclick", this._onChangeModuleLocationClicked);
		
		this.connect(this._moduleLocationTextBox, "onmouseover", function () {
			dojo.addClass(this._moduleLocationTextBox, "dijitTextBoxHover");
		});
		
		this.connect(this._moduleLocationTextBox, "onmouseout", function () {
			dojo.removeClass(this._moduleLocationTextBox, "dijitTextBoxHover");
		});
		
		
		this.connect(this._teamOwnershipTextBox, "onmouseover", function () {
			if (!this._setTeamOwnershipButton.get("disabled")) {
				dojo.addClass(this._teamOwnershipTextBox, "dijitTextBoxHover");
			}
		});
		
		this.connect(this._teamOwnershipTextBox, "onmouseout", function () {
			if (!this._setTeamOwnershipButton.get("disabled")) {
				dojo.removeClass(this._teamOwnershipTextBox, "dijitTextBoxHover");
			}
		});
			
		this.viewModeListener = ChangeNotifier.addListener({
			eventTopics: [com.ibm.rdm.web.notifications.EventTopics.ARTIFACT_VIEW_MODE],
			callbackFunction: dojo.hitch(this, function(params){
				if (!this._active) {
					return;
				}
				var isViewOnlyMode = true;
				if (this.isForSelection && this._descriptionEditInProgress) {
					//Changing selection should save
					this._saveAllEdits();
				}
				this._loadDescriptionField(isViewOnlyMode);
				this._loadTeamOwnershipField(isViewOnlyMode);
				this._loadAssetLocationField(isViewOnlyMode);
				dojo.removeClass(this.domNode, 'underEdit');
			}),
			callbackScope: this,
			resourceUriScope: resourceUriScope
		}); 

		this.saveModeListener = ChangeNotifier.addListener({
			eventTopics: [com.ibm.rdm.web.notifications.EventTopics.ARTIFACT_SAVED],
			callbackFunction: dojo.hitch(this, function(params){
				if (this.isModule) {
					var resources = params.notifierParams.resources;
					if (resources.length) {
						var resource = resources[0];
						if (resource.resourceURI === this._lastSelectedResourceURI) {
							this._loadSelectedResource(
								resource.projectURI, resource.resourceURI,
								resource.revision, resource.projectSnapshotURI
							);
						}
					}
				}
			}),
			callbackScope: this,
			resourceUriScope: resourceUriScope
		}); 
		
		if (this.artifactSuspicionModel) {
			this._createSuspicionElements();
			this._suspicionEntriesListener = dojo.subscribe(
					SuspectLinksUtil.EVENT_TOPICS.entriesChanged, this, this._updateEntries);
			this._suspicionProfileListener = dojo.subscribe(
					SuspectLinksUtil.EVENT_TOPICS.profilesChanged, this, this._handleProfilesChanged);
		}
		this._setTeamOwnershipButton.set({
			'title': this._commonMessages.setTeamOwnership,
			'label': this._commonMessages.setTeamOwnership
		});
		this._setModuleLocationButton.set({
			'title': this._messages.setAssetLocationTooltip,
			'label': this._messages.setAssetLocationTooltip
		});
	},	
	
	postMixInProperties: function() {
		this.inherited(arguments);
		this._commonMessages = dojo.i18n.getLocalization("com.ibm.rdm.web.common", "common");
		this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.attributes", "OverviewSection");
		dojo.forEach(["project", "createdOn", "createdBy", "modifiedOn", "modifiedBy", "doors", "loadingAttributes"], function(prop){
			this[prop] = this._messages[prop];
		}, this);
		
		//Remember if we are really in a module
		this._actuallyInModule = this.isModule;
	},
	
	/*
	 * Methods added to satisfy needs of resource interaction
	 */
	_addButtons: function(){
		var buttons = this._getOverviewButtons();
		
		if (buttons){
			dojo.place(buttons, this._lifecycleStrip, "only");
		}
	},
	
	// ========== subView lifecycle methods ============
	
	setContainerEnvironment: function(params) {
		// summary:
		//		Called by our container to set up callbacks.
		// params:
		//		An object with the following fields:
		//		 	setTitle: a method that takes a string and sets the title.
		//			requestActivate: called to request that this widget is activated.
		this._setTitle = params.setTitle;
		this._requestActivate = params.requestActivate;
	},
	
	activate: function() {
		// summary:
		//		Called by container when this sub-view is to be activated.

		this._active = true;
	},
	
	deactivate: function() {
		// summary:
		//		Called by container when this sub-view is to be deactivated.
		//		At the moment we don't do anything when deactivated, but we could
		//		probably avoid doing screen updates other than for title.

		this._active = false;
	},
	
			
	isExpanded: function() {
		return this._active;
	},
	
	handleRefresh: function() {
		this.loadModifiedInfo();
	},
	
	setSelection: function(resource) {
		this._selectionChangedSinceEdit = true;
		if (this._descriptionEditInProgress) {
			this._saveAllEdits();
		}
		if (this._isStub(resource.resourceURI)) {
			this.setNoSelection();
			return;
		}

		this._lastSelectedResourceURI = resource.resourceURI;

		this._updateEditButtonStyles();
		this._editButton && dojo.attr(this._editButton, "disabled", true);

		this._showBodyMessage(this._messages.loadingSelectionMessage);
		this.resourceURI = resource.resourceURI;

		this._loadSelectedResource(resource.projectURI, resource.resourceURI, resource.revision, resource.projectSnapshotURI);
	},

	_loadSelectedResource: function(projectURI, resourceURI, revision, baselineURI, skipCache) {
		return com.ibm.rdm.web.model.resources.ResourceManager.get({
			resourceURI: resourceURI,
			projectURI: projectURI,
			revision: revision,
			projectSnapshotURI: baselineURI,
			forceGetDescribe: true,
			skipCache: skipCache
		}, dojo.hitch(this, function(resource) {
			this.load(resource);
		}));
	},	
	
	setNoSelection: function() {
		this._lastSelectedResourceURI = null;
		this._selectionChangedSinceEdit = true;
		if (this._descriptionEditInProgress) {
			this._saveAllEdits();
		}
		this._showNoButtons();
		this._showBodyMessage(this._messages.noSelectionMessage);
	},
	
	setMultiSelection: function() {
		this._lastSelectedResourceURI = null;
		this._selectionChangedSinceEdit = true;
		if (this._descriptionEditInProgress) {
			this._saveAllEdits();
		}
		this._showNoButtons();
		this._showBodyMessage(this._messages.multiSelectionMessage);
	},
	
	handleArtifactContentChanged: function() {
		// summary:
		//		This artifact content has changed, refresh the widget content to reflect any changes.
		
		this.handleRefresh();
	},
	
	containerDimensionsHaveChanged: function() {
		var height = this.domNode.parentNode.clientHeight;
		var width = this.domNode.parentNode.clientWidth;
		var contentPaneTop = (
			this._contentPane.domNode.offsetTop
			- this._contentPane.domNode.parentNode.offsetTop);
		var scrollHeightToUse = (height - contentPaneTop);
		this._contentPane.resize({ h: scrollHeightToUse, w: width });
	},

	// =========== end subView lifecycle methods ============

	
	load: function(resource) {
		this._resource = resource;
		this._updateNameField(resource.getDisplayTitle());
		this.updateDescriptionField(resource.getDescription());
		this.setDescription(resource.getDescription());
		
		this._descriptionUri = RRMTypes.getWellKnownAttributeURI(resource.getProjectURI(), RRMTypes.WELL_KNOWN_ATTRIBUTE_NAMES.Description);

		var isReview =
			resource.getMimeType() === com.ibm.rdm.web.common.MimeTypes.CONTENT_TYPE_REVIEW;
		
		var isModule = resource.getMimeType() === RRMTypes.getArtifactFormats(resource.getProjectURI())["Module"];
		
		if (this._resource.isInModule && this._resource.isInModule()) {
			dojo.removeClass(this._moduleGroup, "hidden");
			this._updateModuleField();
		}
		
		// If it's a review, we need to NOT show the "Team Ownership" field.
		if (isReview) {
			dojo.addClass(this._teamOwnershipGroup, "hidden");
		}
		
		if(!isModule){
			//Do not show the asset location field if this artifact is not a module
			dojo.addClass(this._moduleLocationGroup, "hidden");
		}
		
		var projectURI = resource.getProjectURI();
		ProjectCache.getProject(projectURI).addCallback(
			this,
			function(project) {
				this._projectName = project.name;
				this._updateProjectField(project.name);
				// Don't bother attempting to update the "Team Ownership" field
				// if the artifact being viewed is a review.
				if (!isReview) {
					this._updateTeamOwnershipField(
						this._resource.getProcessAreaURI() || projectURI
					);
				}
				if(isModule){
					//Get the asset folder URI and update the text field
					if(this._resource.getAssetFolderURI()){
						this.updateAssetLocationField(this._resource.getAssetFolderURI());
					}					
				}
			}
		);
		
		this._createdInfo(resource);
		this.loadModifiedInfo();
		
		if (resource.getTypes) {
			this.originalTypesArray = [];		
			for (var objectTypeURI in resource.getTypes()) {
				this.originalTypesArray.push(objectTypeURI);
			}
			this.originalType = resource.getTypes();	
			this.loadAttributesSection(resource);
		}
		
		if (this.artifactSuspicionModel) {
			this._loadSuspicion();
		}
		else {
			dojo.addClass(this._suspicionGroup, "hidden");
		}
		
		if (this.isForSelection) {
			//Ensure if we are a secondary display, we show ourselves
			this._clearBodyMessage();
			
			//Enable the edit button
			this._getModifyArtifactPermissions(resource).then(
				dojo.hitch(this, function(params) {
					if (params._isModifyArtifactAllowed) {
						this._editButton && dojo.attr(this._editButton, "disabled", false);
					}
				})
			);
		}

		// The following is a fix for Defect 91768 (to avoid showing a gap at the top
		// when NOT in a module).
		if (!this.isModule) {
			dojo.style(this._infoMsgNode, "display", "none");
		}
	},
	
	_getModifyArtifactPermissions: function(resource) {
		// summary:
		//		Returns a deferred which, when fired, will call you back with a
		//		single argument which will be an instance of
		//		ModifyArtifactPermissions.  This callback argument enables you
		//		to inquire about whether the artifact associated with this
		//		class is editable and, for all of its attributes, which of these
		//		attributes are editable.
		var deferred = new dojo.Deferred();
		var projectURI = resource.getProjectURI();
		var processAreaURI = resource.getProcessAreaURI();
		
		var typeURI = resource.getType();
		var objectTypeDeferred = TypesManager.getObjectType({
			typeURI: typeURI,
			projectURI: projectURI
		});
		objectTypeDeferred.addCallback(this, function(objectType){
			var artifactTypeActionId = objectType.getActionId();
			var getModifyArtifactPermissions =
				ProcessOperationsCache.getModifyArtifactPermissions(
					artifactTypeActionId, processAreaURI, projectURI
				);
			getModifyArtifactPermissions.addCallback(
				this,
				function(modifyArtifactPermissions){
					deferred.callback(modifyArtifactPermissions);
					return modifyArtifactPermissions;
				}
			);
		});
		return deferred;
	},	
	
	_saveAllEdits: function() {
	
		this._descriptionEditInProgress = false;
		EditManager.saveAll(this._resource.getProjectURI());
		this._updateEditButtonStyles();
	},
	
	_cancelAllEdits: function() {

		this._descriptionEditInProgress = false;
		EditManager.cancelAll(this._resource.getProjectURI());
		this._updateEditButtonStyles();
		ChangeNotifier.notifyListeners({
			eventTopic : com.ibm.rdm.web.notifications.EventTopics.ARTIFACT_VIEW_MODE,
			resourceURI : this.resourceURI,
			projectURI : this._resource.getProjectURI(),
			resources : [ this._resource ]
		});	
		//Reload resource
		this._loadSelectedResource(this._resource.projectURI, this._resource.resourceURI, this._resource.revision, this._resource.projectSnapshotURI, true);
	},	
	
	_loadDescriptionField: function(isViewOnlyMode){
		var description = this.description ? this.description : "";
		dojo.place(document.createTextNode(description), this._descriptionView, 'only');
		BidiUtils.applyTextDirSetting(this._descriptionView, description);//bidi

		this._descriptionInput.textbox.value = description;
		BidiUtils.applyTextDirSetting(this._descriptionInput, description);//bidi
		this.connect(this._descriptionInput, "onChange", dojo.hitch(this, this.onDescriptionChanged));
		this.connect(this._descriptionInput, "onKeyPress", dojo.hitch(this, this._handleKeyPress));

		if (!isViewOnlyMode) {
			// We're in Edit mode ... but we only want to use an editable field
			// for the description if we know that the user is allowed to
			// modify the Description field (as specified by this artifact's
			// process area).
			var projectURI = this._resource.getProjectURI();
			var objectTypeUris = this._resource.getTypes();
			// Note: It's always the case that there's a single property found
			// in objectTypeUris ...
			for (var objectTypeURI in objectTypeUris) {
				var objectTypeDeferred = TypesManager.getObjectType({
					typeURI: objectTypeURI,
					projectURI: projectURI,
					projectSnapshotURI: this._resource.getSnapshotURI()
				});
				objectTypeDeferred.addCallback(this, function(objectType){
					var getModifyDeferred = ProcessOperationsCache.getModifyArtifactPermissions(
						objectType.getActionId(),
						this._resource.getProcessAreaURI() || projectURI,
						projectURI
					);
					getModifyDeferred.addCallback(this, function(modifyArtifactPermissions){
						var isModifyDescriptionAllowed =
							modifyArtifactPermissions.isModifyDescriptionAllowed();
						this._descriptionInput.set('disabled', !isModifyDescriptionAllowed);
					});
				});
			}
		}
	},
	
	_startDescriptionEdit: function() {
		//This starts an edit of the local Description field - this only happens when we are looking at module artifacts
		if (this._descriptionEditInProgress) {
			return;
		}
		this._descriptionEditInProgress = true;
		EditManager.startEdit(this._resource.getProjectURI(), this.resourceURI, this._descriptionUri, dojo.hitch(this, this._cancelAllEdits));	
		
		this._descriptionWidget = { //Simulate the expected Widget API
			isDirty: dojo.hitch(this, this.isDesciptionDirty),
			isValid: function() {return true;},
			get: function() {}
		};
		
		EditManager.getEditor({
			projectURI: this._resource.getProjectURI(),
			artifactURI: this.resourceURI,
			lockingURI: this._resource.getBoundArtifact(),
			definitionURI: this._descriptionUri,
			callback: dojo.hitch(this, function (node) {}),
			resource: this._resource,
			existingEditor: this._descriptionWidget,
			isForSelection: this.isForSelection
		});
		
		this._updateEditButtonStyles();
	},
	
	isDesciptionDirty: function() {
		return true;
	},
	
	_showBodyMessage: function(message) {
		dojo.style(this._infoMsgNode, "display", ""); 
		dojo.style(this._artifactDetails, "display", "none"); 
		dojo.style(this._form, "display", "none"); 
		this._attributeGroupSection.hide();
		
		dojo.attr(this._infoMsgNode, {innerHTML: "<span class='overviewInfoMsg'>"+message+"</span>"});
	},
	
	_clearBodyMessage: function() {
		dojo.style(this._infoMsgNode, "display", "none"); 
		dojo.style(this._artifactDetails, "display", ""); 
		dojo.style(this._form, "display", ""); 	
		this._attributeGroupSection.show();
	},	

	_setTeamOwnershipEnablementState: function(/*Boolean*/showAsEnabled) {
		// Appropriately enable or disable the button.
		this._setTeamOwnershipButton.set("disabled", !showAsEnabled);
		// Now appropriately style the text box that appears to the left of the button.
		if (showAsEnabled) {
			// Make the text area appear as editable.
			dojo.removeClass(this._teamOwnershipTextBox, "fieldUneditableViaChooser");
			dojo.addClass(this._teamOwnershipTextBox, "fieldEditableViaChooser");
		} else {
			// Make the text area appear as uneditable.
			dojo.removeClass(this._teamOwnershipTextBox, "fieldEditableViaChooser");
			dojo.addClass(this._teamOwnershipTextBox, "fieldUneditableViaChooser");
		}
	},
	
	
	_isStub: function(artifactURI) {
		return artifactURI.indexOf("STUB_") === 0;
	},
	
	_loadTeamOwnershipField: function(isViewOnlyMode) {
		// summary:
		//		Toggle which presentation is used for displaying the value of
		//		the "Team Ownership" field (either a span or a div containing
		//		a text box and a button) based on whether we're in "view" or
		//		"edit" mode.
		if (!isViewOnlyMode) {
			// We're in Edit mode.  Disable the button for setting the
			// Team Ownership if conditions warrant.
			// Initially we'll set the the button to disabled ...
			this._setTeamOwnershipEnablementState(false);
			
			this._updateTeamOwnershipButtonLabel();
			
			//bidi
			BidiUtils.applyTextDirSetting(this._teamOwnershipTextBox,  this._teamOwnershipFieldValue);
			
			var projectURI = this._resource.getProjectURI();
			var processAreaURI = (this._resource.getProcessAreaURI() || projectURI);
			var projectOperationsDeferred =
				ProcessOperationsCache.isSaveTeamSecuritySpecificationAllowed(
					processAreaURI, projectURI
				);
			projectOperationsDeferred.addCallback(
				this,
				function(isSaveTeamSecuritySpecificationAllowed){
					// We now know the appropriate enablement value.
					// Set the button to appropriately be displayed
					// as enabled or disabled.
					this._setTeamOwnershipEnablementState(isSaveTeamSecuritySpecificationAllowed);
				}
			);
		}
	},
	
	_loadAssetLocationField: function(isViewOnlyMode) {
		// summary:
		//		Toggle which presentation is used for displaying the value of
		//		the "Asset Location" field (either a span or a div containing
		//		a text box and a button) based on whether we're in "view" or
		//		"edit" mode.
		
		//Do not load the asset location field if the current artifact is not a Module
		var isModule = (
			this._resource.getMimeType()
				=== RRMTypes.getArtifactFormats(this._resource.getProjectURI())["Module"]
		);
		
		if (!isModule){
			return;
		}
		
		if (!isViewOnlyMode) {
			// We're in Edit mode.  Disable the button for setting the
			// Asset Location if conditions warrant.
			// Initially we'll set the the button to disabled ...
			this._setModuleLocationButton.set(
				"disabled", true	
			);
			
			this._updateModuleLocationButtonLabel();
			
			var projectURI = this._resource.getProjectURI();
			var objectTypeUris = this._resource.getTypes();
			// Note: It's always the case that there's a single property found
			// in objectTypeUris ...
			for (var objectTypeURI in objectTypeUris) {
				var objectTypeDeferred = TypesManager.getObjectType({
					typeURI: objectTypeURI,
					projectURI: projectURI,
					projectSnapshotURI: this._resource.getSnapshotURI()
				});
				objectTypeDeferred.addCallback(this, function(objectType){
					var getModifyDeferred = ProcessOperationsCache.getModifyArtifactPermissions(
						objectType.getActionId(),
						this._resource.getProcessAreaURI() || projectURI,
						projectURI
					);
					getModifyDeferred.addCallback(this, function(modifyArtifactPermissions){
						// We now know the appropriate enablement value.
						// Set the button to appropriately be displayed
						// as enabled or disabled.
						this._setModuleLocationButton.set(
							"disabled", !modifyArtifactPermissions.isModifyArtifactAllowed()
						);
						return modifyArtifactPermissions;
					});
					return objectType;
				});
			}
		}
	},

	onDescriptionChanged: function(newDescription) { 
		//	connect to receive/broadcast a notification when the description has been modified
		//  callback
		this.updateDescriptionField(this._descriptionInput.textbox.value);
		if (this.isForSelection && this._descriptionEditInProgress) {
			EditManager.setValue(this._descriptionWidget,this._resource.getProjectURI(), this.resourceURI, this._descriptionUri, 
				this.description);
		}
	},

	onTeamOwnershipChanged: function(processAreaURI, projectOrTeamName, isArchived) {
		// summary:
		//		This callback is invoked whenever the user changes the Team Ownership
		//		field.
		// tags:
		//		callback
	},
	
	//Allows JAWS to read current value of setTeamOwnershipButton to JAWS user.
	_updateTeamOwnershipButtonLabel: function() {
		var baseLabel = this._commonMessages.setTeamOwnership;
		var newLabel = baseLabel + " " + this._commonMessages.currently + " " + this._teamOwnershipFieldValue;
		this._setTeamOwnershipButton.set("label", newLabel);
	},
	
	onAssetLocationChanged: function(processAreaURI, projectOrTeamName) {
		// summary:
		//		This callback is invoked whenever the user changes the Asset location
		//		field.
		// tags:
		//		callback
		this._updateModuleLocationButtonLabel();
	},
	
	//Allows JAWS to read current value of setModuleLocationButton to JAWS user.
	_updateModuleLocationButtonLabel: function() {
		var baseLabel = this._messages.setAssetLocationTooltip;
		var newLabel = baseLabel + " " + this._commonMessages.currently + " " + this._moduleLocationFieldValue;
		this._setModuleLocationButton.set("label", newLabel);
	},
	
	onSaveAttributes: function(){
		//	connect to receive a notification when the user has pressed CTRL + S on any attribute
		//  callback
		if (this.isForSelection) {
			this.onDoneButton();
		}
	},
	
	onDoneAttributes: function(){
		//	connect to receive a notification when the user has pressed CTRL + SHIFT + S on any attribute
		//  callback
		if (this.isForSelection) {
			this.onDoneButton();
		}
	},
	
	onCancelAttributes: function(){
		//	connect to receive a notification when the user has pressed ESC on any attribute
		//  callback
		if (this.isForSelection) {
			this.onCancelButton();
		}
	},
	
	loadAttributesSection: function(resource){		
		this._destroyAttributeGroupSection();
		var objectTypeURIs = resource.getTypes();
		for (var objectTypeURI in objectTypeURIs) {
			this._attributeGroupSection = new com.ibm.rdm.web.attributes.AttributeGroupSection({
				resource: resource,
				objectTypeURI: objectTypeURI,
				onChange: this.onChange,
				isForSelection: this.isForSelection,
				onKeyPress: dojo.hitch(this, this._handleKeyPress),
				disableAttributeWidgets: this.disableAttributeWidgets,
				overviewSection: this                
			});
			this._attributeGroupsDiv.appendChild(this._attributeGroupSection.domNode);				
		}		
	},
	
	loadModifiedInfo: function(){
		this._updateNameField(this._resource.getDisplayTitle());
		this._updateModifiedDateField(this._resource.getRevisionDate());
		var authorUri = this._resource.getLastModifiedBy();
		if (authorUri) {
			UserCache.getUser(this._resource.getLastModifiedBy()).addCallback(this, 
				function(user) {
					var authorUri = this._resource.getLastModifiedBy();
					if (!user) {
						// Handle the case of the "rrs-daemon" user.  This user isn't
						// actually found in the project user listing.  However, when
						// displaying a review you might find that the user that last
						// modified the review is "rrs-daemon" (as the R/A service
						// sometimes moves the review from one state to another).
						var usersPrefix = com.ibm.rdm.web.FRONTING_SERVICE_JFS_USERS;
						user = {
							uri: authorUri,
							name: authorUri.substring(usersPrefix.length + 1),
							userId: user.name
						};
					}
					this._updateModifiedByField(user);
				}
			);
		}
	},
	
	_loadSuspicion: function() {
		this.artifactSuspicionModel.getModelObject().then(
			dojo.hitch(this, function(modelObject) {
				this._updateSuspicion(modelObject);
				return modelObject;
			})
		);
	},
	
	reloadSuspicion: function() {
		this.artifactSuspicionModel.load().then(
			dojo.hitch(this, function(modelObject) {
				this._updateSuspicion(modelObject);
				return modelObject;
			})
		);
	},
	
	_updateSuspicion: function(modelObject) {
		this._updateEntries({modelObject: modelObject}, true);
		if (this._clearSuspicionButton) {
			this._clearSuspicionButton.destroyRecursive();
		}
		this._clearSuspicionButton = new com.ibm.rdm.web.suspicion.ClearSuspicionButton({
			source: SuspectLinksUtil.EVENT_SOURCE.overview,
			suspicionModel: this.artifactSuspicionModel,
			projectURI: this._resource.getProjectURI()
		});
		this._updateSuspicionNode(this._clearSuspicionButton.domNode, true);
	},
	
	_updateEntries: function(publishedObject, firstLoad) {
		var projectURI = this._resource.getProjectURI();
		var projectContext = publishedObject.projectContext;
		
		if (firstLoad || projectContext === projectURI) {
			var modelObject = publishedObject.modelObject;
			var suspicionEntries = modelObject.suspicionEntries;
			var suspicionProfileURIs = modelObject.suspicionProfileURIs;
			
			if (suspicionProfileURIs.length > 0) {
				if (suspicionEntries) {
					var isSuspect = false;
					for (var profileUri in suspicionEntries) {
						var profileEntries = suspicionEntries[profileUri];
						profileEntries = dojo.filter(profileEntries, function(entry) {
							return entry.targetUri == this.resourceURI;
						}, this);
						if (profileEntries.length > 0) {
							isSuspect = true;
						}
					}
					if (isSuspect) {
						this._suspicionStatusWidget = new com.ibm.rdm.web.suspicion.SuspicionStatus({
							source: SuspectLinksUtil.EVENT_SOURCE.overview,
							projectURI: modelObject.projectURI,
							suspectEntries: suspicionEntries,
							targetResourceURI: this.resourceURI,
							artifactSuspicionModel: this.artifactSuspicionModel
						});
						this._updateSuspicionNode(this._suspicionStatusWidget.domNode);
					}
					else {
						this._updateSuspicionNode(this._noSuspicion);
					}
				}
				else {
					this._updateSuspicionNode(this._noSuspicion);
				}
			}
			else {
				this._updateSuspicionNode(this._selectProfile);
			}
		}
	},
	
	_handleProfilesChanged: function(publishedObject) {
		var projectURI = this._resource.getProjectURI();
		var projectContext = publishedObject.projectContext;
		
		if (projectContext === projectURI) {
			var source = publishedObject.source;
			var suspicionProfiles = publishedObject.suspicionProfiles;
			var suspicionProfileURIs = [];
			
			dojo.forEach(suspicionProfiles, function(suspicionProfile) {
				if (suspicionProfile.checked) {
					suspicionProfileURIs.push(suspicionProfile.uri);
				}
			}, this);
			this.artifactSuspicionModel.refreshEntries(suspicionProfileURIs, source, projectURI);
		}
	},
	
	_updateSuspicionNode: function(node, isClearSuspicionButton) {
		var pos;
		var refNode;
		if (this._suspicionView) {
			if (this._suspicionView.childNodes.length === 2) {
				pos = "replace";
				refNode = (!isClearSuspicionButton) ? this._suspicionView.childNodes[0] :
					this._suspicionView.childNodes[1];
			} else {
				pos = (!isClearSuspicionButton) ? "first" : "last";
				refNode = this._suspicionView;
			}
			dojo.place(node, refNode, pos);
		}
	},
	
	_createSuspicionElements: function() {
		this._selectProfile = dojo.create("span", {
			innerHTML: this._messages.selectProfile,
			"class": "suspicion-status"
		});
		this._noSuspicion = dojo.create("span", {
			innerHTML: this._messages.no,
			"class": "suspicion-status"
		});
		this._selectProfileHover = new com.ibm.rdm.web.common.HoverPopup({
			around: this._selectProfile,
			destroyContents: false,
			createContents: dojo.hitch(this, function(pop) {
				var div = dojo.create("div");
				var imgSrc = dojo.moduleUrl("com.ibm.rdm.web", "suspicion/images/suspect-link-profiles.gif").toString();
				dojo.create("span", {innerHTML: this._messages.clickSelectProfile + "  "}, div);
				dojo.create("img", {
					src: imgSrc,
					style: {
						position: "relative",
						top: "3px"
					}
				}, div);
				dojo.create("span", {innerHTML: "  " + this._messages.toChooseProfile}, div);
				return div;
			})
		});
	},
	
	_createdInfo: function(resource){
		this._updateCreatedDateField(resource.getCreatedDate());
		this._createdByInfo(resource);
	},

	_createdByInfo: function(resource) {
		var authorUri = "";
		if (resource && resource.getCreatedBy) {
			authorUri = resource.getCreatedBy();
		}
		//Fall back on last modified by if necessary (reviews)
		else if (resource && resource.getLastModifiedBy){
			authorUri = resource.getLastModifiedBy();
		}
		if (authorUri) {
			UserCache.getUser(authorUri).addCallback(this, function(user){
				if (!user) {
					// Handle the case of the "rrs-daemon" user.  This user isn't
					// actually found in the project user listing.  However, when
					// displaying a review you might find that the user that last
					// modified the review is "rrs-daemon" (as the R/A service
					// sometimes moves the review from one state to another).
					var usersPrefix = com.ibm.rdm.web.FRONTING_SERVICE_JFS_USERS;
					user = {};
					user.uri = authorUri;
					user.name = authorUri.substring(usersPrefix.length + 1);
					user.userId = user.name;
				}
				this._updateCreatedByField(user);
			});
		}
	},

	isValid: function(){
		return this._attributeGroupSection.isValid();
	},
	
	_updateProjectField: function(projectName) {
		this._updateField(this._projectView, projectName);
	},

	updateTeamOwnershipField: function(processAreaURI, /*String?*/projectOrTeamName) {
		this._processAreaURI = processAreaURI;
		this._projectOrTeamName = projectOrTeamName;
		this._updateTeamOwnershipField(processAreaURI, projectOrTeamName);
	},
	
	updateAssetLocationField: function(folderURI){
		// summary:
		//		Update the value shown for the "Asset Location" field.	
		var self= this;
		var projectURI = this._resource.getProjectURI();
		var updateDisplayedModuleLocationValue = function(moduleLocationLabelToUse){
			// Update the field used for view mode.
			self._updateField(self._moduleLocationView, moduleLocationLabelToUse);
			// Update the field used for edit mode.
			self._moduleLocationTextBox.innerHTML = moduleLocationLabelToUse;
			// Store the new field value away ...
			self._moduleLocationFieldValue = moduleLocationLabelToUse;
		};
		
		//Get name of folder from folderURI
		ProjectCache.getFolder(folderURI).addBoth(this, function(folderData){
			if (projectURI == folderData.folder.parent.xmlAttributes.resource) {
				var projectDeferred = ProjectCache.getProject(projectURI).addBoth(this, function(projectObj) {
					updateDisplayedModuleLocationValue(projectObj.name);
				});				
			}
			else {
				updateDisplayedModuleLocationValue(folderData.folder.title.value);
			}
		});	
	},

	updateDocumentFields: function(docName, createdDate, modifiedDate, modifiedBy) {
		this._updateField(this._documentNameSpan, docName);
		var artifactId = this._resource.getID();
		var artifactIdText = null;
		if (artifactId !== null) {
			artifactIdText = artifactId + ":";
		} else {
			artifactIdText = "";
		}
		this._updateField(this._documentIdSpan, artifactIdText);
		this._updateField(this._createdOnView, this._composeShortDate(createdDate));
		this._updateField(this._modifiedOnView, this._composeShortDate(modifiedDate));
		this._updateField(this._modifiedByView, modifiedBy);
	},


	_updateCreatedDateField: function(createdDate) {
		this._updateField(this._createdOnView, this._composeShortDate(createdDate));
	},
	
	_updateCreatedByField: function(user) {
		this._updateField(this._createdByView, user.name);
		this._removeCreatedUserHover();
		this._createCreatedUserHover(user, this._createdByView);
	},

	_updateModuleField: function() {
		com.ibm.rdm.web.model.resources.ResourceManager.get({
			baselineURI: this._resource.getSnapshotURI(),
			reviewURI: this.viewer.resource.reviewURI,
			resourceURI: this._resource.getModuleURI()
		}).then(dojo.hitch(this, function (moduleResource) {
			domutils.removeChildNodes(this._moduleUrl);
			this._moduleIcon.className = com.ibm.rdm.web.common.ArtifactIcons.getSmallImageClass(moduleResource.getProjectURI(), moduleResource.getType(), moduleResource.getMimeType());
			this._moduleUrl.appendChild(com.ibm.rdm.web.common.ResourceLinksHandler.createLink({
				uri: this._resource.getModuleURI(),
				baselineURI: this._resource.getSnapshotURI(),
				label: moduleResource.getID() + ": " + moduleResource.getTitle()
			}));
		}));
	},
	
	_getOverviewButtons: function() {
		var buttonStrip = dojo.create("span");
		if (this.isForSelection) {
			//These are the buttons for the Artifact attributes overview - this
			//section manages changes to individual artifacts
			if (!this.viewer.isInHistoryMode() && !this.viewer.isInReviewMode() && this.viewer.userCanEdit()){
				dojo.style(buttonStrip, "display", "inline-block");

				this._saveDoneCancelStrip = dojo.create("span",
						{style:
							{
								"display": "none"
							}
						}, buttonStrip);

				this._editButton = dojo.create("button", {
					innerHTML: this.viewer._abstractMessages.edit,
					"class": "editButton",
					"disabled": true
				});
				this.connect(this._editButton, "onclick", dojo.hitch(this, this.onEditButton));

				this._doneButton = dojo.create("button", {
					innerHTML:this.viewer._abstractMessages.done
				}, this._saveDoneCancelStrip);
				this.connect(this._doneButton, "onclick", dojo.hitch(this, this.onDoneButton));

				this._cancelButton = dojo.create("button", {
					innerHTML: this.viewer._abstractMessages.cancel
				}, this._saveDoneCancelStrip);
				this.connect(this._cancelButton, "onclick", dojo.hitch(this, this.onCancelButton));

				buttonStrip.appendChild(this._editButton);
			}
			return buttonStrip;		
		} else {
			if (this._actuallyInModule) {
				//This is buttons for the Module attributes overview - do it same as it was
				//which means making calls into our viewer
				return this.viewer.getDelegatedLifecycleItems();
			}
		}
		//If we get here, we are neither of the overview sections in a module, and not
		//used by ModuleArtifactViewer - so the original buttons are there (and the overview section has none)
	},
	
	_updateNameField: function(docName) {
		if(this._resource.hasServerGeneratedTitle && this._resource.hasServerGeneratedTitle()) {
			docName = dojo.string.substitute(this._messages.generatedTitleIndicator, [docName]);
			dojo.addClass(this._documentNameSpan, "generatedTitle");
		} else {
			dojo.removeClass(this._documentNameSpan, "generatedTitle");
		}
		
		this._updateField(this._documentNameSpan, docName);
		var artifactId = this._resource.getID();
		var artifactNameToUse = null;
		if (artifactId !== null && artifactId.length > 0) {
			artifactNameToUse = artifactId + ":";
		} else {
			artifactNameToUse = "";
		}
		this._updateField(this._documentIdSpan, artifactNameToUse);
	},
	
	_updateEditButtonStyles: function() {
		if (this._descriptionEditInProgress) {
			this._editButton && dojo.style(this._editButton, "display", "none");
			this._saveDoneCancelStrip && dojo.style(this._saveDoneCancelStrip, "display", "");
		} else {
			this._editButton && dojo.style(this._editButton, "display", "");
			this._saveDoneCancelStrip && dojo.style(this._saveDoneCancelStrip, "display", "none");
		}
	},
	
	_showNoButtons: function() {
		this._editButton && dojo.style(this._editButton, "display", "none");
		this._saveDoneCancelStrip && dojo.style(this._saveDoneCancelStrip, "display", "none");
	},
	
	onEditButton: function() {
		this._selectionChangedSinceEdit = false;
		ChangeNotifier.notifyListeners({
			eventTopic : com.ibm.rdm.web.notifications.EventTopics.ARTIFACT_EDIT_MODE,
			resourceURI : this.resourceURI,
			resourceUriScope: this.resourceURI,
			projectURI : this._resource.getProjectURI(),
			resources : [ this._resource ],
			isForSelection : true,		//Ensure the grid doesn't respond to this.
			lockingURI: this._resource.getBoundArtifact()
		});
	},
	
	onCancelButton: function() {
		this._cancelAllEdits();
	},
	
	onDoneButton: function() {
		this._saveAllEdits();
	},
	
	updateDescriptionField: function(description){	
		this.description = description;
		
		if (BidiUtils.getTextDir() === BidiUtils.TEXT_DIR_CONTEXTUAL && this._descriptionInput) { //bidi,for editing with contextual text direction
			BidiUtils.applyTextDirSetting(this._descriptionInput.focusNode,  description);
		}		
	},
	
	setDescription: function(description){
		if (!description){
			description = " ";
		}
		this._updateField(this._descriptionView, description);
	},

	_updateModifiedDateField: function(modifiedDate) {
		this._updateField(this._modifiedOnView, this._composeShortDate(modifiedDate));
	},

	_updateModifiedByField: function(user) {
		this._updateField(this._modifiedByView, user.name);
		this._removeModifiedUserHover();
		this._createModifiedUserHover(user, this._modifiedByView);
	},

	destroy: function(){
		this.inherited(arguments);
		ChangeNotifier.removeListener(this.editModeListener);
		ChangeNotifier.removeListener(this.viewModeListener);
		ChangeNotifier.removeListener(this.saveModeListener);
		
		if (this._createdUserHover){
			this._createdUserHover.destroyRecursive();
		}
		if (this._modifiedUserHover){
			this._modifiedUserHover.destroyRecursive();
		}
		this._destroyAttributeGroupSection();
		this._removeModifiedUserHover();
		this._removeCreatedUserHover();
		if (this.descriptionTextBox) {
			this.descriptionTextBox.destroyRecursive();
		}
		if (this._processAreaPickerForm) {
			this._processAreaPickerForm.destroyRecursive();
		}
		if (this._suspicionStatusWidget) {
			this._suspicionStatusWidget.destroyRecursive();
		}
		if (this.artifactSuspicionModel) {
			dojo.destroy(this._selectProfile);
			dojo.destroy(this._noSuspicion);
			this._selectProfileHover.destroyRecursive();
			this._clearSuspicionButton.destroyRecursive();
			dojo.unsubscribe(this._suspicionEntriesListener);
			dojo.unsubscribe(this._suspicionProfileListener);
			this.artifactSuspicionModel.destroy();
		}
		// Note this._teamOwnershipTextBox is loaded via template code so
		// we don't need to destroy it here.
	},

	///////////////////////////////////////////////////////////////////////////
	// Private API

	_onChangeTeamOwnershipClicked: function(event) {
		// summary:
		//		The user has clicked on the button in the "Team Ownership"
		//		field ... display a dialog for selecting a new value for
		//		this field.
		var projectURI = this._resource.getProjectURI();
		ProjectCache.getProject(projectURI).addCallback(
			this,
			function(project) {
				var projectName = project.name;
				var teamAreasParserDeferred =
					TeamAreaUtil.getOrCreateTeamAreasParser(projectURI);
				teamAreasParserDeferred.addCallback(this, function(teamAreasParser){
					this._showProcessAreaPicker(
						teamAreasParser, projectURI, projectName
					);
					return teamAreasParser;
				});
			}
		);
	},
	
	_onChangeModuleLocationClicked: function(event) {
		// summary:
		//		The user has clicked on the button in the module location
		//		field ... display a dialog for selecting a new location for this field
		var setAssetFolderURI = function(folderURI){
			//Invoke onAssetLocationChanged to indicate the editor that a change has taken place 
			this.onAssetLocationChanged(folderURI);
		};
		var projectURI = this._resource.getProjectURI();
		var picker = new com.ibm.rdm.web.picker.MoveFolderPicker({
			projectURI: projectURI,
			dialogTitle: this._messages.chooseAssetLocation,
			onGetMoveResult:dojo.hitch(this, setAssetFolderURI),
			currentFolderURI: this.viewer.resource.getParentResourceURI()
		});

		picker.show();
		
	},

	_showProcessAreaPicker: function(teamAreasParser, projectURI, projectName) {
		// summary:
		//		Display a dialog for selecting the new value that should be used
		//		for the "Team Ownership" field.
		var processAreaURI = (this._resource.getProcessAreaURI() || projectURI);
		if (!this._processAreaPickerForm) {
			this._processAreaPickerForm =
				new ProcessAreaPickerForm({
					projectURI: projectURI,
					projectName: projectName,
					teamAreasParser: teamAreasParser,
					selectProjectOrTeamAreaURI: processAreaURI,
					onOk: dojo.hitch(this, function(){
						var selectedProcessAreaProperties =
							this._processAreaPickerForm.getSelectedProcessAreaProperties();
						this.onTeamOwnershipChanged(
							selectedProcessAreaProperties.projectOrTeamAreaURI,
							selectedProcessAreaProperties.projectOrTeamAreaName,
							selectedProcessAreaProperties.isArchived
						);
						this._updateTeamOwnershipButtonLabel();
					})
				});
		}
		this._processAreaPickerForm.set("selectProjectOrTeamAreaURI", processAreaURI);
		this._processAreaPickerForm.show();
	},
	
	_normalizeProcessArea: function( processArea ) { 
		if ( processArea != null && processArea.indexOf( com.ibm.rdm.web.FRONTING_SERVICE_JFS_OAUTH_DOMAIN ) != -1 ) {
			// may be old pre-JAF-migrated JTS URI
			processArea = processArea.replace( com.ibm.rdm.web.FRONTING_SERVICE_JFS_OAUTH_DOMAIN,
				net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL );			
		}
	
		return processArea;
	},
	

	_updateTeamOwnershipField: function(processAreaURI, /*String?*/projectOrTeamName) {
		// summary:
		//		Update the value shown for the "Team Ownership" field.
		var projectURI = this._resource.getProjectURI();
		var updateDisplayedTeamOwnershipValue = dojo.hitch(this, function(teamOwnershipLabelToUse){
			// Update the field used for view mode.
			this._updateField(this._teamOwnershipView, teamOwnershipLabelToUse);
			// Update the field used for edit mode.
			//this._teamOwnershipTextBox.set("value", teamOwnershipLabelToUse);
			this._teamOwnershipTextBox.innerHTML = teamOwnershipLabelToUse;
			// Store the new field value away ...
			this._teamOwnershipFieldValue = teamOwnershipLabelToUse;
			//bidi
			BidiUtils.applyTextDirSetting(this._teamOwnershipTextBox,  teamOwnershipLabelToUse);				
		});
		if (!projectOrTeamName && this._normalizeProcessArea(processAreaURI) !== projectURI) {
			// This artifact's processArea is set to a Team Area.
			var teamAreasParserDeferred =
				TeamAreaUtil.getOrCreateTeamAreasParser(projectURI);
			teamAreasParserDeferred.addCallback(this, function(teamAreasParser){
				var teamAreaProperties =
					teamAreasParser.getTeamAreaProperties(processAreaURI);
				var encodedTeamAreaName =
					jazz.util.html.escape(teamAreaProperties.teamAreaName);
				if (teamAreaProperties.isArchived) {
					var labelToUse =
						dojo.string.substitute(this._commonMessages["archivedTeamAreaLabel"], [encodedTeamAreaName]);
					updateDisplayedTeamOwnershipValue(labelToUse);
				} else {
					updateDisplayedTeamOwnershipValue(encodedTeamAreaName);
				}
				return teamAreasParser;
			});
		} else {
			// This artifact's processArea is not set to a Team Area.
			updateDisplayedTeamOwnershipValue(projectOrTeamName || this._projectName);
		}
	},

	_handleKeyPress: function(event){
		if (event.keyChar === "S" && event.shiftKey && event.ctrlKey) {
			dojo.stopEvent(event);
			if(this.isValid()){
				this.onDoneAttributes();
			}
		} else if (event.keyChar === "s" && event.ctrlKey) {
			dojo.stopEvent(event);
			if(this.isValid()){
				this.onSaveAttributes();
			}
		} else if (event.keyCode === dojo.keys.ESCAPE) {
			dojo.stopEvent(event);
			this.onCancelAttributes();
		}		
	},

	_updateField: function(parentNode, textToAdd) {
		if (textToAdd) {
			var textNode = document.createTextNode(textToAdd);
			dojo.place(textNode, parentNode, "only");

			//bidi
			if (parentNode !== this._documentIdSpan && parentNode !== this._modifiedOnView && parentNode !== this._createdOnView){
				BidiUtils.applyTextDirSetting(parentNode,  textToAdd);			
			}
			
		}
	},

	_composeShortDate: function(date) {
		if (date) {
			//ACGC_Bidi_start Static date support for gregorian and non-gregorian calendars
			return BidiUtils.applyDateSupport(date, { formatLength: "medium" });
			//ACGC_Bidi_End
		} else {
			return "";
		}
	},
	
	_createModifiedUserHover: function(user, node) {
		// Note: The userPropertyInfos is key'd on userURI, not userId, so we need to
		// iterate over them to find the appropriate one ...
		if (user){
			var userHover = new com.ibm.rdm.web.common.UserHover({projectURI : this._resource.getProjectURI()}, null);
			userHover.createHoverPopup(node, user);
			this._modifiedUserHover = userHover;
		}
	},
	
	_createCreatedUserHover: function(user, node) {
		// Note: The userPropertyInfos is key'd on userURI, not userId, so we need to
		// iterate over them to find the appropriate one ...
		if (user){
			var userHover = new com.ibm.rdm.web.common.UserHover({projectURI : this._resource.getProjectURI()}, null);
			userHover.createHoverPopup(node, user);
			this._createdUserHover = userHover;
		}
	},

	_removeModifiedUserHover: function() {
		if (this._modifiedUserHover) {
			this._modifiedUserHover.destroy();
			this._modifiedUserHover = null;
		}
	},
	
	_removeCreatedUserHover: function() {
		if (this._createdUserHover) {
			this._createdUserHover.destroy();
			this._createdUserHover = null;
		}
	},
	
	_destroyAttributeGroupSection: function() {
		if (this._attributeGroupSection) {
			this._attributeGroupSection.destroyRecursive();
			this._attributeGroupSection = null;
		}
	}
});
})();
