/*
 * Copyright (c) 2007, 2014. IBM Corporation
 * All rights reserved.
 */
(function() {
dojo.require("jazz.ui.Dialog");
dojo.provide("com.ibm.rdm.web.pages.WebUIinitializer");
dojo.require("com.ibm.rdm.web.common.Menu");
dojo.require("com.ibm.rdm.web.viewer.RecentlyViewedSection");
dojo.require("com.ibm.rdm.web.viewer.CollectionsMenu");
dojo.require("com.ibm.rdm.web.viewer.ModulesMenu");
dojo.require("com.ibm.rdm.web.viewer.ReportingMenu");
dojo.require("com.ibm.rdm.web.common.ProjectCache");
dojo.require("com.ibm.rdm.web.server.client.RDMClient");
dojo.require("dojo.i18n");
dojo.require("com.ibm.rdm.web.pages.About");
dojo.require("com.ibm.rdm.web.common.ErrorHandler");
dojo.require("com.ibm.rdm.web.common.MessageHandler");
dojo.require("com.ibm.rdm.web.links.LinkTypeUtils");
dojo.require("com.ibm.rdm.web.model.types.RRMTypes");
dojo.require("com.ibm.rdm.web.notifications.ChangeNotifier");
dojo.require("com.ibm.rdm.web.notifications.EventTopics");
dojo.require("com.ibm.rdm.web.model.types.XmlNamespaces");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("com.ibm.team.app.web.internal.welcome.WelcomeUtil");
dojo.require("com.ibm.rdm.web.reqpro.ImportProjectWizard");
dojo.require("com.ibm.rdm.web.query.StoredQueryClient");
dojo.require("com.ibm.rdm.web.parsers.xml2json");
dojo.require("dijit.MenuSeparator");
dojo.require("com.ibm.rdm.web.util.queryutils");
dojo.require("com.ibm.rdm.web.util.sessionutils");
dojo.require("com.ibm.rdm.web.common.AdminMenu");
dojo.require("com.ibm.rdm.web.server.jfs.RRCOAuthInterceptor");
dojo.require("com.ibm.rdm.web.icons.IconManager");
dojo.require("jazz.app._ComponentMenu");
dojo.require("com.ibm.rdm.web.locks.LockManager");
dojo.require("com.ibm.rdm.web.pages.DojoOverrides");
dojo.require("com.ibm.rdm.web.extensions.api.RM"); 
dojo.require("com.ibm.rdm.web.pages.viewStateManager");

dojo.requireLocalization("com.ibm.rdm.web.pages", "application");

/*
  Developers MAY wish to uncomment some of the following
  when looking into focus issues ...
*/
// require([ "dijit/focus" ], function(focusUtil){
// 	focusUtil.on("widget-focus", function(widget){
// 		console.log("Focused widget", widget);
// 	});
// 	focusUtil.on("widget-blur", function(widget){
// 		console.log("Blurred widget", widget);
// 	});
// 	focusUtil.watch("curNode", function(node){
// 		// Note: 2'nd arg is last focus and 3'rd is current focus.
// 		console.log("FOCUS last:", arguments[1]);
// 		console.log("FOCUS curr:", arguments[2]);
// 	});
// });


// JAZZ_ADOPTION
//
// Note by SPC on 6/18/2013 ...
//
// I've filed the following Foundation Defect:
//     https://jazz.net/jazz/web/projects/Jazz%20Foundation#action=com.ibm.team.workitem.viewWorkItem&id=269583
// but until this issue is addressed at the Foundation level we'll deal with
// it here.
//
// We're overriding jazz.ui.Dialog's 'close' function here so that it won't
// attempt to reset the focus if the originally captured focus represents a
// MenuItem.  We do this to avoid an IE9 issue: without this override, if you
// open a dialog from a folder context menu and then click on the dialog's
// "Cancel" button you'll find that immediately clicking on the folder drop
// down button again results in nothing happening - you must click on this
// drop down button 3 times before the context menu will appear again.
// [This turned out to be due to DropDownButton's '_opened' field (from the
// _HasDropDown mixin) being inappropriately set to true when you click on
// the drop down button again.]
// 
// Rather than performing this override explicitly in just those cases where
// we have "folder tree related dialogs", it seems like a safe thing to do
// everywhere, as it would never make sense to set focus back to a MenuItem
// that was associated with a popup menu that has been closed.
(function(){
	// Save the old prototype method.
	var originalCloseMethod = jazz.ui.Dialog.prototype.close;
 
	// Modify the prototype method.
	jazz.ui.Dialog.prototype.close = function(){
		if (this._lastFocus && this._lastFocus.node) {
			var widget = dijit.byId(this._lastFocus.node.id);
			if (widget && widget instanceof dijit.MenuItem) {
				this._lastFocus = null;
			}
		}
		// Call the original method.
		originalCloseMethod.apply(this, arguments);
	};
})();


var ProjectCache = com.ibm.rdm.web.common.ProjectCache;
var ChangeNotifier = com.ibm.rdm.web.notifications.ChangeNotifier;
var domutils = com.ibm.rdm.web.util.domutils;
var PlatformUI = net.jazz.ajax.ui.PlatformUI;
var SessionUtils = com.ibm.rdm.web.util.sessionutils;
var viewStateManager = com.ibm.rdm.web.pages.viewStateManager;

var ADMIN_UI_PATH = net.jazz.ajax._contextRoot+"/rmadmin";

var initializeCommunicationsDeferred = null;

jazz.client.internal.addResponseInterceptor(com.ibm.rdm.web.server.jfs.RRCOAuthInterceptor); 


com.ibm.rdm.web.pages.WebUIinitializer = {
	
	_messages: dojo.i18n.getLocalization("com.ibm.rdm.web.pages", "application"),
	_jazzApplication: null,
	_projectsList: null,
	_projectsMenu: null,
	_userInitDeferred: null,
	_projectListener: null,
	
	
	init: function(jazzApplication) {
		this._jazzApplication = jazzApplication;
		com.ibm.rdm.web.pages.DojoOverrides.performOverrides();
		this.initializeJFS();
		//this.initializeFooter();
		this.setWindowID();
		this.initEditor();
		com.ibm.rdm.web.extensions.api.RM.init();
	},


	_startHeartBeat: function() {
		// summary:
		//		Start calling viewStateManager.updateHeartBeat every
		//		viewStateManager.getUpdateInterval() seconds.
		// description:
		//		Please read the class documentation block for viewStateManager
		//		for more information.

		var updateInterval = viewStateManager.getUpdateInterval();

		viewStateManager.registerViewStateManager();

		// Add the first heart beat entry.
		viewStateManager.updateHeartBeat();

		// Now, every 'updateInterval' milliseconds, update the heart beat entry.
		window.setInterval(
			dojo.hitch(viewStateManager, viewStateManager.updateHeartBeat),
			updateInterval
		);
	},

	initializeFooter: function() {
		//hardcode jazz.net footer links until branding solution is available.
		this._jazzApplication.config.footer = {links: [ 
				{ label: this._messages.contact, href: "/legal/contact.jsp" },
				{ label: this._messages.termsOfUse, href: "/legal/terms.jsp" }
			]};
	},

	initializeJFS: function() {
		var deferred = this.initializeJFSForConfigProperties();
		deferred.addCallback(this, this.onInitializationDataRetreived);
	},
	
	initializeJFSForConfigProperties: function() {
		// summary:
		//		Initialize communications with the server.
		// description:
		//		This is called when */rm/web is loaded and it is also called
		//		when the RM viewlets are loaded in a project or user dashboard
		//		(via a call made in RRCViewlet.js).
		if (!initializeCommunicationsDeferred) {
			initializeCommunicationsDeferred = SessionUtils.initializeCommunications();
		}
		return initializeCommunicationsDeferred;
	},
	

	onInitializationDataRetreived: function(){
		this._startHeartBeat();
		window.addEventListener("beforeunload", function() {
			viewStateManager.deregisterViewStateManager();
		});

		var authenticatedContributorName = "com.ibm.team.repository.web.client.internal.AUTHENTICATED_CONTRIBUTOR";
		if (false === dojo.isObject(authenticatedContributorName)) {
			// making sure AUTHENTICATED_CONTRIBUTOR is an object is currently required, to fix defect 91488
			dojo.setObject(authenticatedContributorName, {});
		}

		this._jazzApplication.config.user = {
			name: com.ibm.rdm.web.AUTHENTICATED_USER.name,
			userId: com.ibm.rdm.web.AUTHENTICATED_USER.userId,
			admin: com.ibm.rdm.web.AUTHENTICATED_USER.isJazzAdmin,
			userSmallPreviewUrl: com.ibm.rdm.web.AUTHENTICATED_USER.uri + "/smallPreview", //TODO should not be hard-coded, later should be in JTS Discovery Doc
			userProfileUrl: com.ibm.rdm.web.AUTHENTICATED_USER.uri
		};
		this._jazzApplication.config.showLoginLogout = true;
		this._jazzApplication.config.logout = {
			callback: function(){
				com.ibm.rdm.web.locks.LockManager.releaseAllLockedResources();
				viewStateManager.deregisterAllViewStateManagers();
			}
		};
		
		if (window.location.pathname !== ADMIN_UI_PATH) {
			// Make sure the navigation bar menu items are left aligned.
			var left_align = "left";
			
			this._jazzApplication.config.navbarItems.push({
				label: this._messages.recentlyViewed, 
				menuProvider: dojo.hitch(
					com.ibm.rdm.web.viewer.RecentlyViewedSection,
					com.ibm.rdm.web.viewer.RecentlyViewedSection._menuProvider
				),
				align: left_align
			});
						
			this._jazzApplication.config.navbarItems.push({
				label: this._messages.collections, 
				menuProvider: dojo.hitch(
					com.ibm.rdm.web.viewer.CollectionsMenu,
					com.ibm.rdm.web.viewer.CollectionsMenu._menuProvider
				),
				align: left_align
			});
			
			if (com.ibm.rdm.web.MODULE_SUPPORT) {
				this._jazzApplication.config.navbarItems.push({
					label: this._messages.modules,
					menuProvider: dojo.hitch(com.ibm.rdm.web.viewer.ModulesMenu, com.ibm.rdm.web.viewer.ModulesMenu._menuProvider),
					align: left_align
				});
			}

			this._jazzApplication.config.navbarItems.push({
				label: this._messages.reportsMenu,
				menuProvider: dojo.hitch(com.ibm.rdm.web.viewer.ReportingMenu, com.ibm.rdm.web.viewer.ReportingMenu._menuProvider),
				align: left_align
			});
						
		}
		//Admin Menu
		this._jazzApplication.config.buttons.push({
			label: this._messages.administration,
			className: "admin-menu",
			align: "right",			
			menuProvider: dojo.hitch(this, function(){
				return new com.ibm.rdm.web.common.AdminMenu();
			})
		});
				
		this._jazzApplication.config.buttons.push({
			label: this._messages.upperCaseHelp,
			className: "help-menu",
			align: "right",
			menuProvider: dojo.hitch(this,this._helpMenuProvider)
		});
				
		// tell the framework to begin processing events (mandatory)
		// this was moved into this callback because it must be called after we determine the administrator permissions tab
		this._jazzApplication.start();	
		this._projectListener = ChangeNotifier.addListener({
			eventTopics: [com.ibm.rdm.web.notifications.EventTopics.BANNER_UPDATE],
			callbackFunction: this._onBannerChange,
			callbackScope: this,
			neverDelayCallback: true
		});
		ChangeNotifier.addListener({
			eventTopics: [
				com.ibm.rdm.web.notifications.EventTopics.PROJECT_CREATED,
				com.ibm.rdm.web.notifications.EventTopics.PROJECT_DELETED
			],
			callbackFunction: function(){
				try {
					if (this._jazzApplication._homeMenu) {
						this._jazzApplication.ui.homeMenuProvider().refresh();
					}
				}
				catch(e) {
				}
			},
			callbackScope: this,
			neverDelayCallback: true
		});
		
		this._onBannerChange();//initialize state of menus
		
		this.initializeHelp();
		
		//Callback other areas depending on initialisation
		
	},
	
	_onBannerChange: function() {
		var menuSelector = "a.net-jazz-ajax-HoverMenu[title='" + domutils.getXMLEncodedText(this._messages.recentlyViewed) + "']";
		menuSelector += ",a.net-jazz-ajax-HoverMenu[title='" + domutils.getXMLEncodedText(this._messages.collections) + "']";
		if (com.ibm.rdm.web.CURRENT_PROJECT) {
			dojo.query(menuSelector, dojo.byId('net-jazz-ajax-WorkbenchRoot')).forEach(function(node, i) {
				 dojo.style(node, "display", "inline");
			});
			dojo.style(this._jazzApplication.config.navbarItems[2]._widget, "display", "inline");
		} else {
			 dojo.query(menuSelector, dojo.byId('net-jazz-ajax-WorkbenchRoot')).forEach(function(node, i) {
				 dojo.style(node, "display", "none");
			});
		//	 dojo.style(this._jazzApplication.config.navbarItems[2]._widget, "display", "none");
		}
	},
	
	destroy: function() {
		ChangeNotifier.removeListener(this._projectListener);
		this.inherited(arguments);
	},
	
	_navigateToUrl: function(url) {
		// Prompt if we're navigating away from a dirty editor.
		if (!PlatformUI.getWorkbench().shouldVetoNavigation(url)) {
			window.location.href = url;
		}
	},
	
	_helpMenuProvider: function() {
		var menu = new jazz.app._ComponentMenu();
		// Hard coding this help URI until the help WAR is functioning correctly
		var helpuri = com.ibm.rdm.web.INFOCENTER_ROOT;
		if (helpuri) {
			this.helpuri = helpuri;
			menu.addItemByArgs({
				label: this._messages.upperCaseHelp,
				icon: dojo.moduleUrl("com.ibm.rdm.web", "pages/images/systemhelp.gif"),
				onClick: dojo.hitch(this,this._openHelpContents)
			});
		}
		menu.addItemByArgs({
			label: this._messages.gettingStarted,
			onClick: dojo.hitch(this,this._opentGettingStarted)
		});
		menu.addItemByArgs({
			label: this._messages.jazzHome,
			onClick: dojo.hitch(this,this._openJazzNet)
		});
		menu.addItemByArgs({
			label: this._messages.jazzForum,
			onClick: dojo.hitch(this,this._openJazzNetForum)
		});
		menu.addItemByArgs({
			label: this._messages.about,
			onClick: dojo.hitch(this,this._openAboutBox)
		});
		return menu;
	},
	
	_openHelpContents: function(event) {
		if (this.helpuri) {
			window.open(this.helpuri,'help');
		}
	},
	
	_openAboutBox: function(){
		var about = this._messages.about;
		new com.ibm.rdm.web.pages.About({dialogTitle: about});
	},
	
	_opentGettingStarted: function() {
		var url = net.jazz.ajax.infoCenterRoot() + "/topic/com.ibm.rational.rrm.help.doc/topics/c_compose_reqs.html";
		window.open(url, 'getting start');
	},
	
	_openJazzNet: function() {
		window.open("https://jazz.net", 'jazz');
	},
	
	_openJazzNetForum: function() {
		window.open("https://jazz.net/forum", 'forum');
	},
	
	populateProjectsMenu: function() {
		this._projectsMenu = new com.ibm.rdm.web.common.Menu();
		dojo.addClass(this._projectsMenu.domNode,"jazz-app-BannerButton-menu");
		
		if (!this._projectsList) {
			var deferred = ProjectCache.getUserMemberProjects();
			deferred.addCallback(this, this.populateProjectsMenuCallback);
		}
		else {
			this.populateProjectsMenuCallback(this._projectsList);
		}
		return this._projectsMenu;
	},
	populateProjectsMenuCallback: function(projects) {
		for (var projectURL in projects) {
			this._projectsMenu.addChild(new dijit.MenuItem({
				label: domutils.getXMLEncodedText(projects[projectURL].name),
				onClick: dojo.hitch(this, this.navigateToProjectDashboard, projects[projectURL].url)
			}));
		}
		return projects;
	},
	
	getNavigateToProjectDashboardURL: function(projectURI, /*Optional*/viewType, /*Optional*/baselineURI) {
		// summary:
		//		Gets the URL to navigate to in order to navigate to the all artifacts view
		//		(previously known as the project dashboard).
		var href;
		if (baselineURI){
			href = "#action=com.ibm.rdm.web.pages.showProjectHistory&projectURI="
				+ encodeURIComponent(projectURI)+"&baselineURI="+encodeURIComponent(baselineURI);
		}
		else{
			href = "#action=com.ibm.rdm.web.pages.showProjectDashboard&projectURI=" + encodeURIComponent(projectURI);
		}
		if (viewType) {
			href += "&viewType=" + viewType;
		}
		return href;
	},
	
	navigateToProjectDashboard: function(projectURI, /*Optional*/viewType, /*Optional*/baselineURI) {
		// Prompt if we're navigating away from a dirty editor.
		var url = this.getNavigateToProjectDashboardURL(projectURI, viewType, baselineURI);
		this._navigateToUrl(url);
	},
	
	navigateToProjectDashboardNoCache: function(projectURI, otherParams) {
		// Prompt if we're navigating away from a dirty editor.
		var href = "#action=com.ibm.rdm.web.pages.showProjectDashboard&projectURI=" 
				+ encodeURIComponent(projectURI) 
				+ "&useCache=false";
		if (!PlatformUI.getWorkbench().shouldVetoNavigation(href)) {
			window.location.href = href;
		}
	},
	
	initializeHelp: function() {
		// dashboard001, projecthome001, projprop001, artifact001, review001, collection001, search001
		dojo.setObject("com.ibm.rdm.web.help.context", []);
		var cshContext = "com.ibm.rational.rrm.web.cshelp";
		com.ibm.rdm.web.help.context.search = cshContext + ".search001";
		com.ibm.rdm.web.help.context.project = cshContext + ".projecthome001";
		com.ibm.rdm.web.help.context.projectprop = cshContext + ".projprop001";
		com.ibm.rdm.web.help.context.collectHome = cshContext + ".collecthome001";
		com.ibm.rdm.web.help.context.artifact = cshContext + ".artifact001";
		com.ibm.rdm.web.help.context.review = cshContext + ".review001";
		com.ibm.rdm.web.help.context.collection = cshContext + ".collection001";
		com.ibm.rdm.web.help.context.dashboard = cshContext + ".dashboard001";
		com.ibm.rdm.web.help.context.viewModules = cshContext + ".viewmods001";
	},
	
	resetCaches: function() {
		com.ibm.rdm.web.common.ProjectCache.resetProjectsCache();
	},
	
	setWindowID: function() {
		dojo.setObject("com.ibm.rdm.web.windowID", new Date().getTime());
	},
	
	initEditor: function() {
		dojo.setObject('CKEDITOR.basePath',  dojo.moduleUrl("com.ibm.rdm.web.ckeditor", "release/"));
	}
};

})();


