/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2015. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.picker.ArtifactPickerPane");

dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.require("dojo._base.event");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.Button");
dojo.require("dijit.form.CheckBox");

dojo.require("jazz.ui.FilterBox");
dojo.require("jazz.ui.ListBox");

dojo.require("com.ibm.rdm.web.common.ArtifactIcons");
dojo.require("com.ibm.rdm.web.common.ArtifactFormatInstancesStore");
dojo.require("com.ibm.rdm.web.common.ArtifactInstancesFilteringSelect");
dojo.require("com.ibm.rdm.web.common.MimeTypes");
dojo.require("com.ibm.rdm.web.common.ProjectCache");
dojo.require("com.ibm.rdm.web.common.ResourceLinksHandler");
dojo.require("com.ibm.rdm.web.grid.PagingWidget"); //Used for pagination
dojo.require("com.ibm.rdm.web.model.resources.ResourceManager");
dojo.require("com.ibm.rdm.web.model.types.RRMTypes");
dojo.require("com.ibm.rdm.web.model.types.TypesManager");
dojo.require("com.ibm.rdm.web.picker.AttributesSection");
dojo.require("com.ibm.rdm.web.project.DashboardConstants");
dojo.require("com.ibm.rdm.web.project.FilterSectionsContainer");
dojo.require("com.ibm.rdm.web.project.FoldersSection");
dojo.require("com.ibm.rdm.web.project.NewArtifactForm");
dojo.require("com.ibm.rdm.web.project.projectutils");
dojo.require("com.ibm.rdm.web.project.SavedFilter");
dojo.require("com.ibm.rdm.web.project.SavedFiltersSection");
dojo.require("com.ibm.rdm.web.project.TagsSection");
dojo.require("com.ibm.rdm.web.query.SavedFilterUtils");
dojo.require("com.ibm.rdm.web.query.ViewServiceClient");
dojo.require("com.ibm.rdm.web.search.domutils");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.require("com.ibm.rdm.web.util.storage");
dojo.require("com.ibm.rdm.web.util.bidiutils"); //bidi

dojo.requireLocalization("com.ibm.rdm.web.common", "common");
dojo.requireLocalization("com.ibm.rdm.web.picker", "ArtifactPickerPane");

var ArtifactFormatInstancesStore = com.ibm.rdm.web.common.ArtifactFormatInstancesStore;
var ArtifactIcons = com.ibm.rdm.web.common.ArtifactIcons;
var ArtifactInstancesFilteringSelect = com.ibm.rdm.web.common.ArtifactInstancesFilteringSelect;
var domutils = com.ibm.rdm.web.util.domutils;
var MimeTypes = com.ibm.rdm.web.common.MimeTypes;
var ProjectCache = com.ibm.rdm.web.common.ProjectCache;
var projectutils = com.ibm.rdm.web.project.projectutils;
var RRMTypes = com.ibm.rdm.web.model.types.RRMTypes;
var TypesManager = com.ibm.rdm.web.model.types.TypesManager;
var searchDomutils = com.ibm.rdm.web.search.domutils;
var BidiUtils = com.ibm.rdm.web.util.bidiutils;  //bidi

var NOT_SELECTABLE_ENTRY = "notSelectableEntry";
var LAST_FOLDER_COOKIE = "lastFoldersSelected_artifactPicker";
var GAP_BETWEEN_FILTER_SECTIONS = 0;
var FILTER_SECTION_WIDTH = 244;

dojo.declare(
	"com.ibm.rdm.web.picker.ArtifactPickerPane",
	[dijit._Widget, dijit._Templated],
{
	templatePath: dojo.moduleUrl("com.ibm.rdm.web",    
		"picker/templates/ArtifactPickerPane.html"),

	// Public fields
	projectURIScope: null,
	displayAllProjects: false,
	baselineURI: null,		
	filterArtifacts: [],
	selectionOptions: null,
	defaultFolderForArtifactCreation: null,
	showModuleRowStyleIcons: false,
	disablePickerModuleReuseMode: true,
	showModuleViews: null,
	modulesComboBoxMap: {},
	createArtifactNode: null,
	
	// Function to execute when the ENTER key is selected on the artifact list.
	enterKeyHandler: null,
	
	// doNotSetInitialFocusOnFolders: Boolean
	//		If true, it won't set initial focus on the filter by folders section
	doNotSetInitialFocusOnFolders: false,
	
	// Private fields	
	_messages: null,
	_commonMessages: null,
	_artifactSelectionPane: {},
	_projectListSelect: null,
	_projectNodeMessage: null,
	_hideProjectSelection: false,
	_artifactPickerDiv: null,

	constructor: function(args) {
		this.displayAllProjects= (args.displayAllProjects) ? args.displayAllProjects : false;
		if (this.displayAllProjects) {
			//Attempt to use last project used cookie if multiple project selection is 
			//allowed, otherwise simply rely on the consumer of ArtifactPickerPane to 
			//pass the projectURI used for scoping. 
			
			//Defect 51253: Cached folder pre-selection in artifact pickers interferes with search by ID or Name
			//Prevent the use of the cookie until this use case is re-designed.			
			//var lastProjectSelectedValue = dojo.cookie("lastProjectSelected_artifactPicker");
			var lastProjectSelectedValue=null;
			
			if (lastProjectSelectedValue) {
				this.projectURIScope = lastProjectSelectedValue;
			}else{
				this.projectURIScope = (args.projectURI)?args.projectURI:null;	
			}	
		} else {
			this.projectURIScope = (args.projectURI)?args.projectURI:null;
			// This was disabled by WI#:51253. No point in setting if never being used.
			//dojo.cookie("lastProjectSelected_artifactPicker" , this.projectURIScope, null);
		
		}
		this.modulesComboBoxMap = {};
		this.showModuleViews = args.showModuleViews || false;
		this.baselineURI = (args.baselineURI) ? args.baselineURI : null;
		this.filterArtifacts= (args.filterArtifacts) ? args.filterArtifacts : [];
		this.selectionOptions = (args.selectionOptions) ? args.selectionOptions : null;
		this._hideProjectSelection = (typeof args.hideProjectSelection) ? args.hideProjectSelection : false;	
	},
	
	postMixInProperties: function() {
		this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.picker", "ArtifactPickerPane");
		this._commonMessages = dojo.i18n.getLocalization("com.ibm.rdm.web.common", "common");
	},
	
	postCreate: function() {
		this._artifactSelectionPane = {};
		this.inherited(arguments);

		//Display project selection drop-down if allowed
		if (this.displayAllProjects && !this._hideProjectSelection) {
			this._projectListSelect = dojo.doc.createElement("select");
			dojo.addClass(this._projectListSelect, "projectInput");          
			dojo.connect(this._projectListSelect, "onchange", dojo.hitch(this, this._projectChanged));           
		}
		
		//set reuse mode
		this.disablePickerModuleReuseMode = 
			(this.disablePickerModuleReuseMode === undefined || this.disablePickerModuleReuseMode === null) ?
				true : this.disablePickerModuleReuseMode;
		
		// Start out with the picker from the current project
		this._showPickerForProject();
	},
	
	startup: function() {
		this.inherited(arguments);
		
		this._buildSearchForModuleSection();
		
		if (!this._hideProjectSelection) {
		
			this._projectNode.innerHTML = this._commonMessages.loading;
			this._showProjectSelectionRow();
			
			// We need information about projects from ProjectCache.
			// This involves deferreds, which will start up our 
			// _ArtifactPickerPane instance
			
			var projectInformationDeferred;
			if (this.displayAllProjects) {	
				//Display project area message		
				this._projectNodeMessage.appendChild(document.createTextNode(this._messages.selectProject));
				// We are going to allow selection of artifacts from all of
				// the current user's projects
				projectInformationDeferred = ProjectCache.getUserMemberProjects();
			}
			else {
				//Display project area message		
				this._projectNodeMessage.appendChild(document.createTextNode(this._messages.currentProject));
				// Scoping to the current project only
				projectInformationDeferred = ProjectCache.getProject(this.projectURIScope);
			}
			
			projectInformationDeferred.addCallback(this, function(projectInformation){
					 
				// Populate project selections in drop-down
				if (projectInformation[this.projectURIScope]) {
					for (var i=this._projectListSelect.options.length - 1; i>=0; i--) {
						var child = this._projectListSelect.options[i];
						this._projectListSelect.removeChild(child);
					}
					var projectsSorted = projectutils.getArrayOfProjectPropertyObjectsSortedByName(projectInformation);
					for (var i=0; i<projectsSorted.length; i++) {
						var projectURI = projectsSorted[i].url;
						this._createProjectOption(projectInformation[projectURI]);
					}
					
					 // Add our project options to the DOM
					dojo.place(this._projectListSelect, this._projectNode, "only");
				}
				else {
					//Artifact pane pinned to a project. Display selected project
					this._projectNode.innerHTML = projectInformation.name;
 
				}
						  
				return projectInformation;
			});
		}
		
		// We've got all we need, create our pane and start up
		if(!this._artifactSelectionPane[this.projectURIScope].isPaneStarted){
			this._artifactSelectionPane[this.projectURIScope].startup();
		}
		
	},
	
	uninitialize: function(){
		// Destroy all module filtering selects.
		for (var projectURI in this.modulesComboBoxMap) {
			var filteringSelect = this.modulesComboBoxMap[projectURI];
			if (filteringSelect) {
				var store = filteringSelect.store;
				filteringSelect.destroyRecursive();
				if (store) {
					store.destroy();
				}
			}
		}
		// Destroy all artifact selection panes created.
		for(var project in this._artifactSelectionPane){
			this._artifactSelectionPane[project].destroyRecursive();
		}
	},
	
	setFocusOnSearchBox: function() {
		this._artifactSelectionPane[this.projectURIScope].setFocusOnSearchBox();
	},
	

	getSelectedArtifacts: function() {
		return this._artifactSelectionPane[this.projectURIScope].getSelectedArtifacts();
	},
	
	getSelectedItemNames: function(){
		return this._artifactSelectionPane[this.projectURIScope].getSelectedItemNames();
	},
	
	getSelected: function() {
		return this._artifactSelectionPane[this.projectURIScope].getSelected();
	},
	
	getIndexedPropertiesForSelectedArtifacts: function() {
		return this._artifactSelectionPane[this.projectURIScope].getIndexedPropertiesForSelectedArtifacts();
	},
	
	removeSelectedArtifacts: function(){
		this._artifactSelectionPane[this.projectURIScope].removeSelectedArtifacts();
	},

	clearSelection: function(){
		this._artifactSelectionPane[this.projectURIScope].clearSelection();
	},

	createNewArtifact: function() {
		// summary:
		//		Create a new artifact for linking.
		this._artifactSelectionPane[this.projectURIScope].createArtifact();
	},
	
	_projectChanged: function() {
		this.projectChanged(dojo.attr(this._projectListSelect, "value"));
	},
	
	projectChanged: function (projectURI) {
	// summary:
	//		The project has changed to the given projectURI.  Update the pane to reflect the
	//		new project.

		this.projectURIScope = projectURI;
		var loadArtifactIconsDeferred = ArtifactIcons.loadTypes(projectURI);
		loadArtifactIconsDeferred.then(
				dojo.hitch(
					this,
					function(){
						this._showPickerForProject();
						//Persist selected project
						// This was disabled by WI#:51253. No point in setting if never being used.
						//dojo.cookie("lastProjectSelected_artifactPicker" , this.projectURIScope, null);
		
						// We've got all we need, create our pane and start up
						if(!this._artifactSelectionPane[this.projectURIScope].isPaneStarted){
							this._artifactSelectionPane[this.projectURIScope].startup();
						}
						this._buildSearchForModuleSection();	
					}
				)
		);
	},
	
	_showPickerForProject: function() {		
		var pane = this._getPaneForLinkType();
		dojo.place(pane.domNode, this._artifactPickerDiv, "only");
	},
	
	_getPaneForLinkType: function() {
		var pane = this._artifactSelectionPane[this.projectURIScope];
		if (pane) {
			// Caching and reusing the pane does not work properly in any of the browsers.
			// Formerly, it was just IE (see work item 67279), so recreate the pane.
			pane.destroyRecursive();
			this._artifactSelectionPane[this.projectURIScope] = null;
		}
		if (!this._artifactSelectionPane[this.projectURIScope]) {
			this._artifactSelectionPane[this.projectURIScope] = new com.ibm.rdm.web.picker._ArtifactPickerPane({
				projectURI: this.projectURIScope,
				baselineURI: this.baselineURI,
				filterArtifacts: this.filterArtifacts,
				selectionOptions: this.selectionOptions,
				displayAllProjects : this.displayAllProjects,
				defaultFolderForArtifactCreation : this.defaultFolderForArtifactCreation,
				onArtifactSelectionChanged:
					dojo.hitch(this, this._handleArtifactSelectionChanged),
				onItemSelected:
					dojo.hitch(this, this._handleItemSelected),
				doNotSetInitialFocusOnFolders: this.doNotSetInitialFocusOnFolders,
				showModuleRowStyleIcons: this.showModuleRowStyleIcons,
				disablePickerModuleReuseMode: this.disablePickerModuleReuseMode,
				showModuleViews: this.showModuleViews,
				enterKeyHandler: this.enterKeyHandler
			});
		
		}
		return this._artifactSelectionPane[this.projectURIScope];
	},
	
	_createProjectOption: function(projectInformation) {

		var projectOption = dojo.doc.createElement("option");
		dojo.attr(projectOption, "value", projectInformation.url);
		projectOption.innerHTML = domutils.getXMLEncodedText(BidiUtils.setTextDirUCC(projectInformation.name));
		
		// Mark selected if it's the current project
		if (projectInformation.url == this.projectURIScope) {
			dojo.attr(projectOption, "selected", true);
		}

		// Append the selection option to the list
		dojo.place(projectOption, this._projectListSelect);
	},
	
	_showProjectSelectionRow: function() {
		dojo.style(this._projectSelectionRow, "display", "");
	},
	
	_hideProjectSelectionRow: function() {
		dojo.style(this._projectSelectionRow, "display", "none");
	},

	_handleArtifactSelectionChanged: function(selectedItems){
		this.onArtifactSelectionChanged(selectedItems);
	},

	_handleItemSelected: function(item){
		if (this.onItemSelected) {
			this.onItemSelected(item);
		}
	},
	
	_buildSearchForModuleSection: function(){
		if (this.disablePickerModuleReuseMode) {
			return;
		}
		if (this._previousProjectURIScope && this.modulesComboBoxMap[this._previousProjectURIScope]) {
			dojo.style(this.modulesComboBoxMap[this._previousProjectURIScope].domNode, "display", "none");
		}
		if (this.projectURIScope && this.modulesComboBoxMap[this.projectURIScope]) {
			dojo.style(this.modulesComboBoxMap[this.projectURIScope].domNode, "display", "");
		}
		else {
			var store = new ArtifactFormatInstancesStore({
				projectURI: this.projectURIScope,
				disableReuse : true,
				artifactFormatURI: RRMTypes.getArtifactFormats(this.projectURIScope)[MimeTypes.MODULE_NAME]
			});
			var filteringSelect = new ArtifactInstancesFilteringSelect({
				store: store,
				searchAttr: "title",
				required: false,
				selectOnClick: false,	//Prevent dijit drop-down issue
				//http://bugs.dojotoolkit.org/ticket/12714
				onChange: dojo.hitch(this, this._onModuleChanged),
				onFocus: dojo.hitch(this, this._onModuleComboFocus),
				onBlur: dojo.hitch(this, this._onModuleComboBlur)
			});
			dojo.style(filteringSelect.domNode, "width", "200px");
			this._setModuleHintText(filteringSelect);
			this._moduleNode.appendChild(filteringSelect.domNode);
			this.modulesComboBoxMap[this.projectURIScope] = filteringSelect;
		}
		if (!this._previousProjectURIScope) {
			this._searchNodeMessage.appendChild(document.createTextNode(this._messages.searchFor));
			
			this.searchArtifactsRadioButton = new dijit.form.RadioButton({
				name: "searchRadioButton",
				checked: true,
				onChange: dojo.hitch(this, this._onSearchTypeChanged)
			});
			this._searchArtifactsNode.appendChild(this.searchArtifactsRadioButton.domNode);
			var labelNode = document.createElement("label");
			labelNode.appendChild(document.createTextNode(this._messages.searchForArtifactsInFolders));
			labelNode.htmlFor = this.searchArtifactsRadioButton.id;
			this._searchArtifactsNodeMessage.appendChild(labelNode);
			
			this.searchModulesRadioButton = new dijit.form.RadioButton({
				name: "searchRadioButton",
				checked: false						
			});
			this._searchModulesNode.appendChild(this.searchModulesRadioButton.domNode);
			labelNode = document.createElement("label");
			labelNode.appendChild(document.createTextNode(this._messages.searchForRowsInModule));
			labelNode.htmlFor = this.searchModulesRadioButton.id;
			this._searchRowsInModuleNodeMessage.appendChild(labelNode);
		
			dojo.style(this._searchForSelectionRow, "display", "");
		}
		this._previousProjectURIScope = this.projectURIScope;
	},
	
	_onModuleComboFocus: function() {
		// Clear the hint text when the modules combo text box gets focus, so user can type to
		// filter the list.
		var moduleComboBox = this.modulesComboBoxMap[this.projectURIScope];
		if (moduleComboBox.textbox.value == this._messages.selectModule) {
			moduleComboBox.textbox.value = "";
			moduleComboBox.textbox.style.color = "black";
		}
	},
	
	_onModuleComboBlur: function() {
		// Put the hint text back in place when focus leaves the the modules combo text box,
		// if a module hasn't been selected.
		var moduleComboBox = this.modulesComboBoxMap[this.projectURIScope];
		if (moduleComboBox.textbox.value == "") {
			moduleComboBox.textbox.value = this._messages.selectModule;
			moduleComboBox.textbox.style.color = "black";
		}
	},
	_onModuleChanged: function() {
		this.moduleURIScope = dojo.attr(this.modulesComboBoxMap[this.projectURIScope], "value");
		// We've got all we need, create our pane and start up
		var artifactSelectionPane = this._artifactSelectionPane[this.projectURIScope];
		if (artifactSelectionPane.isPaneStarted) {
			if (this.moduleURIScope) {
				this.searchModulesRadioButton.set("checked", true);
			}
			artifactSelectionPane.changeModuleScope(this.moduleURIScope);
		}
		
		// Now, update the module saved filters
		if (artifactSelectionPane._savedFiltersSection) {
			artifactSelectionPane._savedFiltersSection.setVisible(false);
		}
		if (artifactSelectionPane._savedModuleFiltersSection) {
			artifactSelectionPane._filterSectionsContainer.removeFilterSection(artifactSelectionPane._savedModuleFiltersSection, true);
			artifactSelectionPane._savedModuleFiltersSection = false;
		}
		if (this.moduleURIScope) {
			artifactSelectionPane._savedModuleFiltersSection = new com.ibm.rdm.web.project.SavedFiltersSection({
				projectURI: this.projectURIScope,
				resourceURI: this.moduleURIScope,
				filterSectionsContainer: artifactSelectionPane._filterSectionsContainer,
				sectionWidth: FILTER_SECTION_WIDTH,
				baselineURI: artifactSelectionPane.baselineURI,
				sectionTitle: this._messages.selectSavedFilter,
				dashboardTabPage: artifactSelectionPane,
				readOnly: true,
				viewType: com.ibm.rdm.web.project.DashboardConstants.VIEW_TYPES.MODULE,
				_handleClearFilters: dojo.hitch(artifactSelectionPane, function(){
					this._clearSelectedSavedFilters(false);
				})
			}, null);
			artifactSelectionPane._filterSectionsContainer.addFilterSection(artifactSelectionPane._savedModuleFiltersSection);
			artifactSelectionPane._savedModuleFiltersSection.setVisible(true);
			artifactSelectionPane._savedModuleFiltersSection._updateClearFilterState();
			artifactSelectionPane._savedModuleFiltersSection._toggleExpansionState();
			
			// Enable search field when a module is selected
			artifactSelectionPane._searchBox.setEnabled(true);
		}
	},
	
	_setModuleHintText: function(comboBox) {
		// Set the text box in the modules combo to indicate "select a module".
		comboBox.textbox.value = this._messages.selectModule;
		comboBox.textbox.style.color = "gray";
	},
	
	_onSearchTypeChanged: function() {
		
		var pane = this._artifactSelectionPane[this.projectURIScope],
			expandedSection;
			
		pane.moduleSearchMode = this.searchModulesRadioButton.get("checked");
		
		// We changed the search type. Unselect any filters.
		pane._clearSelectedFolders();
		pane._clearAttributesFilterSection();
		pane._clearSelectedTags();
		
		if (this.searchModulesRadioButton.get("checked") == true) {
			// Hide Create new artifact node when 'This module' is selected.
			if (this.createArtifactNode) {
				this.createArtifactNode.style.display = "none";
			}
			// If a module is not selected, disable the search field and clear the search field and result list.
			if (!this.moduleURIScope) {
				dojo.removeClass(pane._searchResultsNode,"message-entry");
				pane.resetPageToDisplay();
				pane._resultsList.removeAllEntries();
				pane._removeMatchingResultsNodes(true);
				pane._selectArtifactMessageNode.appendChild(document.createTextNode(this._messages.selectFromRecentMessage));
				pane._searchBox.clear();
				pane._searchBox.setEnabled(false);
			}
			
			// we are in "Rows in Module" mode, so hide the folder picker
			pane._foldersSection.setVisible(false);
			this._moduleSearchMode = true;
			pane._foldersSection.setExpansionStateAndHeight(com.ibm.rdm.web.project.FilterSection.expansionStates.Collapsed);
			// if the folder picker section of the filters was open, collapse it and reveal
			// the saved filter section.
			expandedSection = pane._filterSectionsContainer.getExpandedSection();
			// this second check appears to suggest there is a bug in setExpansionStateAndHeight
			// since we just told the folder section to collapse, but it is returning true for
			// isSectionExpanded()
			if (pane._savedFiltersSection) {
				pane._savedFiltersSection.setVisible(false);
			}
		} else {
			// otherwise, we are not in "Rows in Module" mode, so reveal the 
			// folder section.
			pane._searchBox.setEnabled(true);
			if (this.createArtifactNode) {
				this.createArtifactNode.style.display = "";
			}
			window.setTimeout(dojo.hitch(pane, function() {
				// If we have changed any sections, we need to give the pane a few milliseconds before we modify the sections.
				if (this._savedFiltersSection) {
					this._savedFiltersSection.setVisible(true);
				}
				if (this._savedModuleFiltersSection) {
					this._savedModuleFiltersSection.setVisible(false);
				}
				this._foldersSection.setVisible(true);
				if (!this._filterSectionsContainer.getExpandedSection()) {
					this._foldersSection._toggleExpansionState();
				}
			}), 50);
				
			this._moduleSearchMode = false;
			// cause module data to be cleared
			this.modulesComboBoxMap[this.projectURIScope].set("value", null);
			// set hint text
			this._setModuleHintText(this.modulesComboBoxMap[this.projectURIScope]);
		}	
	}

});

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//	CUSTOMIZED LIST BOX
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	
//Create a customized instance of the jazz.ui.ListBox widget. We need to do this because the current
//jazz.ui.ListBox widget does not allow for disabling entries in the list (such as the HTML select control does)
dojo.declare(
		"com.ibm.rdm.web.picker._ListBox",
		[jazz.ui.ListBox],
		{
			
			//Callers of setSelection may allow to persist currently selected
			//entries by setting this property to true
			persistPreviousSelections: false,
			//Counter for selected items in a page
			_selectedItemsInPage:[],
			_entryValuesMap: {},
			//flag to determine if CTRL key is in use
			_isCtrlKeyInUse: false,
			_isShiftKeyInUse: false,
			_lastEntrySelected: null,
			enterKeyHandler: null,

			getEntryNode: function(entry) {
				// summary:
				//		We need to tweak the DOM for newly added list items.
				//		Therefore we're exposing jazz.ui.ListBox's
				//		'_getEntryNode' for external use.

				return this._getEntryNode(entry);
			},

			//
			// Override the selection in the ListBox to prevent the highlight of
			// elements already added to the document
			//
			_setSelected: function(entry, select){
				var id = entry.id;

				// Check if this operation is applicable
				if (this._isObjectDefined(this._dataMap[id]) === false) {
					// NO-OP
					return;
				}
				// Check if a toggle is necessary
				if (this._selected[id] == select) {
					// NO-OP
					return;
				}

				//We need to check if the current entry is to be disabled. Check the entry.disabled property.
				//If not defined, then assume the entry is enabled
				var isEntryDisabled = (entry.disabled != 'undefined') ? entry.disabled : true;

				//Store the selection, only if we the entry is enabled		
				if (!isEntryDisabled) {
					// Store selection	
					this._selected[id] = select;
				}
				// Update UI
				var node = this._domMap[id];

				var addSelectionEntry = dojo.hitch(this, function(entry){
					var entryValue = entry.value,
						entryLabel = entry.label;
					if (!this._entryValuesMap[entryValue]) {
						this._entryValuesMap[entryValue] = true;
						this._selectedItemsInPage.push({
							"value": entryValue,
							"label": entryLabel,
							"isDisabled": false
						});
					}
				});
				var removeSelectionEntry = dojo.hitch(this, function(entry){
					if (this._entryValuesMap[entry.value]) {
						var i;
						delete this._entryValuesMap[entry.value];
						for (i = 0; i < this._selectedItemsInPage.length; i++) {
							if (this._selectedItemsInPage[i].value === entry.value) {
								this._selectedItemsInPage.splice(i, 1);
								break;
							}
						}
					}
				});

				//Modifying selection here to avoid highlight of an entry that has been tagged as disabled.
				if (select && !isEntryDisabled) {
					if(!(this._isCtrlKeyInUse || this._isShiftKeyInUse)){
						//If we detect a click happening without the control key being pressed, then remove 
						//previous selected items across pages	
						if(!this.persistPreviousSelections){
							this.clearSelectedItemsInPage();
						}						
					}
					dojo.addClass(node, "j-selected");	
					
					//Capture all current selections to our array of selected entries.
					//We do this since there may have been entries that were selected in combination
					//with the SHIFT key
					if (this._isShiftKeyInUse) {
						var selectedItems = this.getSelection(),
							i;
						this._selectedItemsInPage = [];
						this._entryValuesMap = {};
						if (this._lastEntrySelected && selectedItems[0].value === this._lastEntrySelected.value) {
							// The user Shift-Clicked a row ABOVE the first selected row.
							for (i = selectedItems.length-1; i >= 0; i--) {
								addSelectionEntry(selectedItems[i]);
							}
						} else {
							// The user Shift-Clicked a row BELOW the first selected row.
							for (i = 0; i < selectedItems.length; i++) {
								addSelectionEntry(selectedItems[i]);
							}
						}
					}
					else {
						addSelectionEntry(entry);
					}
				}
				else {
					dojo.removeClass(node, "j-selected");	
					//Remove the entry link from our list of collected entries, if it exists.
					removeSelectionEntry(entry);
				}
			},

			//TODO: this overrides internal methods that may change in ListBox. We should phase out
			//the ListBox widget in favor of our custom grid
			//newer method in ListBox
			//
			//Override selection entry method to account for the use of CTRL key
			//		
			_selectEntry: function(entry, ctrlKey, shiftKey){
				this._lastEntrySelected = entry;
				this._isCtrlKeyInUse = false;
				this._isShiftKeyInUse = false;
				if (shiftKey) {
					this._isShiftKeyInUse = true;
				}
				else if (ctrlKey) {
					this._isCtrlKeyInUse = true;
				}
				//Call the original method
				this.inherited(arguments);	
			},
			
			setEntryDisabled: function(entry){
				var id = entry.id;
				//We need to make the current entry disabled. Set the entry.disabled property.
				if (entry.disabled != 'undefined') {
					entry.disabled = true;
				}
				// Update UI
				var node = this._domMap[id];
				dojo.style(node, "color", "#CCCCCC");
				//Clear from the list of selections
				this._selected[id] = false;
				//For convenience, call this._setSelected which takes care of setting the correct highlight state for the entry
				this._setSelected(entry, false);							
			},
			
			_addIcon: function (parent, iconClass) {
				if (!iconClass) {
					return;	
				}
				var icon = document.createElement("span");
				dojo.addClass(icon, "entry-icon");
				var image = document.createElement("img");
				dojo.addClass(image, "entry-image");
				image.src = this._blankGif;
				dojo.addClass(image, iconClass);
				image.alt = this.messages.msgIcon;
				
				icon.appendChild(image);	
				parent.appendChild(icon);	
			},
			
			getSelectedItemsInPage: function(){
				// summary:
				//		Return an ordered array of the selected items (based on "selection
				//		order").
				// description:
				//		The returned array is ordered in "selection order".  For example
				//		if the user did click-Doc2, Ctrl-click-Doc1, the returned array would
				//		be [<Doc2-item>, <Doc1-item>].  Similary, if the listing showed Doc1,
				//		Doc2 and Doc3 and the user did click-Doc3 and Shift-click-Doc1, the
				//		returned array would be [<Doc3-item>, <Doc2-item>, <Doc1-item>].

				return this._selectedItemsInPage;
			},
			
			clearSelectedItemsInPage: function(){
				this._selectedItemsInPage=[];
				this._entryValuesMap = {};
				this._lastEntrySelected = null;
			},
			
			_handleEventOnKeyDown: function(event){
				// summary: 
				//		This overrides a superclass function. It stops certain 
				//		keys from affecting the scroll bar.
				if (event.keyCode === dojo.keys.UP_ARROW || 
					event.keyCode === dojo.keys.DOWN_ARROW || 
					event.keyCode === dojo.keys.SPACE) {
					dojo.stopEvent(event);
				}
				else if (event.keyCode === dojo.keys.ENTER) {
					if (this.enterKeyHandler) {						
						this.enterKeyHandler();
					} 
				}
				
				this.inherited(arguments);				
			}			
});

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//	PRIVATE ARTIFACT PICKER PANE
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//Inner widget used by "com.ibm.rdm.web.picker.ArtifactPickerPane"	
var MimeTypeRegistry = com.ibm.rdm.web.common.MimeTypeRegistry;
var ViewServiceClient = com.ibm.rdm.web.query.ViewServiceClient;
var MAX_PAGE_SIZE = 25;  //Set the size of the page
var MAX_MODULE_PAGE_SIZE = 100;  //Set the size of the module page

var WELL_KNOWN_NAMES = RRMTypes.WELL_KNOWN_ATTRIBUTE_NAMES;

dojo.declare(
	"com.ibm.rdm.web.picker._ArtifactPickerPane",
	[dijit._Widget, dijit._Templated],
{
	templatePath: dojo.moduleUrl("com.ibm.rdm.web", "picker/templates/_ArtifactPickerPane.html"),
	
	// Dojo Attach Points
	contentNode: null,
	
	// Public fields
	projectURI: null,
	baselineURI: null,
	filterArtifacts: [],
	enterKeyHandler: null,

	// Attach points
	_searchNode: null,
	_matchingResults: null,
	_createArtifactNode: null,
	_filterSectionsContainerDiv: null,
		
	//These are the attributes that will be asked for
	_queryAttributes: null,
	_queryAttributesMap: null,
	
	//Selection options
	displayArtifactCreation: false,
	allowMultipleSelection: true,
	displayRecentlyViewedArtifacts: true,
	isDisplayAllProjects:false,
	selectArtifactType:[],
	selectArtifactFormat: null,  // initialize to null so whether or not this parm is specified can be determined
	excludeArtifactFormat: [],
	mimeTypes: null,  // initialize to null so whether or not this parm is specified can be determined
	allRecentResources: [],
	recentResourceListEntries: [],
	displayFilterByTags: true,
	displaySavedFilters: true,
	//In the picker, the search field and the filter by Name do similar things, 
	//disable the filter by Name by default
	enableFilterByName: true,
	enableFilterByUser: true,
	enableFilterByDate: true,
	enableFilterByType: true,
	enableFilterByAttribute: true,
	enableFilterByWhereUsed: true,
	disablePickerModuleReuseMode: true,
	showHideArtifactInModulesCheckBox: null,
	defaultFolderForArtifactCreation: null,
	isPaneStarted: false,
	showModuleRowStyleIcons: null,
	moduleSearchMode: false,
	
	// doNotSetInitialFocusOnFolders: Boolean
	//		If true, it won't set initial focus on the filter by folders section
	doNotSetInitialFocusOnFolders: false,
	
	// Private fields
	_messages: null,
	_commonMessages: null,
	_totalProjectsByResource: null,
	_selectedArtifactFormatHash:[],
	_projectListSelect: null,
	_searchBox: null,
	_timeoutCounter: null,
	_resultsList: null,
	_selectedItems: null,	
	_foldersSection: null,
	_attributesSection: null,
	_savedFiltersSection: null,
	_createArtifactDiv: null,
	_contentTypeToNumber: {},
	_numberToIconMap: [],
	_mapResourceURIToResourceIndexedProperties: null,
	pagingWidget:null,
	_pageToDisplay:1,
	_lastPage:0,
	_pageSize:MAX_PAGE_SIZE,
	_preventRefresh: false,
	_isPageRequest:false,
	_lastSelectedItem: null,
	_selectedResults: null,
	_viewsToModuleNames: null,
	
	constructor: function(args) {
		this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.picker", "ArtifactPickerPane");
		this._commonMessages = dojo.i18n.getLocalization("com.ibm.rdm.web.common", "common");
		this.projectURI = args.projectURI;	
		this.filterArtifacts=(args.filterArtifacts)?args.filterArtifacts:[];
		var selectionOptions= (args.selectionOptions)?args.selectionOptions:null;
		this.isDisplayAllProjects= (args.displayAllProjects)?args.displayAllProjects:false;	
		this.disablePickerModuleReuseMode = 
			(args.disablePickerModuleReuseMode === undefined || args.disablePickerModuleReuseMode === null)?true:args.disablePickerModuleReuseMode;
		this._processSelectionOptionArgs(selectionOptions);
		this._processSelectedArtifactFormats();
		this._setRecentlyResourcesAndTotalProjects();
		this._setIconArtifactTypesImagesForProject();
		this._initializeRecentResources();
		this._viewsToModuleNames = {};
	},
	
	/*
	 * 
	 * [SW: this is not being called for some reason, I'm not sure why]
	postMixInProperties: function() {
		this._messages = dojo.i18n.getLocalization("com.ibm.rdm.web.picker", "ArtifactPickerPane");
		this._commonMessages = dojo.i18n.getLocalization("com.ibm.rdm.web.common", "common");
	},*/
	
	postCreate: function() {
		
		var self = this;
		this.inherited(arguments);
		
		this._queryAttributesMap ={};
		this._queryAttributesMap["wrappedResourceMimeType"] = "http://www.ibm.com/xmlns/rdm/rdf/wrappedResourceMimeType";
		this._queryAttributesMap["wrappedResourceURI"] = "http://www.ibm.com/xmlns/rdm/rdf/wrappedResourceURI";
		this._queryAttributes = [
			RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.Identifier,
			RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.Title,
			RRMTypes.RDF_PROCESS_AREA,
			RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.ArtifactFormat,
			"http://www.ibm.com/xmlns/rdm/rdf/wrappedResourceMimeType",
			"http://www.ibm.com/xmlns/rdm/rdf/wrappedResourceURI",
			RRMTypes.WELL_KNOWN_OBJECT_TYPE
		];
		
		this._searchBox = new jazz.ui.FilterBox({
			defaultText: this._messages.search,
			onChange: dojo.hitch(this, this._beginSearchTimeout) 
		});
		
		this._searchNode.appendChild(this._searchBox.domNode);
		this._searchBox.domNode.style.width = "100%";
		
		this._selectArtifactMessageNode.appendChild(document.createTextNode(this._messages.selectArtifact));
		
		var multipleSelection = this.allowMultipleSelection;
		
		//Create an instance of our customized version of jazz.ui.ListBox
		this._resultsList = new com.ibm.rdm.web.picker._ListBox({
			width: "515px",
			height: "351px",
			margin: "0",
			sortOrder: null,
			allowMultipleSelections: multipleSelection,
			iconMap: this._numberToIconMap,
			onChange: dojo.hitch(this, this._selectionChanged),
			enterKeyHandler: this.enterKeyHandler,
			ondblClick: dojo.hitch(this, this._handleDoubleClick)
		});			
		
		if (this.displayRecentlyViewedArtifacts) {
			//Show recent artifacts messages 	
			dojo.removeClass(this._searchResultsNode,"message-entry");	
			this._removeMatchingResultsNodes(true);
			this._selectArtifactMessageNode.appendChild(document.createTextNode(this._messages.selectFromRecentMessage));			
			this._resultsList.addEntries(this.recentResourceListEntries);					
			this._postProcessListBoxEntries();
		}
		else {
			this.showMessage(this._messages.filterNeededMessage);
		}
		//This line breaks IE7 commenting it for now
		//this._resultsList.domNode.style.border = "solid thin lightgray";
		this._searchResultsNode.appendChild(this._resultsList.domNode);

		// Create a container that will hold all of the filter sections
		// (currently just filter by folder and filter by attributes).
		this._filterSectionsContainer = new com.ibm.rdm.web.project.FilterSectionsContainer({
			gapBetweenFilterSections: GAP_BETWEEN_FILTER_SECTIONS
		});
		this._filterSectionsContainerDiv.appendChild(this._filterSectionsContainer.domNode);

		// Add the filter by folders section.
		this._foldersSection = new com.ibm.rdm.web.project.FoldersSection({
			projectURI: this.projectURI,
			isMultiSelect: true,
			filterSectionsContainer: this._filterSectionsContainer,
			sectionWidth: FILTER_SECTION_WIDTH,
			baselineURI: this.baselineURI,
			sectionTitle: this._messages.filterDisplayByFolder, 
			isFilterDeselectionByFolderAllowed: true,
			includeContextMenu: false,
			suppressInitialRootFolderSelection: true,
			dashboardTabPage: this,  // FoldersSection calls refresh method on this object 
			_handleClearFilters: dojo.hitch(this,function(){			
				this._clearSelectedFolders(false);
			}),
			doNotSetInitialFocusOnFolders: this.doNotSetInitialFocusOnFolders,
			isInPicker: true
		});
		this._filterSectionsContainer.addFilterSection(this._foldersSection);
		this._foldersSection.startup();
		this._foldersSection._updateClearFilterState();
		
		var restrictedValuesToUseInArtifactFormats = null;
		if (this.selectArtifactFormat || this.mimeTypes) {
			// artifact format has been specified, set restricted values so only those
			// values tha apply are displayed
			restrictedValuesToUseInArtifactFormats = {
				artifactFormats: this.selectArtifactFormat, mimeTypes: this.mimeTypes
			};
		}

		// Add the filter by attributes section.
		this._attributesSection = new com.ibm.rdm.web.picker.AttributesSection({
			projectURI: this.projectURI,
			filterSectionsContainer: this._filterSectionsContainer,
			sectionWidth: FILTER_SECTION_WIDTH,
			sectionTitle: this._messages.filterDisplayByAttribute,
			dashboardTabPage: this,
			restrictedValuesToUseInArtifactFormats: restrictedValuesToUseInArtifactFormats,
			enableFilterByName: this.enableFilterByName,
			enableFilterByUser: this.enableFilterByUser,
			enableFilterByDate: this.enableFilterByDate,
			enableFilterByType: this.enableFilterByType,
			enableFilterByArtifactFormat: true,
			enableFilterByAttribute: this.enableFilterByAttribute,
			enableFilterByWhereUsed: this.enableFilterByWhereUsed,
			_handleClearFilters: dojo.hitch(this,function(){			
				this._clearAttributesFilterSection(false);
			})
		});
		this._filterSectionsContainer.addFilterSection(this._attributesSection);
		this._attributesSection.startup();
		this._attributesSection._updateClearFilterState();
		
		
		if (this.displayFilterByTags) {
			// Add the filter by tags section
			this._tagsSection = new com.ibm.rdm.web.project.TagsSection({
				projectURI: this.projectURI,
				filterSectionsContainer: this._filterSectionsContainer,
				sectionWidth: FILTER_SECTION_WIDTH,
				baselineURI: this.baselineURI,
				sectionTitle: this._messages.filterDisplayByTag,
				includeContextMenu: false,
				dashboardTabPage: this,
				readOnly: true,
				_handleClearFilters: dojo.hitch(this, function(){
					this._clearSelectedTags(false);
				})
			}, null);
			this._filterSectionsContainer.addFilterSection(this._tagsSection);
			// Only populate the tags if the section is expanded or used in a saved filter.
			// this._tagsSection.startup();
			this._tagsSection._updateClearFilterState();
		}
		
		if (this.displaySavedFilters) {
			// Add the saved filters section
			this._savedFiltersSection = new com.ibm.rdm.web.project.SavedFiltersSection({
				projectURI: this.projectURI,
				filterSectionsContainer: this._filterSectionsContainer,
				sectionWidth: FILTER_SECTION_WIDTH,
				baselineURI: this.baselineURI,
				sectionTitle: this._messages.selectSavedFilter,
				dashboardTabPage: this,
				readOnly: true,
				viewType: com.ibm.rdm.web.project.DashboardConstants.VIEW_TYPES.ARTIFACTS,
				isContextArtifactPicker: true,
				_handleClearFilters: dojo.hitch(this, function(){
					this._clearSelectedSavedFilters(false);
				})
			}, null);
			this._filterSectionsContainer.addFilterSection(this._savedFiltersSection);
			this._savedFiltersSection.startup();
			this._savedFiltersSection._updateClearFilterState();
		}
		
		if (this.displayArtifactCreation){
			var aTag = dojo.create("a", {
				onclick: dojo.hitch(this, function(evt) {
					dojo.stopEvent(evt);
					this.createArtifact();
				}), 
				href: "#", 
				tabIndex: 0
			});
			aTag.appendChild(document.createTextNode(this._messages.createNewArtifact));
			this._createArtifactNode.appendChild(aTag);
		} else {
			this._createArtifactDiv.style.display = "none";
		}
		
		//Add paging
		this.pagingWidget = new com.ibm.rdm.web.grid.PagingWidget({}, this.pagingWidgetDiv);
		
		//Pass callback parameters to paging widget
		this.pagingWidget.addPaging({
			getTotalPagesFunction: function(){
				if (self._lastPage) {
					return self._lastPage;
				} else { 
					return -1;
				}
			},
			getCurrentPageFunction: function(){
				return self._pageToDisplay || 1;
			},
			gotoPageFunction: function(pageNum){
				self._pageToDisplay = pageNum;
				self.refresh(true);
			}
		});
		
		//Do not display the paging widget until is needed
		dojo.style(this.pagingWidget.domNode, 'visibility','hidden');
		
		//Process last folders selected. If selections are found for the current project, 
		//then select those folders in the folder widget 
		
		//Defect 51253: Cached folder pre-selection in artifact pickers interferes with search by ID or Name
		//Prevent the use of the cookie until this use case is re-designed.				
		//var selectedFolders = this.getLastSelectedFolders();
		var selectedFolders = null;
		if(selectedFolders && selectedFolders != "" ){
			this._foldersSection.selectFoldersByURI(selectedFolders).addCallback(this, function(){
				this.refresh(false);
				this._foldersSection._updateClearFilterState();
			});
			
		}
	},
	
	recordFilterChange: function() {
		//Do nothing. Here as ArtifactPickerPane can be SavedFiltersSection's dashboardTabPage.
	},
	
	setFocusOnSearchBox: function() {
		if (this._searchBox) {
			this._searchBox.focus();
		}
	},
	
	_setPreviouslySelectedItemsInPage: function(){	
		//Go through the previously selected entries and select any we find on this page.
		var entryToBeHighlighted = [],
			selectedItemsInPage = this._resultsList.getSelectedItemsInPage(),
			i;
		for (i = 0; i < selectedItemsInPage.length; i++){
			var item = selectedItemsInPage[i];
			var itemEntry = this._resultsList.getEntry(item.value /* URI */);
				
			if(itemEntry){
				//If the entry has already been added, then set the entry disabled
				if(item.isDisabled){
					this._resultsList.setEntryDisabled(itemEntry);
				}else{
					//Add the entry to be highlighted
					entryToBeHighlighted.push(itemEntry);
				}		
			}	
			
			if (dojo.indexOf(this._selectedItems, item.value) == -1 && (!item.isDisabled)) {
				this._selectedItems.push(item.value);
			}
			
		}
		//Indicate the ListBox that previous selections should not be removed from the list
		//while highlighting the current entries.
		this._resultsList.persistPreviousSelections = true;
		//Now show the entries as selected in the ListBox widget
		this._resultsList.setSelection(entryToBeHighlighted, true);
		//Indicate the ListBox that previous selection are not to be persisted. Only using the CTRL
		//key will preserve the previous selections
		this._resultsList.persistPreviousSelections = false;
	},
				
	startup: function() {
		this.inherited(arguments);
		var height = dojo.position(this._resultsList.domNode).h;
		this._filterSectionsContainer.startup();
		this._filterSectionsContainer.setHeight(height);
		this._filterSectionsContainer.resize();
		this.isPaneStarted=true;
	},

	uninitialize: function() {
		this._searchBox.destroyRecursive();
		this._resultsList.destroyRecursive();
		this._foldersSection.destroyRecursive();
		this._filterSectionsContainer.destroyRecursive();
		this._attributesSection.destroyRecursive();
		this._tagsSection.destroyRecursive();
		this._savedFiltersSection.destroyRecursive();
		
		this.pagingWidget.destroyRecursive();
		this.inherited(arguments);
	},
	
	_removeMatchingResultsNodes: function(removeArtifactMessageNode) {
	// summary:
	//		Helper method.  Remove the number of matches and number of selected artifact
	//		text nodes, so that these text messages can be regenerated based on picker selection.
	// removeArtifactMessageNode: 
	//		boolean flag, if true the text node that indicates to select an artifact will
	//		also be removed.
		domutils.removeChildNodes(this._matchingResults);
		this._selectedResults = null;
		if (removeArtifactMessageNode) {
			domutils.removeChildNodes(this._selectArtifactMessageNode);
		}
	},
	
	_processSelectionOptionArgs: function(selectionArgs){

		for(var prop in selectionArgs){

			switch(prop){
			case "selectArtifactType":
				this.selectArtifactType = selectionArgs[prop];
				break;
			case "selectArtifactFormat":
				this.selectArtifactFormat = selectionArgs[prop];
				break;
			case "mimeTypes":
				this.mimeTypes = selectionArgs[prop];
				break;
			case "excludeArtifactFormat":
				this.excludeArtifactFormat = selectionArgs[prop];
				break;
			case "allowMultipleSelection":
				this.allowMultipleSelection = selectionArgs[prop];
				break;
			case "displayRecentlyViewedArtifacts":
				this.displayRecentlyViewedArtifacts = selectionArgs[prop];
				break;	
			case "displayArtifactCreation":
				this.displayArtifactCreation = selectionArgs[prop];
				break;
			case "displayFilterByTags":
				this.displayFilterByTags = selectionArgs[prop];
				break;
			case "displaySavedFilters":
				this.displaySavedFilters = selectionArgs[prop];
				break;
			case "enableFilterByName":
				this.enableFilterByName = selectionArgs[prop];
				break;
			case "enableFilterByUser":
				this.enableFilterByUser = selectionArgs[prop];
				break;
			case "enableFilterByDate":
				this.enableFilterByDate = selectionArgs[prop];
				break;
			case "enableFilterByType":
				this.enableFilterByType = selectionArgs[prop];
				break;
			case "enableFilterByAttribute":
				this.enableFilterByAttribute = selectionArgs[prop];
				break;
			case "enableFilterByWhereUsed":
				this.enableFilterByWhereUsed = selectionArgs[prop];
				break;
			// TODO Christian : this will go away after Art's type/format changes
			case "newArtifactFormat":
				this.newArtifactFormat = selectionArgs[prop];
				break;
				
			case "disablePickerModuleReuseMode":
				this.disablePickerModuleReuseMode = selectionArgs[prop];
				break;
			}
		}		
	},
	
	_processSelectedArtifactFormats:function(){
		if (!this.selectArtifactFormat) {
			return;
		}
		this._selectedArtifactFormatHash =[];
			// Collect any format type filters to select from recently viewed resources
			for (var i = 0; i < this.selectArtifactFormat.length; i++) {
				var format = this.selectArtifactFormat[i];
				if (format == "WrapperResource" && this.mimeTypes != null) {
					for (var j = 0; j < this.mimeTypes.length; j++) {
						if (dojo.indexOf(this._selectedArtifactFormatHash, this.mimeTypes[j].contentType) == -1) {
							this._selectedArtifactFormatHash.push(this.mimeTypes[j].contentType);
						}				
					}
				}
				else {
					if (dojo.indexOf(this._selectedArtifactFormatHash, RRMTypes.getArtifactFormats(this.projectURI)[format]) == -1) {
						this._selectedArtifactFormatHash.push(RRMTypes.getArtifactFormats(this.projectURI)[format]);
					}					
				}			
			}
	},
	
	_setRecentlyResourcesAndTotalProjects: function(){
		
			//This method captures the recent artifacts and collections and returns an array of the total projects 
			//found per artifact
			
			this._totalProjectsByResource = [];
			
			//First include the current project
			this._totalProjectsByResource.push(this.projectURI);
					
			if (this.displayRecentlyViewedArtifacts) {
				var recentArtifacts = com.ibm.rdm.web.viewer.RecentlyViewedSection.getRecentArtifacts();
				this.allRecentResources = [];
				
				//Collect all recently viewed artifacts
				for (var i = 0; i < recentArtifacts.length; i++) {
					this.allRecentResources.push(recentArtifacts[i]);
				}
									
				for (var i = 0; i < this.allRecentResources.length; i++) {
					//Save the projectURI for this resource in our totals
					//Make sure the project is not already in the totalProject array
					var currentResource = this.allRecentResources[i].resourceObject;
					if (dojo.indexOf(this._totalProjectsByResource, currentResource.getProjectURI() /*projectURI*/) == -1) {
						this._totalProjectsByResource.push(currentResource.getProjectURI() /*projectURI*/);
					}
				}
			}
	},
		
	_setIconArtifactTypesImagesForProject: function(){
		
			this._contentTypeToNumber = {};
			this._numberToIconMap = [];
			var count = 0;
			

			for (var i = 0; i < this._totalProjectsByResource.length; i++) {
				var projectURI = this._totalProjectsByResource[i];
				//Get all the images for all types
				var allTypes = ArtifactIcons.getAllTypeKeys(projectURI);
				
				for (var j = 0; j < allTypes.length; j++) {
					var type = allTypes[j];
					var entryType = this._contentTypeToNumber[type];
					if (!entryType) {
						this._contentTypeToNumber[type] = count;
					
						var imageClass = ArtifactIcons.getSmallImageClass(projectURI, type);
						this._numberToIconMap[count] = imageClass;
						count++;
					}
				}
			}
			
			//Get image uris for Mime Types
			var wrappedMimeTypes = MimeTypes.WRAPPERS;
			if(wrappedMimeTypes) {
				for (var i = 0; i < wrappedMimeTypes.length; i++) {
					var contentType = wrappedMimeTypes[i].contentType;
					this._contentTypeToNumber[contentType] = count;
					var imageClass = wrappedMimeTypes[i].imageClass;
					this._numberToIconMap[count] = "type-icon " + imageClass + " small";
					count++;
				}
			}
			
		},
	
	_initializeRecentResources: function() {
		if (this.displayRecentlyViewedArtifacts) {			
			this.recentResourceListEntries = [];
			var recentResources = [];
			
			//recentResources contains recent artifacts and collections
			for (var i = 0; i < this.allRecentResources.length; i++) {
				var currentResource = this.allRecentResources[i].resourceObject;
				
				//Display only recently open artifacts from the current project
				if(currentResource.getProjectURI() != this.projectURI){
					continue;
				}
				
				var currentResourceId = currentResource.getID();
				
				if(currentResourceId == null){// 41608 - do not display artifacts without valid ID
					continue;
				}
				
				//Check if the artifact type for this recently viewed resource is allowed to be displayed.
				if (this.selectArtifactType.length > 0) {
					if (dojo.indexOf(this.selectArtifactType, currentResource.getMimeType()) === -1) {
						continue;
					}
				}
				//Check if the content type for this recently viewed resource is allowed to be displayed.
				if(this._selectedArtifactFormatHash.length > 0){
					if (dojo.indexOf(this._selectedArtifactFormatHash, currentResource.getMimeType()) === -1) {
						continue;
					}
				}
								
				//Skip this recently viewed resource if it is a review					
				if(currentResource.getEntityType()=== MimeTypes.CONTENT_TYPE_REVIEW) {
					continue;
				}
									
				recentResources.push(currentResource);
				var shouldDisable = false; //The current entry is enabled by default.
				//Check if current artifact needs to be filtered

				for (var x = 0; x < this.filterArtifacts.length; x++) {
					if (currentResource.getResourceURI() === this.filterArtifacts[x]) {
						shouldDisable = true;
						break;
					}
				}
								
				//Check if there is a request to filter current artifact type
				for (var x = 0; x < this.excludeArtifactFormat.length; x++) {
					if (currentResource.getEntityType() === RRMTypes.getArtifactFormats(currentResource.getProjectURI())[this.excludeArtifactFormat[x]]) {
						shouldDisable = true;
						break;
					}
				}
				
				for (var type in currentResource.getTypes()) {}
				var entryType = this._contentTypeToNumber[type];
				
				var artifactLabel = "",
					moduleTitle = "",
					module;
				
				if (currentResource.isInModule()) { 
					module = com.ibm.rdm.web.viewer.RecentlyViewedSection.URI_TO_MODULE_MAP[currentResource.getModuleURI()];
					moduleTitle = module ? module.getTitle() : "";
					artifactLabel = dojo.string.substitute(this._commonMessages["resourceInModule"], [currentResource.getDisplayTitle(), moduleTitle]);
					artifactLabel = dojo.string.substitute(this._messages.artifactLabel, [currentResourceId, artifactLabel]);
					shouldDisable = this.disablePickerModuleReuseMode;
				} else {
					artifactLabel = dojo.string.substitute(this._messages.artifactLabel, [currentResourceId, currentResource.getDisplayTitle()]);
				}
				
				//Encode recently viewed artifact title
				artifactLabel = com.ibm.rdm.web.util.domutils.getXMLEncodedText(artifactLabel);
				
				//Show artifact with a line thru if the artifact needs to be filtered from the selection
				if (shouldDisable) {
					//Change the style of a disable items to be greyed out.
					artifactLabel = "<span style='color:#CCCCCC;'>" + artifactLabel + "</span>";
				}
				
				this.recentResourceListEntries.push({
					label: artifactLabel,
					value: currentResource.getResourceURI(),
					id: currentResource.getResourceURI(), //Use artifact resource URI as record id for ListBox element
					artifactTypeURI: currentResource.getType(),
					processAreaURI: currentResource.getProcessAreaURI(),
					projectURI: currentResource.projectURI,
					type: entryType,
					disabled: shouldDisable
				});
			}
			//Add the resourceInfos to and index map
			this._resetIndexedPropertiesMap(recentResources);
		}
	},
	
	_clearSelectedFolders: function(preventRefresh){
		this._foldersSection._removeSelection();
		if(!preventRefresh){
			this.refresh(false);
		}	
	},
	
	_clearAttributesFilterSection: function(preventRefresh){
		this._attributesSection.clearAllFilters();
		if (!preventRefresh){
			this._attributesSection.handleSectionSelectionHasChanged();
		}
	},
	
	_clearSelectedTags: function(preventRefresh) {
		this._tagsSection.clearSelection();
		if (!preventRefresh) {
			this.refresh(false);
		}
	},
	
	_clearSelectedSavedFilters: function(preventRefresh) {
		if (preventRefresh) {
		   this._savedFiltersSection.clearSelection(); 
		} else {
			this._savedFiltersSection.unselectAll();
		}
	},
	
	setSavedFilterValues: function(savedFilter) {
		// summary:
		//		Resets the other three filter sections based on 
		//		the contents of a saved filter. This is mainly
		//		copied from DashboardTabPage::setSavedFilterValues

		var initialLoadSectionFiltersDeferredList = [];

		this._filterSectionsContainer.setIgnoreFilterSectionHeightChangeRequests(true);

		// Process Folders
		if (this._foldersSection && savedFilter.folderURIs && savedFilter.folderURIs.length > 0) {
			this._foldersSection.unselectAll(true);
			initialLoadSectionFiltersDeferredList.push(this._foldersSection.selectFoldersByURI(savedFilter.folderURIs));
		}
		else if (this._foldersSection) {
			this._foldersSection.unselectAll();
		}
		
		// Process Attributes
		if (this._attributesSection) {
			this._attributesSection.unselectAll(true);
			// search by name
			if (savedFilter.search) {
				initialLoadSectionFiltersDeferredList.push(
					this._attributesSection._filterByNameSection.setAppliedConditionInfo(savedFilter.search)
				);
			}
			// users
			if (savedFilter.users) {
				initialLoadSectionFiltersDeferredList.push(
						this._attributesSection._filterByUserSection.setAppliedConditionInfo(savedFilter.users)
				);
			}
			// artifact format
			if (savedFilter.artifactFormat) {
				this._attributesSection._filterByArtifactFormatSection.setAppliedConditionInfo(savedFilter.artifactFormat);
			}
			// date range
			if (savedFilter.dateRange) {
				initialLoadSectionFiltersDeferredList.push(
					this._attributesSection._filterByDateSection.setAppliedConditionInfo(savedFilter.dateRange)
				);
			}
			// where used
			if (this.enableFilterByWhereUsed && savedFilter.whereUsedConditionInfo) {
				initialLoadSectionFiltersDeferredList.push(
					this._attributesSection.setAppliedWhereUsedConditionInfo(
						savedFilter.whereUsedConditionInfo
					)
				);
			}
			// artifact types
			if (savedFilter.artifactTypeConditionInfo) {
				initialLoadSectionFiltersDeferredList.push(
					this._attributesSection._filterByArtifactTypeSection.setAppliedConditionInfo(savedFilter.artifactTypeConditionInfo)
				);
			}
			// attribute groups
			if (this._attributesSection && savedFilter.attributes && savedFilter.attributes.length > 0) {
				initialLoadSectionFiltersDeferredList.push(
					this._attributesSection._filterByAttributeValueSection.setAppliedAttributeValuesInfo(savedFilter.attributes));
			}
			// calm filters
			if (savedFilter.calm && savedFilter.calm.length > 0 && this._attributesSection._filterByLifecycleStatusSection) {
				this._attributesSection._filterByLifecycleStatusSection.setAppliedConditionInfo(savedFilter.calm);
			}
			// links
			if (savedFilter.linkedArtifactConditionInfos && savedFilter.linkedArtifactConditionInfos.length > 0 && this._attributesSection._filterByLinkedArtifactSection) {
				initialLoadSectionFiltersDeferredList.push(
					this._attributesSection._filterByLinkedArtifactSection.setAppliedAttributeValuesInfo(savedFilter.linkedArtifactConditionInfos));               
			}
		}
		
		// Process Tags
		if (this._tagsSection) {
			var tagOperator = savedFilter.tagOperator;
			this._tagsSection.setSelectedOperatorAndTagsNoRefresh(tagOperator, null);
			if (savedFilter.tagURIs && savedFilter.tagURIs.length > 0) {
				if (!this._tagsSection.isLoaded()) {
					var deferred = this._tagsSection.startup();
					deferred.addCallback(this, function(response) {
						this._tagsSection.setSelectedOperatorAndTagsNoRefresh(
							tagOperator, savedFilter.tagURIs);
					});
					deferred.addErrback(this, function(error) {
						this.showMessage(this._messages.filterCannotBeApplied);
					});
					initialLoadSectionFiltersDeferredList.push(deferred);
				} else {
					this._tagsSection.setSelectedOperatorAndTagsNoRefresh(
						tagOperator, savedFilter.tagURIs);
				}
			} 
		}
		
		// Callbacks
		var filterSectionsDeferredsCount = initialLoadSectionFiltersDeferredList.length;
		var checkIfAllFilterSectionsHaveLoaded = dojo.hitch(this, function(){
			filterSectionsDeferredsCount--;
			if (filterSectionsDeferredsCount <= 0) {
				if (this._tagsSection) {
					this._tagsSection._filterSelectionHasChanged();
					this._tagsSection.resetRefresh();
				}
				if (this._attributesSection) {
					this._attributesSection._filterSelectionHasChanged();
				}
				if (this._foldersSection) {
					this._foldersSection._filterSelectionHasChanged();
				}
				this._savedFiltersSection._filterSelectionHasChanged();
				this.refresh(savedFilter);
				this._filterSectionsContainer.setIgnoreFilterSectionHeightChangeRequests(false);
				
				this._attributesSection.savedFiltersHaveLoaded();

				this._filterSectionsContainer.filterSectionsSelectionsHaveChanged();
			}
		});
		if (filterSectionsDeferredsCount === 0) {
			checkIfAllFilterSectionsHaveLoaded();
		}
		else {
			for (var i = 0; i<initialLoadSectionFiltersDeferredList.length; i++) {
				initialLoadSectionFiltersDeferredList[i].addBoth(
					this, checkIfAllFilterSectionsHaveLoaded);
			}
		}
	},
	
	refresh: function(isPageRequest) {
		if(!this._preventRefresh){
			this._isPageRequest = isPageRequest;
			
			//If this is not a request to query for a particular page reset the page to display to 1.
			if(!this._isPageRequest){
				this.resetPageToDisplay();
				this._resultsList.clearSelectedItemsInPage();
			}
			
			// If we are in module search mode, but a module has not yet been selected,
			// do not attempt to perform view query and refresh the artifact list.  We only
			// want to do this when a module is actually selected.
			if (this.moduleSearchMode && !this.moduleURIScope) {
				return;
			}
			
			this.displayLoadingMsg();
			this._removeMatchingResultsNodes(true);
			this._selectArtifactMessageNode.appendChild(document.createTextNode(this._messages.selectArtifact));	
			this._resultsList.removeAllEntries();

			var viewServiceRequestParams = this._getViewServiceRequestParams();
			if (viewServiceRequestParams) {
				domutils.createHourGlassCursor();
				var viewRequestDeferred =
					ViewServiceClient.getViewContents(viewServiceRequestParams);

				viewRequestDeferred.addCallback(this, this._handleViewSuccessResponse);
			}else{
				this._removeMatchingResultsNodes(false);
				var matchingResultsText = this._messages.zeroMatchFound;
				this._matchingResults.appendChild(document.createTextNode(matchingResultsText));
				this._resultsList.removeAllEntries();
				this.onArtifactSelectionChanged([]); 
				if (this.displayRecentlyViewedArtifacts) {
					//Show recent artifacts messages 		
					dojo.removeClass(this._searchResultsNode,"message-entry");	
					this._removeMatchingResultsNodes(true);
					this._selectArtifactMessageNode.appendChild(document.createTextNode(this._messages.selectFromRecentMessage));				
					this._resultsList.addEntries(this.recentResourceListEntries);
					this._postProcessListBoxEntries();
					//Do not show the pagination widget
					dojo.style(this.pagingWidget.domNode, 'visibility', 'hidden');
				}
				else {
					this.showMessage(this._messages.filterNeededMessage);
				}
			}
		}else{
			//Allow for refreshing subsequent requests.
			this._preventRefresh =false;			
		}
	},
	
	_getViewServiceRequestParams: function() {
		// summary:
		//		Get the params to pass to the ViewServiceClient's
		//		getViewContents function, or null if no filters
		//		have been specified.
		
		this._pageToDisplay = this.getPageToDisplay();
		this._pageSize = this.getPageSize();
		
		var viewServiceRequestParams = null;
		var selectedFolderResources = (
			this._foldersSection ?
			this._foldersSection.getSelectedFolders() :
			[]
		);
		
		this.persistFolderSelections(selectedFolderResources);
		
		var isSearchByIDorTextApplied = (this._searchBox.value != "")?true:false;
		
		var artifactTypeConditionInfo = null;
		
		if(this.enableFilterByType){	
			//Custom type selections override those specified via the filter by artifact
			//widget.		
			if (this.selectArtifactType.length > 0) {
				artifactTypeConditionInfo = this._getCustomArtifactTypeSelection();
			}
			else {
				artifactTypeConditionInfo = this._attributesSection.getAppliedArtifactTypeConditionInfo();
			}
		}else{
			//Check to see if there is a request to select only specific artifact types 
			if (this.selectArtifactType.length > 0) {
				artifactTypeConditionInfo = this._getCustomArtifactTypeSelection();
			}
		}
			
		var isAtLeastOneAttributeApplied = (
			this._attributesSection ?
			this._attributesSection.isAtLeastOneAttributeApplied() :
			false
		);
		
		// Tags added for WI 33181
		var selectedTagResources = (
			this._tagsSection ? 
			this._tagsSection.getSelectedTags():
			[]
		);
		var tagOperator = this._tagsSection.getSelectedOperator();
		
		// Saved filters added for 33181
		
		if (isAtLeastOneAttributeApplied
			|| artifactTypeConditionInfo
			|| isSearchByIDorTextApplied
			|| selectedFolderResources.length > 0
			|| selectedTagResources.length > 0
			|| this.moduleURIScope
			) {
			var attributeConditionInfos =
				this._getAttributeConditionInfosForViewRequest();
			var artifactFormatInstancesConditionInfo = (
				this.enableFilterByWhereUsed ?
				this._attributesSection.getAppliedWhereUsedConditionInfo() :
				null
			);
			var linkedArtifactConditionInfos =
				this._getLinkArtifactConditionInfosForViewRequest();
				
			var fullTextCondition = null;
			if(searchDomutils.isNotAnInteger(this._searchBox.value)) {
				// Add lucene search clause
				fullTextCondition = {
					searchText: this._searchBox.value,
					searchField: ViewServiceClient.LUCENE_SEARCH_FIELDS
						.FullContent
				};
			}
			
			var orderBy = [
				{
					def: RRMTypes.getWellKnownAttributeURI(this.projectURI, RRMTypes.WELL_KNOWN_ATTRIBUTE_NAMES.Title),
					order: "a",
					type: "s"
				}
			];
			
			var viewServiceRequestBaseline = this.baselineURI;
			viewServiceRequestParams = {
				projectURI: this.projectURI,
				baselineURI: viewServiceRequestBaseline,
				collectionURI: null,
				folderURIs: selectedFolderResources,
				tagOperator: tagOperator,
				tagURIs: selectedTagResources,
				//Just get the required attributes
				requestedAttributeUris: [],
				artifactTypeConditionInfo: artifactTypeConditionInfo,
				whereUsedConditionInfo: artifactFormatInstancesConditionInfo,
				attributeConditionInfos: attributeConditionInfos,
				linkedArtifactConditionInfos: linkedArtifactConditionInfos,
				fullTextCondition: fullTextCondition,
				//Paginate to a given size, and cound results
				pageRequested: this._pageToDisplay,
				pageSize: this._pageSize,
				orderBy: orderBy,
				capQueryExecutionTime: true,
				isCoreView: (this.disablePickerModuleReuseMode || !this.moduleURIScope),
				disableModuleReuseMode: (this.disablePickerModuleReuseMode || !this.moduleURIScope)
			};
			
			viewServiceRequestParams.requestedAttributeUris = viewServiceRequestParams.requestedAttributeUris.concat(this._queryAttributes);
			
			if (this.moduleURIScope) {
				viewServiceRequestParams["orderBy"] = [{
					def: RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.BookOrder,
					order: "a"
				}];
				viewServiceRequestParams.moduleURI = this.moduleURIScope;
				viewServiceRequestParams.requestedAttributeUris = viewServiceRequestParams.requestedAttributeUris.concat([
					RRMTypes.WELL_KNOWN_IS_HEADING,
					RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.Section,
					RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.PrimaryText,
					RRMTypes.WELL_KNOWN_MODULE_TYPE,
					RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.BookOrder
				]);
				viewServiceRequestParams.summary = true;
				
				if (!isAtLeastOneAttributeApplied
					&& !artifactTypeConditionInfo
					&& !isSearchByIDorTextApplied
					&& selectedFolderResources.length == 0
					&& selectedTagResources.length == 0) {
						viewServiceRequestParams.requestedAttributeUris = viewServiceRequestParams.requestedAttributeUris.concat([
						RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.Depth						
					]);
					if (this._lastSelectedItem && !this._isPageRequest) {
						viewServiceRequestParams.pageRequested = Math.floor(this._lastSelectedItem.moduleBookOrder/MAX_MODULE_PAGE_SIZE+1);
					}
				}
			}
			else if (!this.disablePickerModuleReuseMode) {
				viewServiceRequestParams.requestedAttributeUris = viewServiceRequestParams.requestedAttributeUris.concat([
					RRMTypes.WELL_KNOWN_MODULE_TYPE,
					[
						RRMTypes.WELL_KNOWN_MODULE_TYPE,
						RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.Title
					]
				]);
			}
		}
		
		return viewServiceRequestParams;
	},
	
	_getCustomArtifactTypeSelection: function(){
		 // Summary:
		   //		Selects artifacts based on the type passed in 
		   //		the "selectArtifactType" picker option. Returns
		   //		an artifact type condition info.
		   
		var artifactTypeConditionInfo=null;		
		//Add an attribute for "limit by Artifact Type"
		if (this.selectArtifactType.length > 0) {
			var attributeValues = [ViewServiceClient.ARTIFACT_TYPE_OPERATORS.IsAny];
			// Filter by requested artifact types
			for (var i = 0; i < this.selectArtifactType.length; i++) {
				attributeValues.push(this.selectArtifactType[i]);
			}
			artifactTypeConditionInfo = {
				attributeDefinitionURI: null,
				attributeType: null,
				conditionType: com.ibm.rdm.web.query.conditionTypes.ArtifactTypeCondition,
				attributeValues: attributeValues,
				isEnum: true
			};
		}
		return artifactTypeConditionInfo;
	},
	_getAttributeConditionInfosForViewRequest: function() {
		// summary:
		//		Return an array that will represent the "attributes" property
		//		that will be passed in a call to the ViewServiceClient's
		//		performViewRequest function.
		var attributeConditionInfos = [];
	
		// Add an attribute for Artifact ID or Title  
		if (this._searchBox.value != ""){
			
			var stringSearch = this._searchBox.value;
			if (! searchDomutils.isNotAnInteger(this._searchBox.value)) {
				//Search By ID 
				var attributeURI = "http://purl.org/dc/terms/identifier";
				attributeConditionInfos.push({
					attributeDefinitionURI: attributeURI,
					attributeType: RRMTypes.ATTRIBUTE_TYPES.IntegerType,
					attributeValues: ["is", stringSearch],
					isEnum: false
				});
			}
		}
		
		// Add an attribute for the "Limit by name" filter, if needed.
		if(this.enableFilterByName){
			var nameAttributeConditionInfo =
				this._attributesSection.getAppliedArtifactNameAttributeConditionInfo();
			if (nameAttributeConditionInfo) {
				attributeConditionInfos.push(nameAttributeConditionInfo);
			}
		}

		// Add an attribute for the "Limit by date" filter, if needed.
		if(this.enableFilterByDate){
			var lastModifiedOnAttributeConditionInfo = this._attributesSection.getAppliedLastModifiedOnAttributeConditionInfo();
			if (lastModifiedOnAttributeConditionInfo) {
				attributeConditionInfos.push(lastModifiedOnAttributeConditionInfo);
			}
		}

		// Add an attribute for the "Limit by user" filter, if needed.
		if(this.enableFilterByUser){
			var usersAttributeConditionInfo =
				this._attributesSection.getAppliedUsersAttributeConditionInfo();
			if (usersAttributeConditionInfo) {
				attributeConditionInfos.push(usersAttributeConditionInfo);
			}
		}
					
		// Add an attribute for the "Limit by artifact format" filter, if needed.
		if(this._selectedArtifactFormatHash.length > 0){
			var attributeValues = [ ViewServiceClient.ENUM_OPERATORS.IsAll ];
			// Filter by requested format types
			for (var i = 0; i < this._selectedArtifactFormatHash.length; i++) {
				attributeValues.push(this._selectedArtifactFormatHash[i]);
			}
			
			var attributeURI =
				RRMTypes.getWellKnownAttributeURI(this.projectURI, RRMTypes.WELL_KNOWN_ATTRIBUTE_NAMES.ArtifactFormat);
			attributeConditionInfos.push({
				attributeDefinitionURI: attributeURI,
				attributeType: RRMTypes.ATTRIBUTE_TYPES.ArtifactFormatsType,
				attributeValues: attributeValues,
				isEnum: true
			});
			
		}
		
		// Add an attribute for the "Limit by attribute" filter, if needed.
		var otherAttributeConditionInfos = this._attributesSection.getAppliedAttributeConditionInfos();
		if (otherAttributeConditionInfos && otherAttributeConditionInfos.length > 0) {
			dojo.forEach(
				otherAttributeConditionInfos,
				function(attributeConditionInfo) {
					attributeConditionInfos.push({
						attributeDefinitionURI: attributeConditionInfo.attributeDefinitionURI,
						attributeType: attributeConditionInfo.attributeType,
						attributeValues: attributeConditionInfo.attributeValues,
						isEnum: attributeConditionInfo.isEnum
					});
				}
			);
		}
		
		return attributeConditionInfos;
	},
	
	_getLinkArtifactConditionInfosForViewRequest: function() {
		return this._attributesSection.getAppliedLinkedArtifactConditionInfos();
	},
	
	_handleViewSuccessResponse: function(response) {
		this._removeMatchingResultsNodes(false);
		this._selectedItems = [];
		this._resultsList.removeAllEntries();
		dojo.removeClass(this._searchResultsNode,"message-entry");

		var results = ViewServiceClient.parseViewContentsResponse(response, this.projectURI, this._queryAttributesMap);
		
		this._preProcessResultSet(results).then(
			dojo.hitch(this, function(results) {
				this._processResultSet(results);
			})
		);
	},
	
	_preProcessResultSet: function(results) {
		var deferred = new dojo.Deferred();
		if (this.showModuleViews) {
			var queryURI;
			var deferreds = [], moduleIndices = [], moduleNames = [];
			var resources = results.resources;
			for (var i = 0, l = resources.length; i < l; i++) {
				if (resources[i].getEntityType() == 
						RRMTypes.getArtifactFormats(this.projectURI)[MimeTypes.MODULE_NAME]) {
					deferreds = this._getSavedViewDeferreds(resources[i], deferreds);
					moduleIndices.push(i);
					var moduleTitle = com.ibm.rdm.web.util.domutils.getXMLEncodedText(resources[i].getTitle());
					moduleTitle = dojo.string.substitute(this._messages.artifactLabel, [resources[i].getID(), moduleTitle]);
					moduleNames.push(moduleTitle);
				}
			}
			var deferredList = new dojo.DeferredList(deferreds);
			deferredList.then(
				dojo.hitch(this, function(savedViewResults) {
					for (var i = 0, l = savedViewResults.length; i < l; i++) {
						this._buildSavedViewResults(savedViewResults[i][1], results, i, moduleIndices, moduleNames);
					}
					deferred.resolve(results);
				}),
				dojo.hitch(this, function(error) {
					console.error(error);
					deferred.resolve(results);
				})
			);
		} else {
			deferred.resolve(results); 
		}
		return deferred;
	},
	
	_getSavedViewDeferred: function(queryURI) {
		return ProjectCache.getSavedFilters(queryURI, this.projectURI, true);
	},
	
	_getSavedViewDeferreds: function(resource, deferredsArray) {
		return deferredsArray.concat([this._getSavedViewDeferred(
					this._getSavedViewQueryURI(com.ibm.rdm.web.project.SavedFilter.scope.MODULE_PUBLIC, resource))]);
	},
	
	_getSavedViewQueryURI: function(scope, resource) {
		return com.ibm.rdm.web.query.SavedFilterUtils
				.getQueryURI(scope, this.baselineURI, resource.getResourceURI());
	},
	
	_buildSavedViewResults: function(publicSavedViewResults, viewServiceResults, 
									 currentIndex, moduleIndices, moduleNames) {
		var savedViews = [];
		var savedViewURI;
		for (savedViewURI in publicSavedViewResults) {
			savedViews.push(this._morphSavedView(publicSavedViewResults[savedViewURI], moduleNames[currentIndex]));
		}
		
		if (savedViews.length > 0) {
			var indexToInsert = moduleIndices[currentIndex] + 1;
			Array.prototype.splice.apply(viewServiceResults.resources, [indexToInsert, 0].concat(savedViews));
			for (var i = 0, l = moduleIndices.length; i < l; i++) {
				moduleIndices[i] += savedViews.length;
			}
			var count = parseInt(viewServiceResults.count);
			count += savedViews.length;
			viewServiceResults.count = count;
		}
	},
	
	_morphSavedView: function(savedView, moduleName) {
		var fakeResource = {};
		var materializedViewURI = com.ibm.rdm.web.query.SavedFilterUtils.getMaterializedViewURI(savedView.resourceURI);
		this._viewsToModuleNames[materializedViewURI] = moduleName;
		fakeResource[RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.Identifier] = com.ibm.rdm.web.project.SavedFilter.IDENTIFIER;
		fakeResource[RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.Title] = savedView.title;
		fakeResource.projectURI = this.projectURI;
		fakeResource.resourceURI = materializedViewURI;
		return com.ibm.rdm.web.model.resources.ResourceManager.getFromPartial(this.projectURI, fakeResource);
	},
	
	_processResultSet: function(results) {
		domutils.createDefaultCursor();
		var entries = [];
		var resourceDescriptions = [];
		var resources = results.resources;
		//Save the current page information
		this._lastPage = results.lastPage;
		this._pageToDisplay = results.currPage;
		var totalResultsMatched = (results.count) ? results.count : 0;		

		for (var i = 0; i < resources.length; i++) {
			var result = resources[i]; 
			var moduleHostUri = result.getModuleHostURI();

			if (!result) {
				continue;
			}
			
			var resultID = result.getID();
			if (resultID == null) {//41608 - skip artifacts without valid id
				continue;
			}
			
			//Show artifact with a different style if the artifact needs to be filtered from the selection	
			var domStyles = "";
			var shouldDisable = this._shouldDisableEntry(result);
			if (shouldDisable){
				//Change the style of a disable items to be greyed out.
				domStyles = "color:#CCCCCC;";					
			}
			
			var primaryText = result.getAttributeValue(RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.PrimaryText);
			if (primaryText && primaryText.length > 50) {
				var cutOffIndex = primaryText.indexOf(" ", 50);
				if (cutOffIndex < 1) {
					cutOffIndex = 50;
				}
				primaryText = primaryText.substring(0, cutOffIndex) + "...";
			} 
			var encodedTitle = com.ibm.rdm.web.util.domutils.getXMLEncodedText(primaryText ? primaryText:result.getTitle());
			encodedTitle = encodedTitle.replace(/ /g,"&nbsp;");
			
			// module mode is turned on
			if (this.moduleURIScope) {
				var artifactLabel = (result.isHeading()) ?
					"<b>" + result.getModuleSection() + "&nbsp;" + encodedTitle + "</b>" :
					encodedTitle;
					
				if (result.getModuleDepth()) {
					artifactLabel = resultID + ':&nbsp;<span style="white-space:nowrap; padding-left:' + (result.getModuleDepth()*5) + 'px">' + artifactLabel + '</span>';
				}
				else {
					artifactLabel = resultID + ':&nbsp;' + artifactLabel;
				}
			} else {
				// Check to see if we are a module saved view
				if (resultID == com.ibm.rdm.web.project.SavedFilter.IDENTIFIER) {
					artifactLabel = '<span style="color: #808080;">' + this._messages.viewLabel + '&nbsp;</span>' + encodedTitle;
					domStyles += "padding-left:20px;";
				} else {
					var artifactLabel = (moduleHostUri) ? encodedTitle : 
						dojo.string.substitute(this._messages.artifactLabel, [resultID, encodedTitle]);
				
					//Check if this is a re-used artifact. Label it appropiately
					if (moduleHostUri) {
						//Module URI specified as binding
						//NO longer display the module icon for the artifact in module
						//artifactLabel += com.ibm.rdm.web.common.ResourceLinksHandler.getModuleLabel(result.getModuleTitle());				
						artifactLabel += dojo.string.substitute(this._messages.artifactInModule, [result.getModuleTitle()]);
						domStyles += "padding-left:16px;font-size:8pt;";
						entryType = null;
					}
				}
			}
			
			//Decorate with any CSS styles
			if (domStyles != "") {
				artifactLabel = "<span style=" + domStyles + ">" + artifactLabel + "</span>";
			}
			
			var typeInformation = this._getTypeInformation(result);
						
			entries.push({
				label: artifactLabel,
				value: result.getResourceURI(),
				id: result.getResourceURI(), //Use artifact resource URI as record id for ListBox element
				type: typeInformation.entryType,
				artifactTypeURI: result.getType(),
				processAreaURI: result.getProcessAreaURI(),
				moduleBookOrder: result.getModuleBookOrder(),
				projectURI: result.projectURI,
				disabled: shouldDisable
			});

			result.displayName = result.getTitle();
			result.contentType = typeInformation.entityType;
			result.type = typeInformation.entryType;
			resourceDescriptions.push(result);
		}
		
		this._displayResults(entries, resourceDescriptions, totalResultsMatched);
	},
	
	_shouldDisableEntry: function(result) {
		var shouldDisable = false; //The current entry is enabled by default.
		//Check if current artifact needs to be filtered
		for (var x = 0; x < this.filterArtifacts.length; x++) {
			if (result.getResourceURI() == this.filterArtifacts[x]) {
				shouldDisable = true;
				break;
			}
		}
		
		//Check if there is a request to filter current artifact type
		for (var x = 0; x < this.excludeArtifactFormat.length; x++) {
			if (result.getEntityType() == RRMTypes.getArtifactFormats(this.projectURI)[this.excludeArtifactFormat[x]]) {
				shouldDisable = true;
				break;
			}
		}
		
		return shouldDisable;
	},
	
	_getTypeInformation: function(result) {
		var entryType, entityType;
	
		for (var type in result.getTypes()) {}
		entryType = this._contentTypeToNumber[type];
		entityType = result.getEntityType();
		
		return {
			entryType: entryType,
			entityType: entityType
		};
	},

	_postProcessListBoxEntries: function() {
		// summary:
		//		Tweak the DOM representing newly added jazz.ui.ListBox entries
		//		to specify appropriate 'alt' and 'title' attributes for the
		//		artifact type icon and, if needed, to add a row style icon
		//		between the artifact type icon and artifact name for every
		//		list item presented.

		var addAltAndTitleToArtifactTypeNode = dojo.hitch(
			this,
			function(listEntryNode, listEntryObjectType){
				// Note: In a dev env, the graphic artifact icons may be missing
				// which is why we're using the following 'if' statement ...
				var node,
					altAndTitleText;
				if (listEntryNode.children.length > 1) {
					altAndTitleText = ArtifactIcons.getAltTitleText(listEntryObjectType);
					node = listEntryNode.children[0];
					node.setAttribute("alt", altAndTitleText);
					node.setAttribute("title", altAndTitleText);
				}
			}
		);
		var addRowTypeIcon = dojo.hitch(this, function(listEntryNode, listEntryObjectType){
			var rowStyleIconTitle,
				rowStyleIconClass;
			if (listEntryObjectType) {
				if (listEntryObjectType.isModuleHeading()) {
					rowStyleIconTitle = this._messages.headingRow;
					rowStyleIconClass = "row-type-icon numbered-row";
				} else {
					rowStyleIconTitle = this._messages.normalRow;
					rowStyleIconClass = "row-type-icon not-numbered-row";
				}
			}
			dojo.place(
				dojo.create(
					"div",
					{
						"role": "presentation",
						"class": rowStyleIconClass,
						"title": rowStyleIconTitle,
						"alt": rowStyleIconTitle
					}
				),
				listEntryNode,
				// Note: The UX team wants the "row style icon" to appear
				// between the artifact type icon and the artifact name
				// (but in a dev env, graphical artifact icons may be missing
				// so we're using the following instead of simply '1'...)
				listEntryNode.children.length-1
			);
		});
		TypesManager.getObjectTypesForProject({
			projectURI: this.projectURI
		}).then(
			dojo.hitch(this, function(objectTypes) {
				var entries = this._resultsList.getAllEntries(),
					mapArtifactTypeUriToObjectType = {},
					objectType,
					entry,
					node,
					i;
				// Fill the 'mapArtifactTypeUriToObjectType' with appropriate entries.
				for (i = 0; i < objectTypes.length; i++) {
					mapArtifactTypeUriToObjectType[objectTypes[i].getResourceURI()] =
						objectTypes[i];
				}
				// Make appropriate DOM tweaks for all the currently displayed
				// list entries.
				for (i = 0; i < entries.length; i++) {
					entry = entries[i];
					node = this._resultsList.getEntryNode(entry);
					objectType = mapArtifactTypeUriToObjectType[entry.artifactTypeURI];
					addAltAndTitleToArtifactTypeNode(node, objectType);
					if (this.showModuleRowStyleIcons) {
						addRowTypeIcon(node, objectType);
					}
				}
			})
		);
	},
	
	_displayResults: function(entries, resourceDescriptions, totalResultsMatched) {
		this._resultsList.showLoading(false);
		this._resultsList.addEntries(entries);
		this._postProcessListBoxEntries();
		
		//If this is a page request, then highlight any previous selections.
		if (this._isPageRequest) {
			//Now that the results have been added, check if we have any previously 
			//selected item in the current page. This is only necessary if this 
			//is a request for a page result.
			this._setPreviouslySelectedItemsInPage();
		} else {
			//This is not a page request. Remove any previous selection messages
			//left over from a previous result set.
			if (this._selectedResults) {
				this._matchingResults.removeChild(this._selectedResults);
				this._selectedResults = null;
			}
		}
		
		//Display or hide the pagination based on the amount of pages found	
		if (this._lastPage == 0 || this._lastPage == 1) {
			this.resetPageToDisplay();
			this._lastPage = 0;
			dojo.style(this.pagingWidget.domNode, 'visibility', 'hidden');
		}
		else {	
			dojo.style(this.pagingWidget.domNode, 'visibility', 'visible');
			this.pagingWidget.refresh();
		}	
		
		//Add the resourceInfos to and index map
		this._resetIndexedPropertiesMap(resourceDescriptions);

		var matchingResultsText = null;
		
		if (entries.length == 0) {  //Case no matches found
			matchingResultsText = this._messages.zeroMatchFound;
		} else {
			if (entries.length == 1){ //Case exactly one match found
				matchingResultsText = this._messages.oneMatchFound;
			} else{							
				//Displaying number of artifacts and account for paging. Format is the following "Displaying (X - Y) of N matches" 
				var pageSize = this.getPageSize();
				var startItemNumber = 1;
				var endItemNumber = 1;
				var pageRange = "";		
					
				startItemNumber = ((this._pageToDisplay * pageSize) - pageSize) + 1;
				endItemNumber = (startItemNumber + (entries.length - 1));						
				pageRange = dojo.string.substitute(this._messages.numberOfArtifactsRange, [startItemNumber, endItemNumber]);
				matchingResultsText = dojo.string.substitute(this._messages.numberOfMatchesFound, [pageRange, totalResultsMatched]);	
			}	
		}
				
		this._matchingResults.appendChild(document.createTextNode(matchingResultsText));
		var selectedItemsCount = this._selectedItems.length;
		if (selectedItemsCount > 0) {
			var selectedMsg="";
			if(selectedItemsCount === 1){
				selectedMsg = dojo.string.substitute(this._messages.oneArtifactSelected,[selectedItemsCount]);
			}else{
				selectedMsg = dojo.string.substitute(this._messages.numberOfArtifactsSelected,[selectedItemsCount]);
			}	
			this._selectedResults = document.createTextNode(selectedMsg);
			this._matchingResults.appendChild(this._selectedResults);
		}
		//Fire an event as soon as results are displayed. This should force the observer buttons 
		//to refresh their state since there are no selections
		this.onArtifactSelectionChanged(this._selectedItems);
	},
	
	onItemSelected: function(/*Object*/selectedItem) {
		// summary:
		//		Callback to handle the case of a single selection occurring in
		//		the artifact selection area.  If specified, this callback will
		//		occur after onArtifactSelectionChanged, when one item is
		//		selected.
		// selectedItem:
		//		The selected item object with its associated attributes.
		// tags:
		//		callback
	},
	
	onArtifactSelectionChanged: function(/*String[]*/numArtifactsSelected) {
		// summary:
		//		Callback to handle the case of a selection change occuring in
		//		artifact selection area.
		// numArtifactsSelected:
		//		The number of artifacts that are currently selected in the
		//		"artifact results" area on the right hand side of the picker.
		// tags:
		//		callback
	},
	
	getIndexedPropertiesForSelectedArtifacts: function() {
		// summary:
		//		Gets indexed properties information about all currently selected
		//		artifacts in the "artifact results" area on the right hand side
		//		of the picker.
		// returns:
		//		An array of property lists.  Please see the comment block for
		//		ProjectCache.getBatchResourceDescriptionsMultiFetch for details
		//		about what information is found in each propery list element.
		
		//Collect the currently selected items along with any previously selected items in a page
		
		var totalSelectedItems = [],
			selectedItemsInPage = this._resultsList.getSelectedItemsInPage(),
			item,
			i;
		for (i = 0; i < selectedItemsInPage.length; i++) {
			item = selectedItemsInPage[i];
			//We have to make sure we don't send back any items previously selected.
			if (!item.isDisabled) {
				totalSelectedItems.push(item.value);
			}
			//Set this entry as disabled so that it shows greyed out in
			//future page requests.
			item.isDisabled = true;
		}
		
		for(var i=0; i < this._selectedItems.length; i++){
			if (dojo.indexOf(totalSelectedItems, this._selectedItems[i]) == -1) {
				totalSelectedItems.push(this._selectedItems[i]);
			}
		}

		var arrayOfIndexedPropertiesForSelectedArtifacts =
			dojo.map(
				totalSelectedItems,
				function(resourceURI) {
					return this._mapResourceURIToResourceIndexedProperties[resourceURI];
				},
				this
			);
		return arrayOfIndexedPropertiesForSelectedArtifacts;
	},
	
	removeSelectedArtifacts: function(){
		//Remove currently list of selections
		this._selectedItems = [];
		var selectedItems = this._resultsList.getSelection();
		
		for (var i = 0; i < selectedItems.length; i++) {
			this.filterArtifacts.push(selectedItems[i].value);
			//Instead of removing the selected artifacts via this._resultsList.removeEntry(entry). We call an added 
			//method to the jazz.ui.ListBox so set the items as disabled (greyed out), yet still displayed in the list.
			this._resultsList.setEntryDisabled(selectedItems[i]);
		}
		
		this.onArtifactSelectionChanged(this._selectedItems);	
	},

	clearSelection: function() {
		this._resultsList.setSelection([]);
	},
	
	_handleDoubleClick: function(entry) {
		// Selects the specified entry and executes the callback
		// that clicks the  OK button if it's enabled.
		if (this.enterKeyHandler) {						
			this._resultsList.setSelection([entry], true);
			this._selectionChanged(entry);					
			this.enterKeyHandler();
		} 
	},
	
	_selectionChanged: function(item) {
		
		//Display num of selected items
		
		var selectedItemsInPage = this._resultsList.getSelectedItemsInPage();

		if (selectedItemsInPage) {
			if (this._selectedResults) {
				this._matchingResults.removeChild(this._selectedResults);
				this._selectedResults = null;
			}
			var totalSelectedItems = selectedItemsInPage.length; 
			if(totalSelectedItems > 0){
				var selectedMsg="";
				if(totalSelectedItems === 1){
					selectedMsg = dojo.string.substitute(this._messages.oneArtifactSelected,[totalSelectedItems]);
				}else{
					selectedMsg = dojo.string.substitute(this._messages.numberOfArtifactsSelected,[totalSelectedItems]);
				}	
				this._selectedResults = document.createTextNode(selectedMsg);
				this._matchingResults.appendChild(this._selectedResults);
			}
		}
				
		this._selectedItems = [];
		var invokeSelectionChanged = true;
		var selectedItems = this._resultsList.getSelection();
		for (var i = 0; i < selectedItems.length; i++) {
			if(selectedItems[i].type == NOT_SELECTABLE_ENTRY){
				invokeSelectionChanged=false;
				break;
			}
			this._selectedItems.push(selectedItems[i].value);
		}
		if(invokeSelectionChanged){
			this.onArtifactSelectionChanged(this._selectedItems);
			if (selectedItems.length == 1) {
				this.onItemSelected(selectedItems[0]);
			} 
		}
		
		this._lastSelectedItem = selectedItems[0];
	},
	
	_displayArtifactList : function (artifactList, folderURI) {
		this._preventRefresh=true;			
		this._clearSelectedFolders(true);
		this._clearAttributesFilterSection(true);
	
		this._resultsList.removeAllEntries();	
		this._removeMatchingResultsNodes(false);
		dojo.removeClass(this._searchResultsNode,"message-entry");
		
		this._selectedItems = [];
		var entries = [];
		var resourceDescriptions = []; 
		dojo.forEach(artifactList, function(artifact) {
			for (var type in artifact.getTypes()) {}
			var entryType = this._contentTypeToNumber[type];
			resourceDescriptions.push(artifact);

			entries.push({
				label: dojo.string.substitute(this._messages.artifactLabel, [artifact.getID(), artifact.getTitle()]),
				value: artifact.getResourceURI(),
				id: artifact.getResourceURI(), //Use artifact resource URI as record id for ListBox element
				processAreaURI: artifact.getProcessAreaURI(),
				projectURI: artifact.projectURI,
				type: entryType,
				disabled: false
			});

			this._selectedItems.push(artifact.getResourceURI());
		}, this);
		
		this._resultsList.showLoading(false);
		this._resultsList.addEntries(entries);
		//Add the resourceInfos to and index map
		this._resetIndexedPropertiesMap(resourceDescriptions);

		var matchingResultsText = null;
		if (artifactList.length > 1) {
			matchingResultsText = dojo.string.substitute(this._messages.numberOfMatchesFound, [artifactList.length, artifactList.length]);	
		} else {
			matchingResultsText = this._messages.oneMatchFound;
		}

		this._matchingResults.appendChild(document.createTextNode(matchingResultsText));
		
		this._resultsList.setSelection(entries, true);
		
		//Fire an event as soon as results are displayed. This should force the observer buttons 
		//to refresh their state since there are no selections
		this.onArtifactSelectionChanged(this._selectedItems);
		
		if(folderURI) {
			this._foldersSection.selectFoldersByURI([folderURI]);
		}
	},
	
	_displaySingleArtifact : function(artifact, folderURI) {
		var artifactList = [];
		artifactList.push(artifact);
		this._displayArtifactList(artifactList, folderURI);
	},
	
	createArtifact: function() {
		
		var newArtifactFormParams = {
			projectURI: this.projectURI,
			hideOpen: true,
			folderURI: this.defaultFolderForArtifactCreation
		};
		
		// TODO Christian : this will go away after Art's type/format changes
		if(this.newArtifactFormat) {
			newArtifactFormParams["artifactFormat"] = this.newArtifactFormat;
		}
		
		if(this.selectArtifactFormat) {
			newArtifactFormParams["disableFormatsDropdown"] = true;
			newArtifactFormParams["artifactFormat"] = RRMTypes.getArtifactFormats(this.projectURI)[this.selectArtifactFormat[0]];
		
			//Will only populate the NewArtifactForm with Collection compatible formats/types, if appropriate.
			if(this.selectArtifactFormat[0] === "Collection"){
				newArtifactFormParams["formatsFilter"] = dojo.hitch(this, function(formatURI){  
					return (formatURI === newArtifactFormParams["artifactFormat"]);
				});
			}
		}
		// wi 75514. set newArtifactFormParams["folderURI"] to initialize starting folder of the artifact creation dialog
		if(this._foldersSection) {
			var selectedFolderResources = this._foldersSection.getSelectedFolders();
			if (selectedFolderResources.length > 0) {
				newArtifactFormParams["folderURI"] = selectedFolderResources[0];
			}
		}
		
		var newArtifactDialog = new com.ibm.rdm.web.project.NewArtifactForm(newArtifactFormParams);
		newArtifactDialog.show();
		var con = dojo.connect(newArtifactDialog, "onCreate", this, function(artifact, openArtifactInEditMode) {
			dojo.disconnect(con);
			this._displaySingleArtifact(artifact, newArtifactDialog.folderURI);
		});
	},
	
	_resetIndexedPropertiesMap: function(indexedPropertiesArray) {
		// summary:
		//		Reset the content found in this._mapResourceURIToResourceIndexedProperties.
		// indexedPropertiesArray:
		//		An array of property lists, as documented in the block comment
		//		for the ProjectCache.getBatchResourceDescriptionsMultiFetch
		//		function.
		
		if(!this._isPageRequest){
			this._mapResourceURIToResourceIndexedProperties = new Object();
		}
		
		dojo.forEach(
			indexedPropertiesArray,
			function(resourceIndexedProperties) {
				this._mapResourceURIToResourceIndexedProperties[resourceIndexedProperties.resourceURI]
					= resourceIndexedProperties;
			},
			this
		);
		
	},
		
	_beginSearchTimeout: function(value) {
		if (this._timeoutCounter) {
			clearTimeout(this._timeoutCounter);
		}
		if (this._resultsList) {
			this._timeoutCounter = setTimeout(dojo.hitch(this, this.refresh), 1000);
		}
	},
	
	getSelectedArtifacts: function() {	
		//Collect the currently selected items along with any previously selected items in a page		
		var totalSelectedItems = [];
		if (!this._selectedItems) {
			return totalSelectedItems;
		}
		var selectedItemsInPage = this._resultsList.getSelectedItemsInPage(),
			item,
			i;
		for (i = 0; i < selectedItemsInPage.length; i++) {
			item = selectedItemsInPage[i];
			//We have to make sure we don't send back any items previously selected.
			if (!item.isDisabled){
				totalSelectedItems.push(item.value);
			}
			//Set this entry as disabled so that it shows greyed out in
			//future page requests.
			item.isDisabled = true;
		}		
		for (var i=0; i < this._selectedItems.length; i++){
			if (dojo.indexOf(totalSelectedItems, this._selectedItems[i]) == -1) {
				totalSelectedItems.push(this._selectedItems[i]);
			}		
		}
		return totalSelectedItems;
	},
	
	getSelectedItemNames: function() {
		//Collect the currently selected items along with any previously selected items in a page		

		var totalSelectedItems = [],
			selectedItemsInPage = this._resultsList.getSelectedItemsInPage(),
			i;

		for (i = 0; i < selectedItemsInPage.length; i++) {
			totalSelectedItems.push(selectedItemsInPage[i].label);
		}	
		
		for (i = 0; i < this._selectedItems.length; i++) {
			if (dojo.indexOf(totalSelectedItems, this._selectedItems[i]) == -1) {
				totalSelectedItems.push(this._selectedItems[i]);
			}		
		}
		
		return totalSelectedItems;
	},
	
	getSelected: function() {
		var selectedURIs = this.getSelectedArtifacts();
		var selectedNames = this.getSelectedItemNames();
		if (this.showModuleViews) {
			for (var i = 0, l = selectedURIs.length; i < l; i++) {
				if (com.ibm.rdm.web.query.SavedFilterUtils.isSavedView(selectedURIs[i])) {
					var moduleName = this._viewsToModuleNames[selectedURIs[i]];
					selectedNames[i] = dojo.string.substitute(
							this._messages.moduleViewLabel, [moduleName, selectedNames[i]]);
				}
			}
		}
		return {
			selectedURIs: selectedURIs,
			selectedNames: selectedNames
		};
	},
	
	showMessage: function(strMsg){
		var self=this;
		this._resultsList.removeAllEntries();
		var messageEntry=[];
		dojo.addClass(this._searchResultsNode,"message-entry");
		messageEntry.push({
			label: strMsg,
			value: "",
			id: 0,
			type: NOT_SELECTABLE_ENTRY
		});
		this._resultsList.addEntries(messageEntry);
	},
	
	displayLoadingMsg: function(){
		var self=this;
		this._resultsList.removeAllEntries();
		var messageEntry=[];
		dojo.addClass(this._searchResultsNode,"message-entry");
		messageEntry.push({
			label: this._messages.loadingArtifactsMessage,
			value: "",
			id: 0,
			type: NOT_SELECTABLE_ENTRY
		});
		this._resultsList.addEntries(messageEntry);
	},
	
	getPageToDisplay: function() {
		//Summary: this returns the page that is the one that 'should' be displayed.
		//         This may or may not be the one that is being displayed.
		return this._pageToDisplay;
	},
	
	resetPageToDisplay: function() {
		//Summary: this resets the page to display to 1.
		this._pageToDisplay = 1;
		//We only care about keeping track of selections accross pages. 
		//Reset the selections if this is a new page.
		this._resultsList.clearSelectedItemsInPage();
	},
	
	getPageSize: function() {
		//Summary: returns the current page size.
		return this._pageSize;
	},
	
	filtersChangedRefresh: function(){
		this.refresh(false);
	},

	persistFolderSelections: function(selectedFolderResources){		
		var selectedFolderString="";
		for(var i=0; i < selectedFolderResources.length; i++){
			//Store the current project along with the folder so that we can 
			//correlate stored folders to current project scope.
			selectedFolderString+= this.projectURI + "|" + selectedFolderResources[i]; 
			if(i != selectedFolderResources.length - 1){
				selectedFolderString+=";";
			}
		}
		//Persist selected folders
		com.ibm.rdm.web.util.storage.set({
			"key": LAST_FOLDER_COOKIE,
			"data": selectedFolderString,
			"cookieFallback": true
		});
	},

	getLastSelectedFolders: function(){
		// Get the cookie with the value
		// "lastFoldersSelected_artifactPicker" and split the
		// string by the delimiter ";".
		var cookieValue = com.ibm.rdm.web.util.storage.get({
			"key": LAST_FOLDER_COOKIE,
			"cookieFallback": true
		});
		if (cookieValue) {
			var folderArray = cookieValue.split(";");
			var selectedFolders =[];
			
			//Selected folders are stored as <projectURI>|<folderURI>, go through the found
			//entries and extract the folderURI portion. Note that we are only intersted in 
			//the folder selections ofr the current project 
			for(var i=0; i < folderArray.length; i++){
				var folderEntry = folderArray[i].split("|");			
				var projectURI = folderEntry[0];
				var projectFolderURI = folderEntry[1];
				//Do not return folder selections for projects other than the 
				//current project.
				if(projectURI != this.projectURI){
					return null;
				}
				//Collect the selected folders
				selectedFolders.push(projectFolderURI);				
			}
			return selectedFolders;
		} else {
			return null;
		}
	},
	
	changeModuleScope: function(moduleURI) {
		this.moduleURIScope = moduleURI;
		this._lastSelectedItem = null;
		this._pageSize = MAX_MODULE_PAGE_SIZE;
		
		if (moduleURI) {
			this._pageSize = MAX_MODULE_PAGE_SIZE;
		}
		else {
			this._pageSize = MAX_PAGE_SIZE;
		}
		this._foldersSection._removeSelection();
//		this._clearSelectedFolders(false);
//		this.refresh(false);
	}
	
});

})();
