/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013, 2014. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.defects.web.ui.internal.ReqProLinksTableView");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.DefaultVirtualContentProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableSorter");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");

dojo.require("com.ibm.rqm.defects.web.ui.utils.DefectsUIUtil");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");

dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.rqm.planning.web.ui.internal.model.action.BulkChangeAction");
dojo.require("com.ibm.rqm.requirement.web.ui.internal.view.requirement.ReqProRequirementTableViewer");
dojo.require("com.ibm.rqm.defects.web.ui.utils.CollectionLinksUtil");

dojo.require("jazz.app.context");

(function(){
    
    var ImageCache = com.ibm.asq.common.web.ui.internal.view.common.ImageCache;
    var TableConstants = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;
    var DefectsUIUtil = com.ibm.rqm.defects.web.ui.utils.DefectsUIUtil;
    var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
	var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
	var CollectionLinksUtil = com.ibm.rqm.defects.web.ui.utils.CollectionLinksUtil;
    var getCurrentProjectArea = jazz.app.context.get;

	dojo.declare("ReqProLinksTableView.TableListener", com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableListener, {
        constructor: function(args){
            dojo.mixin(this, args);
        },
        
        rowCheckStateChanged: function(tableViewer, rowObject, state){
        	var selection = tableViewer.getCheckedRows();
			var realSelection = dojo.isArray(selection);
			var selectionCount = realSelection ? selection.length : 2; // use 2 to indicate select all.
            
            if (this.parentView.deleteActionNode){ 
                this.parentView.deleteActionNode.setDisabled(selectionCount == 0 || !tableViewer.isEditable());
            }
            
            if(this.parentView.createTestCaseActionNode){
            	this.parentView.createTestCaseActionNode.setDisabled(selectionCount == 0 || !tableViewer.isEditable());
            }
			var content = tableViewer.getContent();
//			if(this.parentView.syncViewActionNode){
//				this.parentView.syncViewActionNode.setDisabled((selection.length && content.length != selection.length) || !tableViewer.isEditable())
//			}
			if(this.parentView.syncLinkActionNode){
				this.parentView.syncLinkActionNode.setDisabled(selectionCount == 0 || !tableViewer.isEditable() || !this.parentView.isReqProProjectLinked)
			}
			 if (this.parentView.findAssociatedTestCaseActionNode){ 
                this.parentView.findAssociatedTestCaseActionNode.setDisabled(selectionCount != 1 || !tableViewer.isEditable());
            }
            
        },
        
        groupRowsCheckStateChanged: function(tableViewer, rowObjects, state) 
    	{
        	this.rowCheckStateChanged(tableViewer, rowObjects, state);
   		},
	
                
        tableContentActionInvoked: function(tableViewer, actionId, callbackData, callback){
            if (callback != null && dojo.isFunction(callback)) {
                if (callbackData != null) {
                    callback(callbackData);
                }
                else {
                    callback();
                }
            }
        },
		
		tableHeaderColumnActionInvoked: function(tableViewer, columnIndex, /*optional*/actionId)
	    {
			if (this.parentView && this.parentView.requirementHeaderColumnActionInvoked) {
				this.parentView.requirementHeaderColumnActionInvoked(tableViewer, columnIndex, actionId);
			}         
	    }   
	  });


    
	dojo.declare("com.ibm.rqm.defects.web.ui.internal.ReqProLinksTableProvider", com.ibm.asq.common.web.ui.internal.widgets.tableViewer.DefaultVirtualContentProvider, {

		isVirtualTable: true,
		currentReqProConnection: null,
		
		preDefinedColumns: null,
		showCollections: false,
		
		showTypeColumn: true,

       	constructor: function(args) 
    	{
        	dojo.mixin(this, args); 
			
			this._columnHeadings = [
				"",
				Constants.getMessage("labelTagRequirementTable"),
				Constants.getMessage("labelName"),
		       	Constants.getMessage("labelDescriptionRequirementTable")
			];
			this._columnIdNames = [
				 'type',
				 'tag',
				 'name',
		       	 'summary'
		    ];
			
			if(!this.showTypeColumn){
				this._columnHeadings.shift() ;
				this._columnIdNames.shift();
			}
			
			//Fix defect 64847: ReqPro project name is "undefined:undefined"
			//don't show the project name column in some cases, i.e., in the RequirementExplorer widget. 
			if(this.notShowProjectAreaColumn){
				if(this.showTypeColumn){
					this._columnWidths = [4,0,46,50];
				}else{
					this._columnWidths = [0,50,50];
				}
			} else {
				this._columnHeadings.push(Constants.getMessage("labelProjectName"));
				this._columnIdNames.push('project');
				if(this.showTypeColumn){
					this._columnWidths = [4,0,30,50,16];
				}else{
					this._columnWidths = [0,30,50,20];
				}
				
			}    
    	},

		getNumColumns: function(content){
			return this._columnHeadings.length;
        },
		
		getColumnHeading: function(content, index)
    	{
    		
    		if(this.preDefinedColumns && this.preDefinedColumns[index - 1] && index != 0){
    			return this.preDefinedColumns[index - 1];
    		}
    		
        	return this._columnHeadings[index];
    	},
		
		getColumnIdName: function(columnIndex){
        	return this._columnIdNames[columnIndex];
    	},
		
        getColumnHeadingImage: function(content, atColumnIndex){
			return null;
        },
		getIsColumnVisible: function(atColumnIndex){
			
			if(!this.showTypeColumn){
				atColumnIndex ++;
			}

        	if(this.showCollections && atColumnIndex == 1 //Tag column invisible by default if showing Collections
        		|| !this.showCollections && atColumnIndex == 4) // or Project Name column invisible when showing Requirements
        		return false;
			return true;
	    },
		
        getColumnHeadingAlignment: function(atColumnIndex){
            return this.getCellAlignment(null, atColumnIndex);
        },
        getIsColumnGroupable: function(content, atColumnIndex){
        	
        	if(!this.showTypeColumn){
        		atColumnIndex ++;
        	}
        	
        	if(this.showCollections && atColumnIndex == 4)
        		return true;
            return false;
        },
        getIsColumnSortable: function(content, atColumnIndex){
        	// Since Reqpro does not support sort in API so the column sort is disable.
			// Unless sorting is specifically allowed when the table viewer is in the requirement section
			// without server side paging, we can use client side sorting.
       		return this.allowSorting ? true : false;
        },
		
        getColumnWidth: function(content, atColumnIndex){	
        	
        	if(this.showTypeColumn && !this.showCollections && atColumnIndex == 1){
        		return 16;
        	}
        	else if(!this.showTypeColumn && !this.showCollections && atColumnIndex == 0){
        		return 20;
        	}
        	
    		return this._columnWidths[atColumnIndex];
        },
        
        getContentObjects: function(content){
            return content;
        },
        
        getCellDisplayType: function(rowData, atColumnIndex){
        	
        	if(!this.showTypeColumn){
        		atColumnIndex ++;
        	}
        	
        	if(atColumnIndex == 0){
        		return TableConstants.RENDER_TYPE_IMAGE;
        	}else if(atColumnIndex == 2 && rowData.type == "requirement"){
        		return "RENDER_TYPE_RICHHOVERLINK";
        	}

            return TableConstants.RENDER_TYPE_TEXT;
        },
		
        getCellAlignment: function(rowData, atColumnIndex){
        	
        	if(!this.showTypeColumn){
        		atColumnIndex ++;
        	}
        	
        	if(atColumnIndex == 0){
        		return TableConstants.ALIGN_CENTER;
        	}
        	
            return TableConstants.ALIGN_LEFT;
        },
		
        getCellDisplayData: function(rowData, atColumnIndex){
            var val = "";
            
            if(!this.showTypeColumn){
            	atColumnIndex ++;
            }
            
            switch (atColumnIndex) {
				case 1:
					val = rowData.tag;
					break;
				case 4:
					// Display ServerName: ProjectName
					if(rowData.reqProServerName && rowData.reqProProjectName){
						val = rowData.reqProServerName + ": " + rowData.reqProProjectName;
					} else if(rowData.srcId) {
						var srcId = rowData.srcId;
			  			srcId = srcId.substring(srcId.indexOf("/") + 1, srcId.lastIndexOf("/"));
						var projectName = unescape(srcId.substring(srcId.indexOf("/") + 1));
						val = projectName;
					} 
					break;
				case 2: 
					if(rowData.name && rowData.name != ""){
						val = rowData.name;
					}else{
						val = rowData.description;
					}
	                break;   
				case 3: 
	                val = rowData.description;
	                break;
            }
            return val;
        },
                
        getCellDisplayExtendedInfo: function(rowData, atColumnIndex){
            
          var val = null;
          
          if(!this.showTypeColumn){
          	atColumnIndex ++;
          }
          
          if(atColumnIndex == 0){
		  	
		  	val = {};
			
			if(rowData.type == "view"){
				val.image = ImageCache.REQPRO_VIEW;
				val.tooltip = Constants.getMessage("labelReqProView");

			}else if(rowData.type == "package"){
			    val.image = ImageCache.REQPRO_PACKAGE;
			    val.tooltip = Constants.getMessage("labelReqProPackage");	
			}else{
				val.image = ImageCache.REQPRO_REQUIREMENT;
				val.tooltip = Constants.getMessage("labelReqProRequirement"); 
			}

		  }else if(atColumnIndex == 1){
		  	val = {};
			val.label = rowData.tag;
		  } 
		  // atColumnIndex = 2 for ProjectName column
		  else if(atColumnIndex == 2){
		  	
		  	if(rowData.type == "requirement"){
		  		val = {};
		  		
                // rich hover for ReqPro requirements is provided from RQM's OSLC service 
                val.uri = location.protocol
                       + "//" + location.host + net.jazz.ajax._contextRoot
                       + "/oslc_qm/contexts/" + getCurrentProjectArea().itemId
                       + "/resources/com.ibm.rqm.requirement.ReqProRequirement/";
                                  
                // if the requirement has already been imported into RQM then it has an itemId                   
		  		if (rowData.itemId != null) {
		  		  val.uri += rowData.itemId;
		  		}
		  		
		  		
		  		// otherwise the requirement has not been imported yet so we have
		  		// to provide extra hints to the renderer
		  		else {
		  		  var projectItemid;
		  		  for(var index in this.currentReqProConnection.projects) {
		  		      if (this.currentReqProConnection.projects[index].name == this.currentReqProProject) {
		  		            
		  		        	projectItemid = this.currentReqProConnection.projects[index].itemId;
		  		        	break;
		  		        }
		  		    }
		  		    
		  		  val.uri += "_PKIHir3vEeCreLvF4XVd0A" // This UUID is hard-coded in ReqProRequirementRenderer
		  		          + "?reqproProjectItemId="
                          + projectItemid
                          + "&reqproReqKey="
                          + rowData.reqKey
                          + "&reqproReqName="
                          + encodeURIComponent(rowData.name);
		  		} 
                    
                val.label = rowData.name;
                
		  		if(rowData.srcId){
		  			val.presentationUri = CollectionLinksUtil.getReqProItemUrl(rowData.srcId);
		  		}else if(this.currentReqProConnection){
		  			var href = this.currentReqProConnection.protocol;
		  			href += "://";
		  			href += this.currentReqProConnection.hostname;
		  			href += ":";
		  			href += this.currentReqProConnection.port;
		  			href += "/ReqWeb/ReqTabbedProp_Page.jsp?txtProject=" + this.currentReqProProject + "&ReqKey=" + rowData.reqKey + "&shortCut=true&showMainFrame=true&txtUID=<TypeYourUserID>&txtPWD=<TypeYourPassword>";
                    val.presentationUri = href;
		  		}
		  	}
		  }

          return val;
          
        },
		
		isVirtualContent: function(){
        	return this.isVirtualTable;
    	},
    	
    	getRowDataId: function(rowData){
        	return rowData.id;
    	}
    	
    });
    
    dojo.declare("com.ibm.rqm.defects.web.ui.internal.ReqProLinksTableViewRowCheckedHandler", TableViewer.RowCheckedHandler,{
    
    	operator: null,
   
    	
    	setInVirtualSelectAll: function(inSelectAll){
			this.inherited("setInVirtualSelectAll", arguments);
			
			if(this.operator){
				this.operator._notifySelectionChanged();
			}
			
    	},
    
    	setRowCheckState: function(rowObject, checkState, updateVisual){
    		this.inherited("setRowCheckState", arguments);
    		
    		if(this.operator){
    			this.operator._notifySelectionChanged();
    		}
    	},
    	
    	
    	clearRowStates: function(){
    		this.inherited("clearRowStates", arguments);
    		if(this.operator){
    			this.operator._notifySelectionChanged();
    		}	
    	},
    
    	getRowCheckState: function(rowObject)
    	{
        	var provider = this.viewer.getContentProvider();
        	var id = provider.getRowDataId(rowObject);
        	var check = false;
        
        	// virtual select all, if the item is in our state 
        	// map, it was de-selected.
        	if(this.inVirtualSelectAll) {
           		return (typeof this.rowStates[id] == "undefined");
        	}
        	if(id && (typeof this.rowStates[id] != "undefined")) {
           		check = this.rowStates[id].checked;
        	} else if (rowObject && rowObject.selected) {
				this.setRowCheckState(rowObject, true);
				return true;
			}
        	return check;
    	}	
    	
    
    	
    });
    
    
    dojo.declare("com.ibm.rqm.defects.web.ui.internal.ReqProLinksTableView", [dijit._Widget, dijit._Templated], {
        templateString: '<div dojoAttachPoint="_tableViewerNode" tabindex="-1">' +
        '</div>',
        
        widgetsInTemplate: true,
        editable: true,
		hideResetButton: false,
		inDialogMode: false,
		
		preDefinedSearchFun: null,
    	preDefinedSearchCallBackFun: null,
    	preDefinedBuildQueryParamFun: null,
    	preDefinedNoItemMessage: null,
    
   		preDefinedSearchResultHandler: null,
    
    	rowsCheckable: true,
    
    	operator: null,
    	
    	isVirtualContent: true,
    	
    	showCollections: false,
    	showTypeColumn: true,
    	
    	currentReqProProject: null,
    	
    	currentReqProConnection: null,
    	
    	preDefinedColumns: null,
    	
        constructor: function(){
			this.reqLinks = [];
        },
        
        postCreate: function(){
            
            this.inherited("postCreate", arguments);
			
        	var tableViewer = new com.ibm.rqm.requirement.web.ui.internal.view.requirement.ReqProRequirementTableViewer({
            	//turn off all controls if in print mode
            	isCompactMode: !this.printable, 
				showPagingControl: !this.printable,
				showPagingRangeControl: !this.printable,
           	 	showGroupFilterControls: !this.printable,
            	showPagingActionsControls: !this.printable,
            	allowAdjustSettings: !this.printable,
            	autoSaveSettings: !this.printable,
            	handleVirtualSelectAll: true,
            	operator: this.operator,
            	displayInPopup: this.inDialogMode,
            	tableId: this.tableId ? this.tableId : "ReqproLinksTable",   
            	defaultIndividualSelectionStatus: this.defaultIndividualSelectionStatus,
            	runQuery: dojo.hitch(this, "runQuery"),
            	preDefinedNoItemMessage: this.preDefinedNoItemMessage,
            	allowMultiSort: this.allowSorting ? true : false
            	
        	});
        	
        	this.isReqProProjectLinked = true;
        	
            this._tableViewerNode.appendChild(tableViewer.domNode);
            tableViewer.setTableSummary(Constants.getMessage("labelTableSummary", 
            		this.showCollections  ? Constants.getMessage("labelReqProViewsPackages") : Constants.getMessage("labelReqProRequirement")));
           
			var content = this.reqLinks ? this.reqLinks : [];
            tableViewer.setContentPageSize(this.printable?-1:5);
            tableViewer.setContent(content);
            
            this.provider = new com.ibm.rqm.defects.web.ui.internal.ReqProLinksTableProvider({
            	tableViewer: tableViewer,
            	preDefinedColumns: this.preDefinedColumns,
            	isVirtualTable: this.isVirtualContent,
            	currentReqProConnection: this.currentReqProConnection,
        		currentReqProProject: this.currentReqProProject,
        		showCollections: this.showCollections,
        		showTypeColumn: this.showTypeColumn,
        		allowSorting: this.allowSorting,
        		vcFetchVirtualPage: dojo.hitch(this, function(pageIndex, pageSize)
            	{
	                this.runQuery();
            	}),
            	
            	vcRunQuery: dojo.hitch(this, function(pageSize)
            	{
                	this.runQuery();
            	}),
            	
            	vcPerformFilterBy: function(filterText){
            		
            		this.tableViewer.showLoadingProgress();
            		
            		if(!filterText){
            			
            			this.tableViewer.setContent(this.filterContent);
            			this.totalContentCount = this.filterTotalContentCount;
            			this.totalPageCount = this.filterTotalPageCount;
            			this.currentPageIndex = this.filterCurrentPageIndex;
            			this.filterContent = null;
            			this.tableViewer.refresh();
            			this.tableViewer.endLoadingProgress();
            			return;
            			
            		}
            		
            		if(!this.filterContent){
            			this.filterContent = this.tableViewer.getContent();
            			this.filterTotalContentCount = this.totalContentCount;
            			this.filterTotalPageCount = this.totalPageCount;
            			this.filterCurrentPageIndex = this.currentPageIndex;
            		}
            		
            		var content = this.filterContent;
            		
            		if(!content)
            			return;
            			
            		var filteredReqs = [];
            			
            		for(var step = 0;step < content.length;step ++){
            			
            			var req = content[step];
            			
            			if(req.tag){
            				if(req.tag.indexOf(filterText) != -1){
            					filteredReqs.push(req);
            					continue;
            				}
            			}
            			
            			if(req.name){
            				if(req.name.indexOf(filterText) != -1){
            					filteredReqs.push(req);
            					continue;
            				}
            			}
            			
            			if(req.description){
            				if(req.description.indexOf(filterText) != -1){
            					filteredReqs.push(req);
            					continue;
            				}
            			}
            			
            		}
            		
            		this.tableViewer._pendingFilterBy = null;
            		this.totalContentCount = filteredReqs.length;
            		this.totalPageCount = 1;
            		this.currentPageIndex = 0;
            		this.tableViewer.setContent(filteredReqs);
            		this.tableViewer.refresh();
            		this.tableViewer.endLoadingProgress();
            	}, notShowProjectAreaColumn: this.notShowProjectAreaColumn
        	
        	
        	});          

            tableViewer.setContentProvider(this.provider);
            tableViewer.setTableListener(com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableListener({//new ReqProTableView.TableListener({
                parentView: this
            }));
            
            tableViewer.setTableListener(new ReqProLinksTableView.TableListener({
                parentView: this
            }));

            tableViewer.setTableRowsCheckable(this.rowsCheckable, /* isSingleSelect */ false);
            
            tableViewer.rowCheckedHandler = new com.ibm.rqm.defects.web.ui.internal.ReqProLinksTableViewRowCheckedHandler(tableViewer);
            tableViewer.rowCheckedHandler.operator = this.operator;
            
            if (this.allowSorting) {
                // all columns are text, use default sorter
                tableViewer.setSorter(new com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableSorter());
            }
            
            tableViewer.show();
            
            // Enh 59013 - Display the Views and Packages grouped by Project Name
            if(this.showCollections) {
            	tableViewer.headerRenderer._setGroupByColumn(4);
            }
            
            for (var i = 0; i < (this.actions ? this.actions.length : 0); i++) {
                var action = this.actions[i];
                if (action.label == DefectsUIUtil.getMessage("labelRemoveRmLinks") || action.label == Constants.getMessage("labelDisassociateRequirements")) {
                    this.deleteActionNode = tableViewer.addAction(action.label, action.label, null, action.iconClass, action.actionHandler);
                    this.deleteActionNode.setDisabled(true);
                }
                else if(action.label == DefectsUIUtil.getMessage("lableResetRequirementLinkStatus")){
					this.clearStatusActionNode = tableViewer.addAction(action.label, action.label, null, action.iconClass, action.actionHandler);
                    this.clearStatusActionNode.setDisabled(true);
				}else if (action.label == Constants.getMessage("labelSyncReq")) {
                    
                        this.syncLinkActionNode = tableViewer.addAction(action.label, action.label, null, action.iconClass, action.actionHandler);
						this.syncLinkActionNode.setDisabled(true);
                }else if (action.label == DefectsUIUtil.getMessage("titleSyncReqViewAndPackage")) {
                    
                        this.syncViewActionNode = tableViewer.addAction(action.label, action.label, null, action.iconClass, action.actionHandler);
                        //Always enable it, because the sych is not based on the selected reqpro requirement views but defaultly all of them.
						this.syncViewActionNode.setDisabled(false);
                }else if(action.label == Constants.getMessage("labelCreateTestCases")){
                	this.createTestCaseActionNode = tableViewer.addAction("", action.label, null, action.iconClass, action.actionHandler);
                	this.createTestCaseActionNode.setDisabled(true);
                } else if (action.label == Constants.getMessage("labelAssociatedTestCases")){
                	this.findAssociatedTestCaseActionNode = tableViewer.addAction("", action.label, null, action.iconClass, action.actionHandler);
                	this.findAssociatedTestCaseActionNode.setDisabled(true);
                }
                else 
				{
                     var actionButton = tableViewer.addAction("", action.label, null, action.iconClass, action.actionHandler);
                     if (action.label == DefectsUIUtil.getMessage("labelAddRmLinks")) {
                         this.addRmLinksActionNode = actionButton;
                     } else if (action.label == Constants.getMessage("labelAssociateRequirements")) {
                         this.addReqActionNode = actionButton;
                     }
                     actionButton.setDisabled(!this.editable);
                }
            }

            tableViewer.setEditable(this.editable);
            this.tableViewer = tableViewer;
            
        },
        
        performSelectAll: function(select){
        	this.tableViewer.performSelectAll(select);
        },

        init: function(){
        	this.provider.pageSize = TableConstants.MIN_PAGE_SIZE;
        	this.runQuery(); 	
        },
        
        runQuery: function(){
        	
        	this.tableViewer.showLoadingProgress();
           		
        	var self = this;
        	
        	var callback = {
            	success: function(results){
                	self.showQueryResults(results);
            	},
            	failure: function(error){
                	self.showQueryError(error);
            	}
        	}
        
        	var responseHandler = new ServiceResponseHandler(callback, "success", "failure");	
        	
        	if(dojo.isFunction(this.preDefinedSearchFun)){
        		this.preDefinedSearchFun(responseHandler, []);
        	}
        },
        
        showQueryResults: function(resultsDTO){

        	if(!resultsDTO){
        		resultsDTO = [];
        	}
        	
        	if(dojo.isFunction(this.preDefinedSearchResultHandler)){
        		this.preDefinedSearchResultHandler(resultsDTO);
        	}
        	
        	var pageResults = [];
        	
        	if(resultsDTO.results){
        		
        		//for "Show All"
        		if(resultsDTO.pageSize == -1)
        			resultsDTO.pageSize = resultsDTO.results.length;
        		
        		if(this.provider.isVirtualTable){
        			
        			for(var step = 0;step < resultsDTO.results.length && step < resultsDTO.pageSize;step ++){
        				pageResults.push(resultsDTO.results[step]);		
        			}
        		}

        		else{
        			for(var step = 0;step < resultsDTO.results.length;step ++){
        				pageResults.push(resultsDTO.results[step]);		
        			}
        		}
        		
        	}
        	this._processResults(resultsDTO);
        	
			this.provider.loadPageContent(pageResults);
        },
        
        showQueryError: function(errorMessage){
        	this.tableViewer.endLoadingProgress();
        	var resultsDTO = [];
        	
        	if(dojo.isFunction(this.preDefinedSearchResultHandler)){
        		this.preDefinedSearchResultHandler(resultsDTO);
        	}
        	
        	this._processResults(resultsDTO);
        	
			this.provider.loadPageContent(resultsDTO.results);
        },
        
        refresh: function(links){
            this.tableViewer.setContent(links);
            this.tableViewer.refresh();
        },
        
        getContent: function(){
            return this.tableViewer.getContent();
        },

        getCheckedRows: function(){
            return this.tableViewer.getCheckedRows();
        },
                
        _setCurrentPageIndex: function(index){
        	this.provider.currentPageIndex = this.currentPage = index;
        },
        
        _setTotalContentCount: function(totalCount){
        	
        	var provider = this.provider;
        	
        
        	var totalPageCount = 1;
        	
        	if(provider.pageSize != -1){
        		totalPageCount = totalCount/provider.pageSize;
        	}       
        	
        	totalPageCount = parseInt(totalPageCount);
        
        	if((totalCount%provider.pageSize) > 0){
        		totalPageCount ++;
        	}
        
        	provider.totalPageCount = this.totalPages = totalPageCount;
        	provider.totalContentCount = this.totalSize = totalCount;
        },
        
        _loadContent: function(items){
        	this.provider.loadPageContent(items);
        },
        
        _processResults: function(resultsDTO){
        	
        	if(!resultsDTO){
        		return;
        	}
        	
        	var provider = this.provider;

        	provider.currentPageIndex = this.currentPage = resultsDTO.currentPage;
        	
        	var totalPageCount = 1;
        	
        	if(provider.pageSize != -1){
        		totalPageCount = resultsDTO.totalRequirments/provider.pageSize;
        	}
        	
        	totalPageCount = parseInt(totalPageCount);
        
        	if((resultsDTO.totalRequirments%provider.pageSize) > 0 && provider.pageSize != -1){
        		totalPageCount ++;
        	}
        
        	provider.totalPageCount = this.totalPages = totalPageCount;
        	provider.totalContentCount = this.totalSize = resultsDTO.totalRequirments;
    	},
    	
    	getSelectedRows: function(){
    		return this.tableViewer.getCheckedRows();
    	},
    	
    		//When user clicks "select all", have to gather the deselected rows in the table.
		getVirutalDeselectedItems: function(){
			return this.tableViewer.getVirtualDeselectedRows();
		},
		
		setActionByCheckingReqProProject: function(isReqProProjectLinked) {
		    this.isReqProProjectLinked = isReqProProjectLinked;
		    if (this.addRmLinksActionNode) {
		        this.addRmLinksActionNode.setDisabled(!isReqProProjectLinked);
		    }
		    if (this.syncViewActionNode) {
		        this.syncViewActionNode.setDisabled(!isReqProProjectLinked);
		    }
		    if (this.addReqActionNode) {
		        this.addReqActionNode.setDisabled(!isReqProProjectLinked);
		    }
		}

	  });
    
})();
