/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2014. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.util.ActionBarRegistry");

dojo.require("jazz.core.loader");
dojo.require("jazz.app.context");

dojo.require("com.ibm.team.repository.web.client.init.InitializationData");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.rqm.planning.web.client.internal.HiddenMenuClient");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");

(function(){
	var InitializationData    = com.ibm.team.repository.web.client.init.InitializationData;
	var Constants             = com.ibm.rqm.planning.web.client.internal.Constants;
	var HiddenMenuClient      = com.ibm.rqm.planning.web.client.internal.HiddenMenuClient;
	var ServiceResponseHandler= com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
	
	dojo.declare("com.ibm.rqm.planning.web.ui.internal.util.ActionBarRegistry", null, {
		
	    constructor: function(args) {	
            //mixin any properties
			dojo.mixin(this, args);
			
            this._registry = [];
            this._allModules = [];
			this._initialized = false;
			this._initializedCallback = null;
			this._initialize();			
	    },
		
        ////////////////////////////////////////////////////////////////////////////////////////////////////
        //                                             API                                                //
        ////////////////////////////////////////////////////////////////////////////////////////////////////        
        getActionBarEntries: function() {
            return this._registry || [];
        },
        
        /**
         * Return true if the menu with the specified id is configured to be entirely hidden
         * 
         * @param menuId
         * @returns {Boolean}
         */
        isMenuHidden: function(menuId) {
            return this._hiddenMenus && this._hiddenMenus[menuId] && this._hiddenMenus[menuId]['ALL'];
        },
        
        /**
         * Return the menu items that are configured to be hidden in the menu with the specified id
         * 
         * @param menuId
         * @returns
         */
        getHiddenMenusConfig: function(menuId) {
            return (this._hiddenMenus && this._hiddenMenus[menuId]) ? this._hiddenMenus[menuId] : null;
        },
		
		/**
		 * Sets a callback to be invoked when initialization of the contributed widgets
		 * is complete.  In most cases, it will be called immediately and synchronously.
		 * If contributed widgets load slowly, will be called when load is completed
		 */
		setInitCallback: function(callback) {
			if (callback != null && dojo.isFunction(callback)) {
				this._initializedCallback = callback;
				if (this.isInitialized()) {
					this._doInitCallback();
				}
			}
		},
		
		isInitialized: function() {
			return this._initialized;
		},

        ////////////////////////////////////////////////////////////////////////////////////////////////////
        //                                           INTERNALS                                            //
        ////////////////////////////////////////////////////////////////////////////////////////////////////        		
	    _initialize: function() {
			var planningData = InitializationData.get(Constants.REGISTRY_INIT_DATA_KEY);
            if (!planningData) {
                throw new Error("InitializationData missing required property: " + Constants.REGISTRY_INIT_DATA_KEY);
            }
			var registry = planningData ? planningData.actionBarRegistry : null;
            if (!registry) {
                throw new Error("InitializationData missing required property: actionBarRegistry");
            }
			
	        this._registry = registry;
	        
	           var self = this;
	            var handlerObj = {
	                success: function(hiddenMenus) {
						if(hiddenMenus){
							// there is hidden menus config for this project area, convert the config data for easy lookup
							self._hiddenMenus = {};                
	                		for (var x = 0; x < hiddenMenus.length; x++) {
	                    		var menu = hiddenMenus[x];
	                    		var hiddenMenu = self._hiddenMenus[menu.id] = {};
	                    		if (menu.menuItems) {
	                        		// some menu items are hidden
	                        		for (var y = 0; y < menu.menuItems.length; y++) {
	                            		var hiddenMenuItem = menu.menuItems[y];
	                            		hiddenMenu[hiddenMenuItem.id] = true;
	                        		}
	                    		} else {
	                        		// no menu item is defined if entire menu is hidden
	                        		hiddenMenu['ALL'] = true;
	                    		}
	                		}
	                	}
	                	self._loadModules();
	                },
	                
	                failure: function(e) {
	                    console.error(e);
	                    self._loadModules();
	                },
	            };
	            var responseHandler = new ServiceResponseHandler(handlerObj, "success", "failure");
	            HiddenMenuClient.getHiddenMenusEntries(responseHandler);			
		    },

	    
		/*
		 * dynamically loads all contributed action bar entries (if needed)
		 */
		_loadModules: function() {
			//gather any modules references from the action bar registry that need to be dynamically loaded
            this._allModules = [];
			for (var i = 0; i < this._registry.length; i++) {
				var jsClass = this._registry[i].jsClass;
				if (jsClass && !dojo.getObject(jsClass) && !this._contains(this._allModules, jsClass)) {
					this._allModules.push(jsClass);
				}
			}
	
			if (this._allModules.length > 0) {
				//console.debug(this.declaredClass + ": Loading " + this._allModules.length + " required Modules...");
				var callback = dojo.hitch(this, "_initComplete", this._allModules);
				
				jazz.core.loader.batch_load_async(this._allModules, callback);
			} else {
				//console.debug(this.declaredClass + ": No Modules to Load...");
				this._initComplete();
			}
		},
		
		
		_initComplete: function(modules) {
//			if (modules && modules.length > 0) {
//				console.debug(this.declaredClass + ": " + modules.length + " Modules Loaded Successfully...");
//			}
			this._initialized = true;
			this._doInitCallback();
		},		
		
		_doInitCallback: function() {
			if (this._initializedCallback != null && dojo.isFunction(this._initializedCallback)) {
				//console.debug(this.declaredClass + ":_doInitCallback --> invoking registered callback...");
				this._initializedCallback();
			}
		},
		
		_contains: function(/* String[] */values, /* String */ value) {
			if (dojo.isArray(values)) {
				for (var i = 0; i < values.length; i++) {
					if (values[i] == value) {
						return true;
					}
				}
			}
			return false;
		}	
	});

})();
