/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013, 2014. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.administration.categories.CategoryTree");

dojo.require("com.ibm.asq.common.web.client.internal.PermissionsClient");

dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.navigator.TreeView");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.navigator.NodeTypes");

dojo.require("com.ibm.rqm.planning.web.client.internal.CategoryClient");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");

dojo.require("com.ibm.rqm.planning.web.ui.internal.util.EditorUtil");

dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.categories.CategoryConstants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.categories.CategoryDialog");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.categories.CategoryTreeActionRegistry");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.administration.categories.CategoryTreeModel");
dojo.require("com.ibm.asq.common.web.ui.internal.util.Utility");
dojo.require("com.ibm.asq.common.web.ui.internal.util.BidiUtil");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit.tree.dndSource");

(function(dojo) {
	
	var PermissionsClient      = com.ibm.asq.common.web.client.internal.PermissionsClient;
	
	var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
	var AlertDialog            = com.ibm.asq.common.web.ui.internal.widgets.AlertDialog;
	var YesNoDialog	           = com.ibm.asq.common.web.ui.internal.widgets.YesNoDialog;
	
	var NodeTypes              = com.ibm.asq.common.web.ui.internal.widgets.navigator.NodeTypes;
	var TreeView               = com.ibm.asq.common.web.ui.internal.widgets.navigator.TreeView;
	
	var CategoryClient				= com.ibm.rqm.planning.web.client.internal.CategoryClient;
	var Constants		            = com.ibm.rqm.planning.web.client.internal.Constants;
	var EditorUtil                  = com.ibm.rqm.planning.web.ui.internal.util.EditorUtil;
	var CategoryConstants           = com.ibm.rqm.planning.web.ui.internal.view.administration.categories.CategoryConstants;
	var CategoryTreeActionRegistry  = com.ibm.rqm.planning.web.ui.internal.view.administration.categories.CategoryTreeActionRegistry;
	var CategoryDialog              = com.ibm.rqm.planning.web.ui.internal.view.administration.categories.CategoryDialog;
	var BidiUtil 					= com.ibm.asq.common.web.ui.internal.util.BidiUtil;	
	var Utility 					= com.ibm.asq.common.web.ui.internal.util.Utility;
	var getMessage = dojo.hitch(Constants, "getMessage");
	
	var CategoryTreeMessages = {};
	dojo.forEach([
		"labelRemoveSubtype",
		"labelError",
		"msgCannotDeleteType",
		"labelConfirmation",
		"msgArchiveCategoryType",
		"msgArchiveCategory",
		"msgNewRequiredSubcategory"
	], function(key) {
		CategoryTreeMessages[key] = getMessage(key);
	});
	
	dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.administration.categories.CategoryTree", [dijit._Widget, dijit._Templated],{

		templateString: ['<div dojoAttachPoint="mainContainer" class="com-ibm-common-timeline-iteration-selector">',
		                 '    <label dojoAttachPoint="treeLabel" style="position:absolute;top:-9999px;"></label>',
		                 '    <div dojoAttachPoint="treeContainer"></div>',		                 
		                 '</div>'].join(''),
		
		// CategoryTreePanel                 
		treePanel: null,
		
		postCreate: function() {
			dojo.mixin(this, CategoryTreeMessages);			
			this._createModel();			
			this._initPermissions();
		},
		
		// Summary:
		//    - Get permissions
		//    - Get contributor list
		//    - Creates the tree
		//
		_initPermissions: function() {
			var operations = [
				{
					id: PermissionsClient.OP_CATEGORY_TYPE,
					actions: PermissionsClient.DEFAULT_ACTIONS
				}, {
					id: PermissionsClient.OP_CATEGORY,
					actions: PermissionsClient.DEFAULT_ACTIONS
				}
			];
			var responseHandler = new ServiceResponseHandler(this, "_initPermissionsCallback", "_error");
	        PermissionsClient.checkAllPermissions(responseHandler, operations);
		},
		
		_initPermissionsCallback: function(permissions) {
			this._permissions = permissions;			
	        EditorUtil.getContributors({
	        	success: dojo.hitch(this, function(contributors) {
	        		this._contributors = contributors;
	        	}),
	        	error: dojo.hitch(this, "_error")
	        }, "success", "error");
	        
			//if this is being shown as a dialog, init the dialog
	        this._createTree();
		},
		
		_createModel: function(){
			if(!this.model){
				this.model = new com.ibm.rqm.planning.web.ui.internal.view.administration.categories.CategoryTreeModel({
					artifactType: this.artifactType,
					typeHierarchyOnly: false
				});
			}
		},
		_createTree: function() {
			if(this.tree){
				this.tree.destroyRecursive();
			}
			dojo.empty(this.treeContainer);
			
			var hitch_this = dojo.hitch(dojo, "hitch", this);
			this.treeLabel.innerHTML = getMessage("labelCategoryGroup");
			
			var args = {
					showHeader: false,
					state: "open",
					isRemote: false,
					openOnClick: false,
					model: this.model,					
					actionRegistry: new CategoryTreeActionRegistry({ parentView: this }),
					persist: false,
					treeId: this.getTreeKey(),
					expandNodeHidden: false,
					onLoad: hitch_this("onLoad"),
					onClick: hitch_this('onClick'),
					onNodeOpen: hitch_this('onClick'),
					onNodeClose: hitch_this('onClick'),
					itemCreator: hitch_this("_dndItemCreator"),
					checkItemAcceptance: hitch_this("checkItemAcceptance"),
					checkAcceptance: hitch_this("checkAcceptance")
				};
			
			if(!this.hierarchyOnly){
				args.dndController = "dijit.tree.dndSource";
			}
			this.tree = new TreeView(args);
			
			this.connect(this.tree, "onLoad", hitch_this(function() {
				this.connect(this.tree.tree.dndController, "onDndDrop", hitch_this("onDndDrop"));
				this.connect(this.tree.tree.dndController, "userSelect", hitch_this("onUserSelect"));
				this.connect(this.tree.tree, "onLoad", hitch_this("onTreeLoaded"));
				this.tree.tree.dndController.singular = true;
			}));

			this.connect(this.model, "pasteItem", hitch_this("onPasteItem"));
			
			dojo.style(this.tree.containerNode, "width", "98%");
			this.treeContainer.appendChild(this.tree.domNode);
			this.tree.loadContent();
		},				
		
		reloadTreeModel: function(model,notCreateTree/*Optional if true, we will not create tree*/){
			this.model = model;
			if(!notCreateTree){
				this._createTree();
			}
		},
		
		// Summary:
		//    Handle the pasteItem event from the model...
		//    Updates both parent nodes involved...
		//
		onPasteItem: function(item, from, to) {
			var tree = this.tree;
			if (tree.model.typeHierarchyOnly) {
				// Refresh root only in outline mode
				var model = tree.model;
				if (from == model.root) {
					tree.onItemChildrenChange(model.root, model.root.children || []);
				} else {
					tree.onItemChildrenChange(from, from.childrenHierarchyOnly || []);
				}
				tree.onItemChildrenChange(to, to.childrenHierarchyOnly || []);
			} else {
				//when user drops an item onmouseleave is not activated, so we will manually remove the "move" class here
				var leaf = tree.tree?tree.tree.getNodesByItem(to)[0]:null;
				if(leaf && leaf.rowNode){
					dojo.removeClass(leaf.rowNode, "onMoveTarget");				
				}				 
				tree.onItemChildrenChange(from, from.children || []);
				tree.onItemChildrenChange(to, to.children || []);
			}
			this.setDirty(true);
		},
		
		// Summary:
		//    Collapses the complete categories hierarchy
		//
		collapseAll: function() {
			this.tree.tree.collapseAll();
		},
		
		isDirty: function() {
			return this._dirty;
		},
		
		setDirty: function(dirty) {
			this._dirty = dirty;
			if (this._dialog) {
				this._dialog.getOkButton().disabled = !dirty;
			}
		},
		
		// Summary:
		//    Returns the selected item in the underlying ActionableTree
		//
		getSelectedItem: function() {
			var tree = this.tree.tree;
			return tree && tree.get("selectedItem");
		},
		
		// Summary:
		//    Handler for the user selection event in the tree.
		//    Used mainly to propagate the event and provide a connection
		//    point for clients.
		//
		onUserSelect: function() {
			// To be connected to.
		},
		
		// Summary:
		//    Handler for the drop event. It takes care of refreshing the
		//    parent node where the drop happened.
		//
		onDndDrop: function(source, nodes, copy, target, evt) {
			if (this.lastAcceptedTargetItem !== undefined) {
				var item = this.lastAcceptedTargetItem;
				if (this.tree.model.typeHierarchyOnly) {
					this.tree.onItemChildrenChange(item, item.childrenHierarchyOnly);
				} else {
					this.tree.onItemChildrenChange(item, item.children);
				}
				this.setDirty(true);
			}
		},
		
		// Summary:
		//    Item creator function for drag and drop functionality.
		//    This method is provided to the tree's dnd source in order
		//    for it to know how to create the items to be stored in 
		//    the model.
		//
		_dndItemCreator: function(nodes, target, source) {
			var dndListener = this.treePanel.dndListener;
			var dndItems = dndListener.getDndItems(source, nodes);
			return dojo.map(dndItems, function(item) {
				if (dojo.indexOf(item.type, CategoryConstants.DND_TYPE_CATEGORY_VALUE) !== -1) {
					return dojo.mixin(item.data, {
						isAssociationCandidate: true
					});
				} else {
					return item.data;
				}
			});
		},
		
		// Summary:
		//    Verifies if a source node can be dropped in (i.e. become a child of) a 
		//    given target node.
		//		
		checkItemAcceptance: function(node, source, position) {			
			if (!source) {
				return false;
			}
			// First extract the target node
			var target = dijit.getEnclosingWidget(node).item;
			if (!target || !target.type) {
				return false;
			}
			var model = this.model;
			
			var isTargetCategoryType = model.isCategoryType(target);
			if (!isTargetCategoryType) {
				// We only allow dropping over category types...
				return false;
			}
			// Figure out the types of the items being dropped...
			var dndListener = this.treePanel.dndListener;
			var dndItems = [];
			var types = [];
			source.forInSelectedItems(function(dndItem) {
				var type = dndListener.getDndType(dndItem);
				dndItems.push(dndItem);
				types.push(type);
			});
			
			// Iterate over the types... 
			// Note how we are not actually caring about every item, we just check the 
			// first one and get done with it, we might change that in the future, for 
			// now I just leave the loop ready for the posterity
			for (var i = 0, length = types.length; i < length; i++) {
				var type = types[i];
				var dndItem = dndItems[i];
				
				if (type === CategoryConstants.DND_TYPE_TREENODE) {
					// Internal TreeNode move...
					
					var treeNode = dijit.getEnclosingWidget(dndItem.data.domNode);
					var item = treeNode.item;
					
					var isItemCategoryType = model.isCategoryType(item);
					if (isItemCategoryType) {						
						if (!model.typeHierarchyOnly) {
							// We only allow moving category types in outline mode
							if(model.isCategoryTypeRoot(target) && model.isCategoryTypeRoot(dndItem.data.item)){
								dojo.addClass(node, "onMoveTarget");
								return true;
							}
							return false;
						} else {
							// Move Category Types - Outline mode							
							return false;
						}
					}
										
					if (target.isRoot) {
						// Moving values to another category type is allowed.
						this.lastAcceptedTargetItem = target;
						this.lastDroppingActionCandidate = CategoryConstants.DND_ACTION_CREATE_VALUE;
						return true;
					}
					
				} else {
					// External drop
					if (type === CategoryConstants.DND_TYPE_TERM ||
						type === CategoryConstants.DND_TYPE_CATEGORY_VALUE) {
						// Cache dropping target
						this.lastAcceptedTargetItem = target;
						// Find out candidate action...
						if (this.treePanel.valuesPanel.isAssociationsMode()) {
							// Associate only
							this.lastDroppingActionCandidate = CategoryConstants.DND_ACTION_ASSOCIATE;
						} else {
							// Item dragged is only a value, not a real thing, we 
							// need to create and associate
							this.lastDroppingActionCandidate = CategoryConstants.DND_ACTION_CREATE_AND_ASSOCIATE;
						}
						this.onCategoryTypeHover(type, dndItem.data, target);
						return true;
					}
				}
			}
			
			return false;
		},
		
		onCategoryTypeHover: function(draggedItemType, item, categoryType) {
			switch (draggedItemType) {
			case CategoryConstants.DND_TYPE_TERM:
			case CategoryConstants.DND_TYPE_CATEGORY_VALUE:
				this.treePanel.setMessage(getMessage("labelDropValueToCreateContextual", item.value, categoryType.name));
				return;
			}
		},
		
		checkAcceptance: function() {
			return true;
		},
		
		onLoad: function(onComplete) {
			onComplete(this.model);
		},
		
		// Summary:
		//    Notification method to react when the tree has been completely
		//    loaded. Additionally, it sets the selection to nothing.
		//
		onTreeLoaded: function() {
			dojo.setAttr(this.treeLabel, "for", this.tree.tree.id);			
			this.tree.tree.dndController.setSelection([]);				
		},
		
		refreshItemsChildren: function(items){
			var len = items ? items.length : 0;
			for (var i = 0; i < len; i++){
				var item = items[i];
				var children = (!this.hierarchyOnly || item.type == NodeTypes.ROOT_TYPE)? item.children : item.childrenHierarchyOnly;
				if(children){
					this.tree.onItemChildrenChange(item, children);
				}
			}
		},
		
		refreshItemChildren: function(item){
			this.tree.onItemChildrenChange(item, !this.hierarchyOnly ? item.children : item.childrenHierarchyOnly);
		},

		refreshItems: function(items) {
			dojo.forEach(items || [], this.refreshItem, this);
		},
		
		onClick: function(item) {
			// To be overriden by consumers
		},
		
		refreshItem: function(item){
			this.tree.onItemChange(item);
		},		
		
		
		// Summary:
		//    Switches between outline and full mode...
		//    Note: Calling this method will recreate the tree.
		//
		switchTreeMode: function(newMode) {
			this.tree.destroyRecursive();
			if (newMode !== undefined) {
				this.hierarchyOnly = newMode === CategoryConstants.VIEW_MODE_OUTLINE;
			} else {
				// Just switch between modes
				this.hierarchyOnly = !this.hierarchyOnly;
			}
			this.model.setHierarchyOnlyMode(this.hierarchyOnly);
			
			this._createTree();			
		},
		
		
		// Summary:
		//    Creates the id for the tree, which in turn is the key to the cookie
		//    used to persistent the underlying dijit.Tree's state.
		//
		getTreeKey: function() {
			var pre = this.isDialog ? this.id + "-" : "";
			return pre + "CATEGORIES-TREE-" + this.artifactType + "-" + (this.hierarchyOnly ? "OUTLINE" : "COMPLETE");
		},
		
		
		_onDialogClose: function() {
	        return true;
	    },
	    
		// Create 
		showCreateDialog: function(isCreateSubCategory, parentItem, item, treeNode) {
			this.treePanel.expandNode(treeNode).then(dojo.hitch(this, function() {
				var model = this.model;
				
				if(isCreateSubCategory && item.type == NodeTypes.CATEGORY_VALUE_TYPE){
					item = item.ancestorType;
				}
				var categoryDialog = new CategoryDialog({
					onOk: dojo.hitch(this, function(values, isCategoryType, createAsRoot) {
						this.treePanel._handleCreateItems(values, isCategoryType, createAsRoot, item);
					}),
					parent: item,
					isCreateSubCategory: isCreateSubCategory,
					model: model
				});
			categoryDialog.show();
			}));
		},
		
		// Edit Action
		showEditCategoryItemDialog: function(parentItem, item, treeNode) {
			var type = item.type;
			var args = {
				item: item,
				okCallback: dojo.hitch(this, "_handleEditItem"),
	            onClose: dojo.hitch(this, "_onDialogClose")
			};
			if (type == NodeTypes.CATEGORY_TYPE) {
				args.categoryTypes =  this.model.getAllCategoryTypes();
				args.categoryTypesUnchanged =  this.model.getAllCategoryTypesUnchanged();
			} else {
				var name = item.categoryType.label ? item.categoryType.label : item.categoryType.name;
				args.categoryValues = this.model.getAllCategoryValuesForType(item.categoryType.itemId || item.categoryType.name);
				args.categoryTypeName = name;
			}
			var dialog = new CategoryDialog.EditCategoryItemDialog(args);
			dialog.show();
		},
		
		_handleEditItem: function(item) {
			var changedItems = this.model.updateItem(item);
			if(changedItems){
				this.refreshItems(changedItems);
			} else {
				this.refreshItem(item);
			}
			this.setDirty(true);
		},
		
		// Associate Type Action
		showAssociateDialog: function(parentItem, item, treeNode) {
			this.treePanel.expandNode(treeNode).then(dojo.hitch(this, function() {
				var args = {
					categoryTypes: this.model.getAllCategoryTypes(),
					categoryType: item,
					okCallback: dojo.hitch(this, "_handleAssociateType"),
					checkParent: dojo.hitch(this,"hasParent"),
		            onClose: dojo.hitch(this, "_onDialogClose")
				};
				var dialog = new CategoryDialog.AssociateCategoryTypeDialog(args);
				dialog.show();
			}));
		},
		
		_handleAssociateType: function(parentType, existingTypeItemId) {
			var changedItems = this.model.associateExistingTypeInParentType(parentType, this.model.categoryBase[existingTypeItemId]);
			this.refreshItemsChildren(changedItems);
			this.setDirty(true);
		},
		
		hasParent: function(itemId) {
			var type =  this.model.categoryBase[itemId];
			return type.dependsOn;
		},
		
		// Manage value associations action
		showEditAssociatedValuesDialog: function(inChild, parentItem, item, treeNode) {
			this.treePanel.expandNode(treeNode).then(dojo.hitch(this, function() {
				var subCategoryType = this.model.categoryBase[item.itemId || item.name];
				var parentValue = item.ancestorValue;
				var args = {
					parentCategory:parentValue,
					model: this.model,
					valueSet: (item && item.children) ? item.children : [],
					subCategoryType: subCategoryType,
					okCallback: dojo.hitch(this, "_handleAssociatedValues", parentItem, inChild, item),
		            onClose: dojo.hitch(this, "_onDialogClose")
				};		
				var dialog = new CategoryDialog.EditAssociatedValuesDialog(args);
				dialog.show();
			}));
		},
		
		_handleAssociatedValues: function(parentItem, inChild, item, values) {
			var addedValues = values.addedValues;
			var removedValues = values.removedValues;
			var subCategoryType = this.model.categoryBase[item.itemId || item.name];
			// Add values to the value set belonging to this ancestor value
			var itemsToRefresh = [];
			for (var itemId in addedValues) {
				itemsToRefresh = itemsToRefresh.concat(this.model.associateExistingValueInParentSubType(item,subCategoryType.allValues[itemId]));
			}
			// Remove values from this ancestor value
			var len = removedValues.length;
			for (var i = 0; i < len; i ++) {
				 itemsToRefresh = itemsToRefresh.concat(this.model.removeItem(values.removedValues[i]));
			}
			// Refresh parent
			this.refreshItemsChildren(itemsToRefresh);
			
			this.setDirty(true);
		},
		
		// Required Action
		_handleRequiredAction: function(required, parentItem, item, treeNode) {
			item.isRequired = item.required = required;
			var changedItems = this.model.updateItem(item);
			var catType = this.model.categoryBase[item.itemId || item.name];
            
            // set a flag to do validation on setting the type
            // to required.
			if (catType.dependsOn && required) {
				this.treePanel.setWarningMessage(this.msgNewRequiredSubcategory);
			}
			this.setDirty(true);
			if(changedItems){
				this.refreshItems(changedItems);
			} else {
				this.refreshItem(item);
			}			
		},
		
		// Remove Association Action
		removeAssociatedType: function(parentItem, item, treeNode) {
			this.treePanel.expandNode(treeNode).then(dojo.hitch(this, function() {
				var catType = this.model.categoryBase[item.itemId || item.name];
				var childTypes = catType.childTypes;		
				var len = childTypes ? childTypes.length : 0;
				if (len > 1) {
					var dialog =  new CategoryDialog.RemoveSubCategoryDialog({
						childTypes: childTypes,
						labelTitle: this.labelRemoveSubtype,
						okCallback: dojo.hitch(this, function(values) {														
							this._handleRemoveAssociatedType(item, values);
							this.refreshItems(item);
							this.setDirty(true);
						}),
						width: "450px"
					});
					dialog.show();
				} else {			
					// Need to send a copy since we are using the childs directly 
					// when we remove from the model we lose the original value too
					this._handleRemoveAssociatedType(item, [childTypes[0]]);
					this.refreshItems(item);
					this.setDirty(true);
				}
			}));
		},
		
		_handleRemoveAssociatedType: function(item, childs) {
			var len = childs.length;
			var itemsToRefresh = [];
			for (var i = 0; i < len; i++) {
				var child = childs[i];
				var identifier = (typeof child === 'object') ? (child.itemId || child.name) : child;
				var subCat = this.model.categoryBase[identifier];
				itemsToRefresh = itemsToRefresh.concat(this.model.removeItem(subCat, item));
			}
			this.refreshItemsChildren(itemsToRefresh);
			this.setDirty(true);			
		},
		
		removeAssociatedValue: function(parentItem, item, treeNode) {
			this.refreshItemsChildren(this.model.removeItem(item));
			this.setDirty(true);
		},
		
		// Multiselectable Action
		_handleMultiSelectableAction: function(multiSelectable, parentItem, item, treeNode) {
			item.isMultiValued = item.multiSelectable = multiSelectable;
			var iconClass = multiSelectable ? "set-multi-value-icon" : "";
			item.stateIconClass = iconClass;
			var changedItems =  this.model.updateItem(item);
			if(changedItems){
				this.refreshItems(changedItems);
			} else {
				this.refreshItem(item);
			}
			this.setDirty(true);
		},
		
		_handleRestore: function(parentItem, item){
			// No restore for now
		},
	    
		// Delete Action
		_handleDeleteAction: function(parentItem, item, treeNode) {
			var categoryType = this.model.getCategoryTypeFromBase(item.itemId || item.name);
			var itemLabel = item.label ? item.label :item.name;
			itemLabel = Utility.encodeHTMLTags(itemLabel);
			if (categoryType) {
				var childTypes = categoryType.childTypes || [];
				for (var idx = 0, length = childTypes.length; idx < length; idx++) {
					var childType = childTypes[idx];
					// Stop and display an error message once we have found
					// one child that is not archived.
					if (!childType.archived) {
						new AlertDialog({
							primaryTitle: this.labelError,
							message: getMessage("msgCannotDeleteType", itemLabel),
							width: "450px"
						}).open();
						return;
					}
				}
			}
			
			var args = {
				primaryTitle: this.labelConfirmation,
	            message: item.type == NodeTypes.CATEGORY_TYPE 
	            	? getMessage("msgArchiveCategoryType", BidiUtil.enforceTextDirWithUcc(itemLabel))
	            	: getMessage("msgArchiveCategory", BidiUtil.enforceTextDirWithUcc(itemLabel)),
	            yesCallback: dojo.hitch(this, "_handleRemove", parentItem, item),
	            onClose: dojo.hitch(this, "_onDialogClose"),
	            width: "450px"
	        };
	        var confirmDialog = new YesNoDialog(args);
	        confirmDialog.open();
		},
		
		_handleRemove: function(parent, item) {
		    item.toBeDeleted = true;
		    this.refreshItemsChildren(this.model.removeItem(item));
		    this.setDirty(true);
		},
	    
		// Set Default Action
		_handleDefaultAction: function(isDefault, parentItem, item, treeNode) {
		    this.treePanel.expandNode(treeNode).then(dojo.hitch(this, function() {
				var treeNode = this.tree.tree.getNodesByItem(item);
				if (treeNode && treeNode.length) {
				    treeNode[0].set("stateIcon",  (isDefault ? "set-default-category-icon" : ""));
				}
				item.stateIconClass = isDefault ? "set-default-category-icon" : null;
				item.isDefault = isDefault;
				if (!isDefault) {
				    item.removeDefault = true;
				}
				var changedItems  = this.model.updateItem(item);
				if(changedItems){
					this.refreshItems(changedItems);
				} else {
					this.refreshItem(item);
				}
				this.setDirty(true);
			}));
		},
		
		_moveCategory: function(parentItem, item, treeNode){
			this.treePanel.expandNode(treeNode).then(dojo.hitch(this, function() {
				var args = {
					categoryTypes: this.model.getAllCategoryTypes(),
					categoryType: item,
					okCallback: dojo.hitch(this, "_handleMoveCategory", item),					
					onClose: dojo.hitch(this, "_onDialogClose")
				};
				var dialog = new CategoryDialog.MoveCategoryTypeDialog(args);
				dialog.show();
			}));
		},
		
		_handleMoveUpCategory: function(parentItem, item, treeNode){
			this.model.moveUpItemInRoot(item);
			this.refreshItemsChildren([this.model.root]);
			this.setDirty(true);
		},
		
		_handleMoveDownCategory: function(parentItem, item, treeNode){			
			this.model.moveDownItemInRoot(item);
			this.refreshItemsChildren([this.model.root]);
			this.setDirty(true);
		},
		
		_handleMoveCategory: function(sourceCategoryTypeItemElement, destinationCategoryTypeItemId){
			var changedItems = [];
			var parentItem = this.model.searchRepresentativeItemInHierarchy(destinationCategoryTypeItemId);
		
			if(!parentItem){
				//no representative item found, try using a copy of category base
				parentItem = dojo.clone(this.model.categoryBase[destinationCategoryTypeItemId]);
			}
			changedItems = changedItems.concat(this.model.associateExistingTypeInParentType(parentItem, this.model.categoryBase[sourceCategoryTypeItemElement.itemId || sourceCategoryTypeItemElement.name]));	
			if(changedItems){
				this.refreshItemsChildren(changedItems);
			} 
			this.setDirty(true);
		},
		
		_addExistingValue: function(parentItem, item, treeNode){
			this.treePanel.expandNode(treeNode).then(dojo.hitch(this, function() {
				var values = this.model.getAllCategoryValuesForType(item.itemId || item.name);
				var handler = {
						success: dojo.hitch(this, function(allValues) {
							allValues = allValues || [];	
							var args = {
									allValues: allValues,
									selectedCategoryValues: values,
									okCallback: dojo.hitch(this, "_handleAddExistingValue", item)
							};
							var dialog = new CategoryDialog.AddCategoryValuesDialog(args);
							dialog.show();
						}),
						error: function(error) {
							//silently fail?
						}
				};					
				var args = {
						searchTerm: "%"
				};
				var serviceResponseHandler = new ServiceResponseHandler(handler, "success", "error");
				CategoryClient.getCategoryTerms(serviceResponseHandler, args);

			}));
		},
		
		_handleAddExistingValue: function(item, selectedValues){
			var len = selectedValues?selectedValues.length:0;
			var changedItems = [];
			var categoryValueLabel = Constants.getMessage("labelCategoryValueLowercase");
			var categoryValuesLabel = Constants.getMessage("labelCategoryValues");
			for(var i=0;i<len;i++){
				var value = selectedValues[i];
				changedItems = changedItems.concat(this.model.newItem(value, item, false));				
				this.treePanel.setMessage(dojo.string.substitute(Constants.getMessage("labelCreatingTemplate"), [i, len, categoryValueLabel]));
			}			
			if(len > 0){
				this.refreshItemsChildren(changedItems);				
				this.setDirty(true);
				this.treePanel.setMessage(dojo.string.substitute(Constants.getMessage("labelCreatedTemplate"), [len, categoryValuesLabel]));
			}
		},
		
		hasPermission: function(operationId, actionId) {
	        var permission = this._permissions[operationId];
	        return permission && permission[actionId] && permission[actionId] == true;         
	    },		
		
		hasCategoryTypePermission: function(actionId) {
			return this.hasPermission(PermissionsClient.OP_CATEGORY_TYPE, actionId);
		},
		
		hasCategoryPermission: function(actionId) {
	        return this.hasPermission(PermissionsClient.OP_CATEGORY, actionId);
	    },
	  
		_error: function(e) {
			console.error(this.declaredClass + ": Error: " + e.message);
	    },
	    
	    // Summary:
	    //    Convenience method to turn the calls to dojo.stopEvent 
	    //    when the event argument is set, into one-liners.
	    //
	    stopEvent: function(event) {
	    	if (event) {
				dojo.stopEvent(event);
			}
	    }	    
	  
	});
	
})(dojo);
